@@ -63,8 +63,10 @@ extern void dkeep(tree t);
 # endif
 #elif D_GCC_VER < 41
 #include "d-bi-attrs-40.h"
-#else
+#elif D_GCC_VER < 43
 #include "d-bi-attrs-41.h"
+#else
+#include "d-bi-attrs-43.h"
 #endif
 
 #if D_GCC_VER >= 41
@@ -161,7 +163,7 @@ lookup_C_type_name(const char * p)
 #endif
 
 #if D_GCC_VER >= 41
-tree do_build_builtin_fn(enum built_in_function fncode,
+void do_build_builtin_fn(enum built_in_function fncode,
     const char *name,
     enum built_in_class fnclass,
     int fntype_index, tree fntype, int libtype_index,
@@ -178,12 +180,17 @@ tree do_build_builtin_fn(enum built_in_function fncode,
     libname = name + strlen ("__builtin_"); 
 
 
-    /*if (!BOTH_P)*/							
+    /*if (!BOTH_P)*/
+#if D_GCC_VER >= 43
+    decl = add_builtin_function(name, fntype, fncode, fnclass,
+	fallback_p ?libname : NULL, fnattrs);
+#else
     decl = lang_hooks.builtin_function (name, fntype,	
 	fncode,					
 	fnclass,					
 	fallback_p ?libname : NULL,				
-	fnattrs);	
+	fnattrs);
+#endif
 
     built_in_decls[(int) fncode] = decl;				
     if (implicit_p)							
@@ -203,6 +210,7 @@ void d_init_builtins(void)
 #define DEF_FUNCTION_TYPE_4(NAME, RETURN, ARG1, ARG2, ARG3, ARG4) NAME,
 #define DEF_FUNCTION_TYPE_5(NAME, RETURN, ARG1, ARG2, ARG3, ARG4, ARG5) NAME,
 #define DEF_FUNCTION_TYPE_6(NAME, RETURN, ARG1, ARG2, ARG3, ARG4, ARG5, ARG6) NAME,
+#define DEF_FUNCTION_TYPE_7(NAME, RETURN, ARG1, ARG2, ARG3, ARG4, ARG5, ARG6, ARG7) NAME,
 #define DEF_FUNCTION_TYPE_VAR_0(NAME, RETURN) NAME,
 #define DEF_FUNCTION_TYPE_VAR_1(NAME, RETURN, ARG1) NAME,
 #define DEF_FUNCTION_TYPE_VAR_2(NAME, RETURN, ARG1, ARG2) NAME,
@@ -219,6 +227,7 @@ void d_init_builtins(void)
 #undef DEF_FUNCTION_TYPE_4
 #undef DEF_FUNCTION_TYPE_5
 #undef DEF_FUNCTION_TYPE_6
+#undef DEF_FUNCTION_TYPE_7
 #undef DEF_FUNCTION_TYPE_VAR_0
 #undef DEF_FUNCTION_TYPE_VAR_1
 #undef DEF_FUNCTION_TYPE_VAR_2
@@ -254,7 +263,7 @@ void d_init_builtins(void)
   
   intmax_type_node = intDI_type_node;
   uintmax_type_node = unsigned_intDI_type_node;
-  signed_size_type_node = (* lang_hooks.types.signed_type)(size_type_node);
+  signed_size_type_node = d_signed_type(size_type_node);
   string_type_node = build_pointer_type (char_type_node);
   const_string_type_node = build_pointer_type (build_qualified_type
       (char_type_node, TYPE_QUAL_CONST));
@@ -350,6 +359,27 @@ void d_init_builtins(void)
 					 tree_cons (NULL_TREE,		\
 					      builtin_types[(int) ARG6],\
 					      void_list_node)))))));
+#define DEF_FUNCTION_TYPE_7(ENUM, RETURN, ARG1, ARG2, ARG3, ARG4, ARG5, \
+			    ARG6, ARG7)					\
+  builtin_types[(int) ENUM]						\
+    = build_function_type						\
+      (builtin_types[(int) RETURN],					\
+       tree_cons (NULL_TREE,						\
+		  builtin_types[(int) ARG1],				\
+		  tree_cons (NULL_TREE,					\
+			     builtin_types[(int) ARG2],			\
+			     tree_cons					\
+			     (NULL_TREE,				\
+			      builtin_types[(int) ARG3],		\
+			      tree_cons					\
+			      (NULL_TREE,				\
+			       builtin_types[(int) ARG4],		\
+			       tree_cons (NULL_TREE,			\
+					 builtin_types[(int) ARG5],	\
+					 tree_cons (NULL_TREE,		\
+					      builtin_types[(int) ARG6],\
+					      tree_cons(NULL_TREE,builtin_types[(int) ARG7], \
+					      void_list_node))))))));
 #define DEF_FUNCTION_TYPE_VAR_0(ENUM, RETURN)				\
   builtin_types[(int) ENUM]						\
     = build_function_type (builtin_types[(int) RETURN], NULL_TREE);
@@ -537,11 +567,24 @@ void d_init_builtins(void)
    the name to be called if we can't opencode the function.  If
    ATTRS is nonzero, use that for the function's attribute list.  */
 
+#if D_GCC_VER >= 43
+
 tree
-builtin_function (const char *name, tree type, int function_code,
-		  enum built_in_class klass, const char *library_name,
-		  tree attrs)
+d_builtin_function43 (tree decl)
 {
+  d_bi_builtin_func(decl);
+
+  return decl;
+}
+
+#else
+
+tree
+d_builtin_function (const char *name, tree type, int function_code,
+		    enum built_in_class klass, const char *library_name,
+		    tree attrs)
+{
+  // As of 4.3.x, this is done by add_builtin_fucntion 
     //%% for D, just use library_name?
   tree decl = build_decl (FUNCTION_DECL, get_identifier (name), type);
   DECL_EXTERNAL (decl) = 1;
@@ -576,4 +619,6 @@ builtin_function (const char *name, tree type, int function_code,
   return decl;
 }
 
+#endif
+
 #include "gt-d-d-builtins.h"
