@@ -221,7 +221,6 @@ IRState::var(VarDeclaration * v)
     if (vp)
     {
 	FuncDeclaration * f = vp->isFuncDeclaration();
-	/* //testing...
 	if (f && getFrameInfo(f)->creates_closure)
 	{
 	    for (unsigned i = 0; i < f->closureVars.dim; ++i)
@@ -300,10 +299,10 @@ IRState::convertTo(tree exp, Type * exp_type, Type * target_type)
 		} else if (offset) {
 		    tree t = target_type->toCtype();
 		    exp = maybeMakeTemp(exp);
-		    return build(COND_EXPR, t,
+		    return build3(COND_EXPR, t,
 			boolOp(NE_EXPR, exp, nullPointer()),
-			build(PLUS_EXPR, t, exp, size_int(offset)),
-			nullPointer());
+			nop(pointerOffset(exp, size_int(offset)), t),
+			nop(nullPointer(), t));
 		} else {
 		    // d_convert will make a NOP cast
 		    break;
@@ -464,7 +463,7 @@ IRState::convertTo(tree exp, Type * exp_type, Type * target_type)
 		// default conversion
 		break;
 	    }
-	    result = build(COMPLEX_EXPR, target_type->toCtype(), c1, c2);
+	    result = build2(COMPLEX_EXPR, target_type->toCtype(), c1, c2);
 	} else {
 	    assert( TREE_CODE( exp ) != STRING_CST );
 	    // default conversion
@@ -542,7 +541,7 @@ IRState::convertForAssignment(Expression * exp, Type * target_type)
 	TREE_TYPE( ctor ) = target_type->toCtype();
 	if (count) {
 	    CtorEltMaker ce;
-	    ce.cons( build(RANGE_EXPR, Type::tsize_t->toCtype(),
+	    ce.cons( build2(RANGE_EXPR, Type::tsize_t->toCtype(),
 		    integer_zero_node, integerConstant( count - 1 )),
 		g.ofile->stripVarDecl(convertForAssignment(exp, sa_type->next)));
 	    CONSTRUCTOR_ELTS( ctor ) = ce.head;
@@ -607,12 +606,12 @@ IRState::convertForCondition(tree exp_tree, Type * exp_type) {
 	a = delegateObjectRef(tmp);
 	b = delegateMethodRef(tmp);
 	if (TYPE_MODE(TREE_TYPE(a)) == TYPE_MODE(TREE_TYPE(b)))
-	    result = build(BIT_IOR_EXPR, TREE_TYPE(a), a, b);
+	    result = build2(BIT_IOR_EXPR, TREE_TYPE(a), a, b);
 	else {
 	    a = d_truthvalue_conversion(a);
 	    b = d_truthvalue_conversion(b);
 	    // probably not worth TRUTH_OROR ...
-	    result = build(TRUTH_OR_EXPR, TREE_TYPE(a), a, b);
+	    result = build2(TRUTH_OR_EXPR, TREE_TYPE(a), a, b);
 	}
 	break;
     case Tdelegate:
@@ -627,7 +626,7 @@ IRState::convertForCondition(tree exp_tree, Type * exp_type) {
 	}
 	// not worth using  or TRUTH_ORIF...
 	// %%TODO: Is this okay for all targets?
-	result = build(BIT_IOR_EXPR, TREE_TYPE(a), a, b);
+	result = build2(BIT_IOR_EXPR, TREE_TYPE(a), a, b);
 	break;
     default:
 	break;
@@ -662,36 +661,91 @@ IRState::pointerIntSum(tree ptr_node, tree idx_exp)
     tree result_type_node = TREE_TYPE( ptr_node );
     tree intop = idx_exp;
     tree size_exp;
+    
+    tree prod_result_type;
+#if D_GCC_VER >= 43
+    prod_result_type = sizetype;
+#else
+    prod_result_type = result_type_node;
+#endif
 
     // %% TODO: real-not-long-double issues...
 
-    // %% test for void case ...
     size_exp = size_in_bytes( TREE_TYPE( result_type_node ) ); // array element size
-    if (integer_zerop(size_exp))
-	size_exp = integer_one_node;
-
-    if (TYPE_PRECISION (TREE_TYPE (intop)) != TYPE_PRECISION (sizetype)
-	|| TREE_UNSIGNED (TREE_TYPE (intop)) != TREE_UNSIGNED (sizetype))
-	intop = convert (d_type_for_size (TYPE_PRECISION (sizetype), 
-			     TREE_UNSIGNED (sizetype)), intop);
-    
-    intop = convert (result_type_node,
-	build/*_binary_op*/ (MULT_EXPR, TREE_TYPE(size_exp), intop,  // the type here may be wrong %%
-	    convert (TREE_TYPE (intop), size_exp)));
-    intop = fold(intop);
+    if (integer_zerop(size_exp) || // Test for void case...
+	integer_onep(size_exp))    // ...or byte case -- No need to multiply.
+    {
+	intop = fold(convert(prod_result_type, intop));
+    }
+    else
+    {
+	
+	if (TYPE_PRECISION (TREE_TYPE (intop)) != TYPE_PRECISION (sizetype)
+	    || TREE_UNSIGNED (TREE_TYPE (intop)) != TREE_UNSIGNED (sizetype))
+	    intop = convert (d_type_for_size (TYPE_PRECISION (sizetype), 
+				 TREE_UNSIGNED (sizetype)), intop);
+	intop = convert (prod_result_type,
+	    build2/*_binary_op*/ (MULT_EXPR, TREE_TYPE(size_exp), intop,  // the type here may be wrong %%
+		convert (TREE_TYPE (intop), size_exp)));
+	intop = fold(intop);
+    }    
 
     if ( integer_zerop(intop) )
 	return ptr_node;
     else
-	return build(PLUS_EXPR, result_type_node, ptr_node, intop);
+    {
+#if D_GCC_VER >= 43
+	return build2(POINTER_PLUS_EXPR, result_type_node, ptr_node, intop);
+#else
+	return build2(PLUS_EXPR, result_type_node, ptr_node, intop);
+#endif	
+    }
 }
 
+// %% op should be tree_code
+tree 
+IRState::pointerOffsetOp(int op, tree ptr, tree idx)
+{
+#if D_GCC_VER >= 43    
+    if (op == PLUS_EXPR)
+	{ /* nothing */ }
+    else if (op == MINUS_EXPR)
+	// %% sign extension...
+	idx = fold_build1(NEGATE_EXPR, sizetype, idx);
+    else
+    {
+	gcc_unreachable();
+	return error_mark_node;
+    }
+    return build2(POINTER_PLUS_EXPR, TREE_TYPE(ptr), ptr, 
+	convert(sizetype, idx));
+#elif D_GCC_VER >= 40
+    return build2((enum tree_code) op, TREE_TYPE(ptr), ptr, idx);
+#else
+    return build2(op, TREE_TYPE(ptr), ptr, idx);
+#endif    
+}
+
+tree 
+IRState::pointerOffset(tree ptr_node, tree byte_offset)
+{
+    tree ofs = fold(convert(sizetype, byte_offset));
+    tree t;
+#if D_GCC_VER >= 43
+    t = build2(POINTER_PLUS_EXPR, TREE_TYPE(ptr_node), ptr_node, ofs);
+#else
+    t = build2(PLUS_EXPR, TREE_TYPE(ptr_node), ptr_node, ofs);
+#endif
+    return fold(t);
+}
+
+
 // Doesn't do some of the omptimizations in ::makeArrayElemRef
 tree
 IRState::checkedIndex(Loc loc, tree index, tree upper_bound, bool inclusive)
 {
     if (global.params.useArrayBounds) {
-	return build(COND_EXPR, TREE_TYPE(index),
+	return build3(COND_EXPR, TREE_TYPE(index),
 	    boundsCond(index, upper_bound, inclusive),
 	    index,
 	    assertCall(loc, LIBCALL_ARRAY_BOUNDS));
@@ -707,14 +761,14 @@ IRState::boundsCond(tree index, tree upper_bound, bool inclusive)
 {
     tree bound_check;
 
-    bound_check = build(inclusive ? LE_EXPR : LT_EXPR, boolean_type_node,
+    bound_check = build2(inclusive ? LE_EXPR : LT_EXPR, boolean_type_node,
 	convert( d_unsigned_type(TREE_TYPE(index)), index ),
 	upper_bound);
     
     if (! TREE_UNSIGNED( TREE_TYPE( index ))) {
-	bound_check = build(TRUTH_ANDIF_EXPR, boolean_type_node, bound_check,
+	bound_check = build2(TRUTH_ANDIF_EXPR, boolean_type_node, bound_check,
 	    // %% conversions
-	    build(GE_EXPR, boolean_type_node, index, integer_zero_node));
+	    build2(GE_EXPR, boolean_type_node, index, integer_zero_node));
     }
 
     return bound_check;
@@ -765,20 +819,74 @@ IRState::hwi2toli(HOST_WIDE_INT low, HOST_WIDE_INT high)
 
 
 #if D_GCC_VER >= 40
+int xxx;
 tree
 IRState::binding(tree var_chain, tree body)
 {
     // BIND_EXPR/DECL_INITIAL not supported in 4.0?
     gcc_assert(TREE_CHAIN(var_chain) == NULL_TREE); // TODO: only handles one var
 
+#if D_GCC_VER >= 43
+    // %%EXPER -- if a BIND_EXPR is an a SAVE_EXPR, gimplify dies
+    // in mostly_copy_tree_r.  prevent the latter from seeing
+    // our shameful BIND_EXPR by wrapping it in a TARGET_EXPR
+    
+    /*
+    
+    tree bt = build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
+    tree tt = build_decl(VAR_DECL, NULL_TREE, TREE_TYPE(bt));
+    DECL_CONTEXT( tt ) = DECL_CONTEXT(var_chain); // cheat -> exprVar
+    DECL_ARTIFICIAL( tt ) = 1;
+    DECL_IGNORED_P( tt ) = 1;    
+    
+    return build4(TARGET_EXPR, TREE_TYPE(tt), tt, bt, NULL_TREE, NULL_TREE);
+    
+    */
+    /*
+    tree ini = DECL_INITIAL(var_chain);
+    DECL_INITIAL(var_chain) = NULL_TREE;
+    
+    return compound( build4(TARGET_EXPR, void_type_node, var_chain, ini,
+	NULL_TREE, NULL_TREE), body );
+    */
+    /*
+    tree ini = DECL_INITIAL(var_chain);
+    if (ini)
+    {
+	DECL_INITIAL(var_chain) = NULL_TREE;
+	
+	return compound( build4(TARGET_EXPR, void_type_node, var_chain, ini,
+	    NULL_TREE, NULL_TREE), body );
+    }
+    else
+    {
+	 ++xxx;
+	 
+	tree bt = build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
+	tree tt = build_decl(VAR_DECL, NULL_TREE, TREE_TYPE(bt));
+	DECL_CONTEXT( tt ) = DECL_CONTEXT(var_chain); // cheat -> exprVar
+	DECL_ARTIFICIAL( tt ) = 1;
+	DECL_IGNORED_P( tt ) = 1;    
+
+	return build4(TARGET_EXPR, TREE_TYPE(tt), tt, bt, NULL_TREE, NULL_TREE);
+    }*/
     if ( DECL_INITIAL(var_chain) )
     {
 	tree ini = build2(MODIFY_EXPR, void_type_node, var_chain, DECL_INITIAL(var_chain));
 	DECL_INITIAL(var_chain) = NULL_TREE;
 	body = compound(ini, body);
     }
-	
+    return save_expr(build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE));
+#else
+    if ( DECL_INITIAL(var_chain) )
+    {
+	tree ini = build2(MODIFY_EXPR, void_type_node, var_chain, DECL_INITIAL(var_chain));
+	DECL_INITIAL(var_chain) = NULL_TREE;
+	body = compound(ini, body);
+    }
+
     return build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
+#endif
 }
 #endif
 
@@ -1320,8 +1428,8 @@ fix_d_va_list_type(tree val)
 tree IRState::maybeExpandSpecialCall(tree call_exp)
 {
     // More code duplication from C
-
-    tree callee = TREE_OPERAND(call_exp, 0);
+    CallExpr ce(call_exp);
+    tree callee = ce.callee();
     tree t, op;
     if (POINTER_TYPE_P(TREE_TYPE( callee ))) {
 	callee = TREE_OPERAND(callee, 0);
@@ -1344,7 +1452,7 @@ tree IRState::maybeExpandSpecialCall(tree call_exp)
 	    case BUILT_IN_FABSL:
 	    case BUILT_IN_FABSF:
 		*/
-		op = TREE_VALUE( TREE_OPERAND( call_exp, 1 ));
+		op = ce.nextArg();
 		t = build1(ABS_EXPR, TREE_TYPE(op), op);
 		return d_convert_basic(TREE_TYPE(call_exp), t);
 		// probably need a few more cases:
@@ -1360,7 +1468,7 @@ tree IRState::maybeExpandSpecialCall(tree call_exp)
 		// %% should_check c_promotes_to as in va_arg now
 		// just drop though for now...
 	    case INTRINSIC_STD_VA_ARG:
-		t = TREE_VALUE(TREE_OPERAND(call_exp, 1));
+		t = ce.nextArg();
 		// signature is (inout va_list), but VA_ARG_EXPR expects the
 		// list itself... but not if the va_list type is an array.  In that
 		// case, it should be a pointer
@@ -1417,7 +1525,7 @@ tree IRState::maybeExpandSpecialCall(tree call_exp)
 		// address taken.  The second argument, however, is
 		// inout and that needs to be fixed to prevent a warning.
 		{
-		    tree val_arg = TREE_VALUE(TREE_OPERAND(call_exp, 1));
+		    tree val_arg = ce.nextArg();
 		    // kinda wrong... could be casting.. so need to check type too?
 		    while ( TREE_CODE( val_arg ) == NOP_EXPR )
 			val_arg = TREE_OPERAND(val_arg, 0);
@@ -1428,7 +1536,7 @@ tree IRState::maybeExpandSpecialCall(tree call_exp)
 		    } else
 			val_arg = fix_d_va_list_type(val_arg);
 
-		    t = TREE_VALUE(TREE_CHAIN(TREE_OPERAND(call_exp, 1)));
+		    t = ce.nextArg();
 		    if ( TREE_CODE( t ) == ADDR_EXPR ) {
 			t = TREE_OPERAND(t, 0);
 		    }		
@@ -1460,7 +1568,7 @@ tree IRState::maybeExpandSpecialCall(tree call_exp)
 #if D_GCC_VER >= 34
 		return buildCall(TREE_TYPE(call_exp),
 		    built_in_decls[intrinsic == INTRINSIC_BSF ? BUILT_IN_CTZ : BUILT_IN_CLZ],
-		    TREE_OPERAND(call_exp, 1));
+		    tree_cons(NULL_TREE, ce.nextArg(), NULL_TREE));
 #else
 		return call_exp;
 #endif
@@ -1551,7 +1659,7 @@ IRState::arrayElemRef(IndexExp * aer_exp, ArrayScope * aryscp)
 		oob_cond = boundsCond(index_expr, array_len_expr, false);
 		throw_expr = assertCall(aer_exp->loc, LIBCALL_ARRAY_BOUNDS);
 
-		subscript_expr = build( COND_EXPR, TREE_TYPE( index_expr ),
+		subscript_expr = build3( COND_EXPR, TREE_TYPE( index_expr ),
 		    oob_cond, index_expr, throw_expr );
 	    }
 
@@ -1587,7 +1695,7 @@ IRState::darrayPtrRef(tree exp)
     // Get the backend type for the array and pick out the array data
     // pointer field (assumed to be the second field.)
     tree ptr_field = TREE_CHAIN( TYPE_FIELDS( TREE_TYPE( exp )));
-    //return build(COMPONENT_REF, TREE_TYPE( ptr_field ), exp, ptr_field);
+    //return build2(COMPONENT_REF, TREE_TYPE( ptr_field ), exp, ptr_field);
     return component(exp, ptr_field);
 }
 
@@ -1797,7 +1905,7 @@ IRState::delegateObjectRef(tree exp)
     // Get the backend type for the array and pick out the array data
     // pointer field (assumed to be the first field.)
     tree obj_field = TYPE_FIELDS( TREE_TYPE( exp ));
-    //return build(COMPONENT_REF, TREE_TYPE( obj_field ), exp, obj_field);
+    //return build2(COMPONENT_REF, TREE_TYPE( obj_field ), exp, obj_field);
     return component(exp, obj_field);
 }
 
@@ -1807,7 +1915,7 @@ IRState::delegateMethodRef(tree exp)
     // Get the backend type for the array and pick out the array length
     // field (assumed to be the second field.)
     tree method_field = TREE_CHAIN( TYPE_FIELDS( TREE_TYPE( exp )));
-    //return build(COMPONENT_REF, TREE_TYPE( method_field ), exp, method_field);
+    //return build2(COMPONENT_REF, TREE_TYPE( method_field ), exp, method_field);
     return component(exp, method_field);
 }
 
@@ -1913,10 +2021,10 @@ IRState::objectInstanceMethod(Expression * obj_exp, FuncDeclaration * func, Type
 		vtbl_ref = indirect(this_expr);
 		
 	    tree field = TYPE_FIELDS( TREE_TYPE( vtbl_ref )); // the vtbl is the first field
-	    //vtbl_ref = build( COMPONENT_REF, TREE_TYPE( field ), vtbl_ref, field ); // vtbl field (a pointer)
+	    //vtbl_ref = build2( COMPONENT_REF, TREE_TYPE( field ), vtbl_ref, field ); // vtbl field (a pointer)
 	    vtbl_ref = component( vtbl_ref, field ); // vtbl field (a pointer)
 	    // %% better to do with array ref?
-	    vtbl_ref = build( PLUS_EXPR, TREE_TYPE(vtbl_ref), vtbl_ref,
+	    vtbl_ref = pointerOffset(vtbl_ref, 
 		size_int( PTRSIZE * func->vtblIndex ));
 	    vtbl_ref = indirect(vtbl_ref, TREE_TYPE( functionPointer(func) ));
 
@@ -1973,6 +2081,12 @@ IRState::twoFieldType(tree rec_type, tree ft1, tree ft2, Type * d_type, const ch
 	TYPE_STUB_DECL(rec_type) = stub_decl;
 	TYPE_NAME(rec_type) = stub_decl;
 	DECL_ARTIFICIAL(stub_decl) = 1;
+#if USE_MAPPED_LOCATION
+	DECL_SOURCE_LOCATION(stub_decl) = BUILTINS_LOCATION;
+	DECL_SOURCE_LOCATION(f0) = BUILTINS_LOCATION;
+	DECL_SOURCE_LOCATION(f1) = BUILTINS_LOCATION;
+// %% not sure if >= 4.3 and ! USE_MAPPED_LOCATION
+#endif
 	g.ofile->rodc(stub_decl, 0);
     }
     return rec_type;
@@ -2347,10 +2461,14 @@ IRState::integerConstant(xdmd_integer_t value, tree type) {
 # else
 #  error Fix This
 # endif
+
+# if D_GCC_VER < 43
     /* VALUE may be an incorrect representation for TYPE.  Example:
          uint x = cast(uint) -3; // becomes "-3u" -- value=0xfffffffffffffd type=Tuns32
        Constant folding will not work correctly unless this is done. */
     tree_value = force_fit_type(tree_value, 0, 0, 0);
+# endif
+
 #endif
     return tree_value;
 }
@@ -2361,9 +2479,10 @@ IRState::exceptionObject()
     tree obj_type = getObjectType()->toCtype();
     // Like gjc, the actual D exception object is one
     // pointer behind the exception header
-    tree t = build (EXC_PTR_EXPR, ptr_type_node);
+    tree t = build0 (EXC_PTR_EXPR, ptr_type_node);
     t = build1(NOP_EXPR, build_pointer_type(obj_type), t); // treat exception header as ( Object* )
-    t = build(MINUS_EXPR, TREE_TYPE(t), t, TYPE_SIZE_UNIT(TREE_TYPE(t)));
+    //t = build2(MINUS_EXPR, TREE_TYPE(t), t, TYPE_SIZE_UNIT(TREE_TYPE(t)));
+    t = pointerOffsetOp(MINUS_EXPR, t, TYPE_SIZE_UNIT(TREE_TYPE(t)));
     t = build1(INDIRECT_REF, obj_type, t);
     return t;
 }
@@ -2658,7 +2777,7 @@ IRState::getClosureRef(FuncDeclaration * outer_func)
 		   a class instances.)  With the current implementation,
 		   the link the closure record always points to the
 		   outer function's frame even if there are intervening
-		   nested classes.  So, we can just skip over those...
+		   nested classes.  So, we can just skip over those... */
 		/*
 		tree vthis_field = cd->vthis->toSymbol()->Stree;
 		result = nop(result, cd->type->toCtype());
@@ -3246,6 +3365,8 @@ void AggLayout::addField(tree field_decl, target_size_t offset)
 {
     DECL_CONTEXT( field_decl ) = aggType;
     // DECL_FCONTEXT( field_decl ) = aggType; // caller needs to set this
+    SET_DECL_OFFSET_ALIGN( field_decl,
+	TYPE_ALIGN( TREE_TYPE( field_decl )));
     DECL_FIELD_OFFSET (field_decl) = size_int( offset );
     DECL_FIELD_BIT_OFFSET (field_decl) = bitsize_int( 0 );
     Loc l(aggDecl->getModule(), 1); // Must set this or we crash with DWARF debugging
