@@ -21,7 +21,12 @@
 
 #include "d-irstate.h"
 
+#if V1
+typedef dinteger_t xdmd_integer_t;
+#endif
+#if V2
 typedef integer_t xdmd_integer_t;
+#endif
 
 #include "d-objfile.h"
 
@@ -301,6 +306,9 @@ struct IRState : IRBase
     tree pointerIntSum(Expression * ptr_exp, Expression * idx_exp) {
 	return pointerIntSum(ptr_exp->toElem(this), idx_exp->toElem(this)); }
     tree pointerIntSum(tree ptr_node, tree idx_exp);
+//    static tree pointIntOp(int op, tree ptr, tree idx);
+    static tree pointerOffsetOp(int op, tree ptr, tree idx);
+    static tree pointerOffset(tree ptr_node, tree byte_offset);
 
     static tree nop(tree e, tree t) {
 	return build1(NOP_EXPR, t, e); }
@@ -316,7 +324,7 @@ struct IRState : IRBase
     }
 
     tree boolOp(enum tree_code code, tree a, tree b) {
-	return build(code, boolean_type_node, a, b);
+	return build2(code, boolean_type_node, a, b);
     }
     
     tree checkedIndex(Loc loc, tree index, tree upper_bound, bool inclusive);
@@ -327,14 +335,14 @@ struct IRState : IRBase
 #if D_GCC_VER < 40
     static tree binding(tree var_chain, tree body)
     {
-	return build(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
+	return build3(BIND_EXPR, TREE_TYPE(body), var_chain, body, NULL_TREE);
     }
 #else
     static tree binding(tree var_chain, tree body);
 #endif
     
     static tree compound(tree a, tree b, tree type = 0) {
-	return build(COMPOUND_EXPR, type ? type : TREE_TYPE(b), a, b);
+	return build2(COMPOUND_EXPR, type ? type : TREE_TYPE(b), a, b);
     }
     tree voidCompound(tree a, tree b) { return compound(a, b, void_type_node); }
     tree maybeCompound(tree a, tree b) {
@@ -343,7 +351,7 @@ struct IRState : IRBase
 	else if (b == NULL_TREE)
 	    return a;
 	else
-	    return build(COMPOUND_EXPR, TREE_TYPE(b), a, b);
+	    return build2(COMPOUND_EXPR, TREE_TYPE(b), a, b);
     }
     tree maybeVoidCompound(tree a, tree b) {
 	if (a == NULL_TREE)
@@ -351,13 +359,13 @@ struct IRState : IRBase
 	else if (b == NULL_TREE)
 	    return a;
 	else
-	    return build(COMPOUND_EXPR, void_type_node, a, b);
+	    return build2(COMPOUND_EXPR, void_type_node, a, b);
     }
 
     static tree component(tree v, tree f)
     {
 #if D_GCC_VER < 40
-	return build(COMPONENT_REF, TREE_TYPE(f), v, f);
+	return build2(COMPONENT_REF, TREE_TYPE(f), v, f);
 #else
 	return build3(COMPONENT_REF, TREE_TYPE(f), v, f, NULL_TREE);
 #endif
@@ -391,10 +399,12 @@ struct IRState : IRBase
     // sets it depending on the const/pure attributes of the funcion
     // and the SIDE_EFFECTS flags of the arguments.
     static tree buildCall(tree type, tree callee, tree args) {
-#if D_GCC_VER < 40
-	tree t = build(CALL_EXPR, type, callee, args);
-#else
+#if D_GCC_VER >= 43
+	tree t = build_call_list(type, callee, args);
+#elif D_GCC_VER >= 40
 	tree t = build3(CALL_EXPR, type, callee, args, NULL_TREE);
+#else
+	tree t = build2(CALL_EXPR, type, callee, args);
 #endif
 #if D_GCC_VER <= 33
 	TREE_SIDE_EFFECTS(t) = 1;
@@ -611,6 +621,35 @@ public:
     }
 };
 
+class CallExpr {
+public:
+    tree ce;
+#if D_GCC_VER >= 43
+    int argi;
+    CallExpr(tree ce_) : ce(ce_), argi(0) { }
+    tree callee() { return CALL_EXPR_FN(ce); }
+    tree nextArg() 
+    {
+	tree result = argi < call_expr_nargs(ce) ? 
+	    CALL_EXPR_ARG(ce, argi) : NULL_TREE;
+	++argi;
+	return result;
+    }
+#else
+    tree arge;
+    CallExpr(tree ce_) : ce(ce_), arge(TREE_OPERAND(ce, 1)) { }
+    tree callee() { return TREE_OPERAND(ce, 0); }
+    tree nextArg()
+    {
+	tree result = arge ? TREE_VALUE(arge) : NULL_TREE;
+	arge = TREE_CHAIN(arge);
+	return result;
+    }    
+#endif
+private:
+    CallExpr() { }
+};
+
 #endif
 
 
