@@ -764,6 +764,7 @@ Symbol *ClassDeclaration::toSymbol()
 	dkeep(decl);
  
 	g.ofile->setupStaticStorage(this, decl);
+	g.ofile->setDeclLoc(decl, this);
 
 	TREE_CONSTANT( decl ) = 0; // DMD puts this into .data, not .rodata...
 	TREE_READONLY( decl ) = 0;
@@ -818,6 +819,7 @@ Symbol *Module::toSymbol()
 
 	tree decl = build_decl(VAR_DECL, get_identifier(csym->Sident),
 	    TREE_TYPE(some_type->toCtype())); // want the RECORD_TYPE, not the REFERENCE_TYPE
+	g.ofile->setDeclLoc(decl, this);
 	csym->Stree = decl;
 	
 	dkeep(decl);
@@ -854,6 +856,7 @@ Symbol *ClassDeclaration::toVtblSymbol()
 	dkeep(decl);
 
 	g.ofile->setupStaticStorage(this, decl);
+	g.ofile->setDeclLoc(decl, this);
 
 	TREE_READONLY( decl ) = 1;
 	TREE_CONSTANT( decl ) = 1;
@@ -900,13 +903,14 @@ Symbol *AggregateDeclaration::toInitializer()
 	dkeep(t);
 
 	g.ofile->setupStaticStorage(this, t);
+	g.ofile->setDeclLoc(t, this);
 
 	// %% what's the diff between setting this stuff on the DECL and the
 	// CONSTRUCTOR itself?
 
 	TREE_ADDRESSABLE( t ) = 1;
 	TREE_CONSTANT( t ) = 1;
-	TREE_READONLY( t ) = 1;
+	TREE_INVARIANT( t ) = 1;
 	DECL_CONTEXT( t ) = 0; // These are always global
     }
     return sinit;
@@ -955,6 +959,7 @@ Symbol *EnumDeclaration::toInitializer()
 	dkeep(t);
 
 	g.ofile->setupStaticStorage(this, t);
+	g.ofile->setDeclLoc(t, this);
 	TREE_CONSTANT( t ) = 1;
 	TREE_READONLY( t ) = 1;
 	DECL_CONTEXT( t ) = 0;
@@ -989,7 +994,11 @@ Symbol *Module::toModuleArray()
  *		1	add value signature
  */
 
+#if V1
+Symbol *TypeAArray::aaGetSymbol(const char *func, int flags)
+#else
 Symbol *TypeAArray::aaGetSymbol(char *func, int flags)
+#endif
 {
     // This is not used in GCC (yet?)
     return 0;
