@@ -68,6 +68,13 @@ static char lang_name[6] = "GNU D";
 #undef LANG_HOOKS_TYPE_FOR_SIZE
 #define LANG_HOOKS_TYPE_FOR_SIZE d_type_for_size
 
+#undef LANG_HOOKS_BUILTIN_FUNCTION
+#if D_GCC_VER >= 43
+# define LANG_HOOKS_BUILTIN_FUNCTION d_builtin_function43
+#else
+# define LANG_HOOKS_BUILTIN_FUNCTION d_builtin_function
+#endif
+
 #if D_GCC_VER < 40
 #undef LANG_HOOKS_EXPAND_EXPR
 #define LANG_HOOKS_EXPAND_EXPR d_expand_expr
@@ -85,12 +92,8 @@ static char lang_name[6] = "GNU D";
 #undef LANG_HOOKS_WRITE_GLOBALS
 #define LANG_HOOKS_WRITE_GLOBALS d_write_global_declarations
 
-// Some phobos code (isnormal, etc.) breaks with strict aliasing...
-// so I guess D doesn't have aliasing rules.  Would be nice to enable
-// strict aliasing, but hooking or defaulting flag_strict_aliasing is
-// not trivial
 #undef LANG_HOOKS_GET_ALIAS_SET
-#define LANG_HOOKS_GET_ALIAS_SET hook_get_alias_set_0
+#define LANG_HOOKS_GET_ALIAS_SET d_hook_get_alias_set
 
 // Needed for try/finally -- statements cannont be re-evaluated
 #if D_GCC_VER < 40
@@ -103,12 +106,13 @@ static char lang_name[6] = "GNU D";
 #undef LANG_HOOKS_FORMAT_ATTRIBUTE_TABLE
 #define LANG_HOOKS_FORMAT_ATTRIBUTE_TABLE d_common_format_attribute_table
 
-#if D_GCC_VER >= 40
+#if D_GCC_VER >= 40 && D_GCC_VER < 43
 #undef LANG_HOOKS_CALLGRAPH_EXPAND_FUNCTION
 #define LANG_HOOKS_CALLGRAPH_EXPAND_FUNCTION d_expand_function
 #endif
 
 #if D_GCC_VER >= 40
+#undef LANG_HOOKS_TYPES_COMPATIBLE_P
 #define LANG_HOOKS_TYPES_COMPATIBLE_P d_types_compatible_p
 
 #if V2
@@ -192,13 +196,16 @@ char cygwin_d_target_dir[sizeof(D_PHOBOS_TARGET_DIR) + 1
 #undef D_PHOBOS_TARGET_DIR
 #define D_PHOBOS_TARGET_DIR (cygwin_d_target_dir)
 
+#ifdef D_OS_VERSYM
 const char * cygwin_d_os_versym = D_OS_VERSYM;
 #undef D_OS_VERSYM
 #define D_OS_VERSYM cygwin_d_os_versym
+#endif
 
 void
 maybe_fixup_cygwin()
 {
+#ifdef D_OS_VERSYM
     char * env = getenv("GCC_CYGWIN_MINGW");
     char * p;
     char ** av;
@@ -228,6 +235,7 @@ maybe_fixup_cygwin()
 		    strcat (*av, CYGWIN_MINGW_SUBDIR);
             }
     }
+#endif
 }
 
 static bool is_target_win32 = false;
@@ -315,6 +323,16 @@ d_init ()
 #    endif
 #  endif
 #endif
+
+#ifdef USE_MAPPED_LOCATION
+    /* input_location is initially set to BUILTINS_LOCATION (2).  This 
+       will cause a segfault if errors are reported before any line maps 
+       are created. Setting input_location to zero stops those segfaults,
+       but then decls are created with DECL_SOURCE_LOCATION set to zero
+       which causes segfaults in dwarf2out. */
+    linemap_add(line_table, LC_RENAME, 0, "<builtin>", 1);
+    input_location = linemap_line_start(line_table, 1, 1);
+#endif
     
     Type::init();
     Id::initialize();
@@ -342,8 +360,15 @@ d_init ()
 	VersionCondition::addPredefinedGlobalIdent((char*) cpu_versym);
 #ifdef D_OS_VERSYM
     VersionCondition::addPredefinedGlobalIdent((char*) D_OS_VERSYM);
+    if (strcmp(D_OS_VERSYM, "darwin") == 0)
+    VersionCondition::addPredefinedGlobalIdent((char*) "OSX");
     if (strcmp(D_OS_VERSYM, "Win32") == 0)
-	is_target_win32 = true;
+	 {
+		 VersionCondition::addPredefinedGlobalIdent("Windows");
+		 is_target_win32 = true;
+	 }
+    if (strcmp(D_OS_VERSYM, "freebsd") == 0)
+    VersionCondition::addPredefinedGlobalIdent((char*) "FreeBSD");
 #endif
 #ifdef D_OS_VERSYM2
     VersionCondition::addPredefinedGlobalIdent((char*) D_OS_VERSYM2);
@@ -392,13 +417,16 @@ d_init ()
 	int flag_iso = 0;   // ditto
 
 	linemap_init(& lm);
+#if D_GCC_VER >= 43
+	lm.reallocator = NULL;
+#endif
 	parse_in = pfile = cpp_create_reader(CLK_STDC89, NULL
 #if D_GCC_VER >= 40
 	    , & lm
 #endif
 					     );
 	cpp_change_file(pfile, LC_ENTER, "<built-in>");
-
+	
 	// from c-cppbuiltin.c
 #ifndef TARGET_OS_CPP_BUILTINS
 # define TARGET_OS_CPP_BUILTINS()
@@ -411,7 +439,12 @@ d_init ()
 # define preprocessing_trad_p() (cpp_get_options (pfile)->traditional)
 # define preprocessing_asm_p() (0)
 # define preprocessing_trad_p() (0)
+# define c_dialect_cxx() (0)
+# define c_dialect_objc() (0)
 # define builtin_define(TXT) (cpp_define (pfile, TXT))
+# define builtin_define_with_value(m,e,s)
+# define builtin_define_with_int_value(m,i)
+# define builtin_define_std(TXT) (cpp_define (pfile, TXT))
 # define builtin_assert(TXT) (cpp_assert (pfile, TXT))
 	TARGET_CPU_CPP_BUILTINS ();
 	TARGET_OS_CPP_BUILTINS ();
@@ -735,6 +768,21 @@ d_write_global_declarations()
 #endif
 }
 
+// Some phobos code (isnormal, etc.) breaks with strict aliasing...
+// so I guess D doesn't have aliasing rules.  Would be nice to enable
+// strict aliasing, but hooking or defaulting flag_strict_aliasing is
+// not trivial
+static
+#if D_GCC_VER < 43
+HOST_WIDE_INT
+#else
+alias_set_type
+#endif
+d_hook_get_alias_set(tree)
+{
+    return 0;
+}
+
 #if D_GCC_VER < 40
 // taken from c_common_unsafe_for_reeval
 int
@@ -805,7 +853,7 @@ d_parse_file (int /*set_yydebug*/)
     gen.originalOmitFramePointer = flag_omit_frame_pointer;
 
     // better to use input_location.xxx ?
-    (*debug_hooks->start_source_file) (input_line, input_filename);
+    (*debug_hooks->start_source_file) (input_line, main_input_filename);
 
     /*
     printf("input_filename = '%s'\n", input_filename);
@@ -844,7 +892,7 @@ d_parse_file (int /*set_yydebug*/)
     modules.reserve(num_in_fnames);
     
     // %% FIX
-    if ( ! input_filename ) {
+    if ( ! main_input_filename ) {
 	::error("input file name required; cannot use stdin");
 	goto had_errors;
     }
@@ -852,7 +900,7 @@ d_parse_file (int /*set_yydebug*/)
     if (fonly_arg) {
 	/* In this mode, the first file name is supposed to be
 	   a duplicate of one of the input file. */
-	if (strcmp(fonly_arg, input_filename))
+	if (strcmp(fonly_arg, main_input_filename))
 	    ::error("-fonly= argument is different from main input file name");
 	if (strcmp(fonly_arg, in_fnames[0]))
 	    ::error("-fonly= argument is different from first input file name");
@@ -900,7 +948,7 @@ d_parse_file (int /*set_yydebug*/)
 	
 	id = new Identifier(name, 0);
 	Module * m = new Module(the_fname, id, global.params.doDocComments, global.params.doHdrGeneration);
-	if (! strcmp(in_fnames[i], input_filename))
+	if (! strcmp(in_fnames[i], main_input_filename))
 	    an_output_module = m;
 	modules.push(m);
     }
@@ -1669,7 +1717,7 @@ const char *const tree_code_name[] = {
 };
 #undef DEFTREECODE
 
-#if D_GCC_VER >= 40
+#if D_GCC_VER >= 40 && D_GCC_VER < 43
 
 static void
 d_expand_function(tree fndecl)
