@@ -35,7 +35,8 @@ d_using_sjlj_exceptions(void)
     return USING_SJLJ_EXCEPTIONS;
 }
 
-/* ASM_FORMAT_PRIVATE_NAME implicitly casts to char* from void* (alloca call) */
+/* ASM_FORMAT_PRIVATE_NAME implicitly casts to char* from void*
+   (from an alloca call).  This is fixed in GCC 4.x */
 
 char *
 d_asm_format_private_name(const char * name, int value)
