@@ -74,16 +74,23 @@ ObjectFile::hasModule(Module *m)
 void
 ObjectFile::finish()
 {
+#if D_GCC_VER < 43
+#define D_FFN_I 'I'
+#define D_FFN_D 'D'
+#else
+#define D_FFN_I "I"
+#define D_FFN_D "D"
+#endif
     /* If the target does not directly support static constructors,
        staticCtorList contains a list of all static constructors defined
        so far.  This routine will create a function to call all of those
        and is picked up by collect2. */
     if (staticCtorList.dim) {
-	doFunctionToCallFunctions(IDENTIFIER_POINTER(get_file_function_name('I')),
+	doFunctionToCallFunctions(IDENTIFIER_POINTER(get_file_function_name(D_FFN_I)),
 	    & staticCtorList, true);
     }
     if (staticDtorList.dim) {
-	doFunctionToCallFunctions(IDENTIFIER_POINTER(get_file_function_name('D')),
+	doFunctionToCallFunctions(IDENTIFIER_POINTER(get_file_function_name(D_FFN_D)),
 	    & staticDtorList, true);
     }
 }
@@ -102,12 +109,51 @@ ObjectFile::doLineNote(const Loc & loc)
     }
 }
 
+#ifdef USE_MAPPED_LOCATION
+static location_t
+cvtLocToloc_t(const Loc loc)
+{
+    //assert(sizeof(StringValue.intvalue) == sizeof(location_t));
+    static StringTable lmtab;
+    StringValue * sv = lmtab.update(loc.filename, strlen(loc.filename));
+    const struct line_map * lm = 0;
+    unsigned new_line_count = 0;
+    
+    if (! sv->intvalue)
+    {
+	new_line_count = 5000;
+    }
+    else
+    {
+	lm = linemap_lookup(line_table, sv->intvalue);
+	unsigned last_line = LAST_SOURCE_LINE(lm);
+	if (loc.linnum > last_line)
+	    new_line_count = last_line * 5;
+    }
+    if (new_line_count)
+    {
+	lm = linemap_add(line_table, LC_ENTER, 0/*...*/, loc.filename, 0);
+	sv->intvalue = linemap_line_start(line_table, new_line_count, 0);
+	linemap_add(line_table, LC_LEAVE, 0, NULL, 0);
+	lm = linemap_lookup(line_table, sv->intvalue);
+    }
+    
+    // cheat...
+    return
+    	lm->start_location + ((loc.linnum - lm->to_line) << lm->column_bits);
+}
+#endif
+
 void
 ObjectFile::setLoc(const Loc & loc)
 {
     if (loc.filename) {
+#ifdef USE_MAPPED_LOCATION
+	input_location = cvtLocToloc_t(loc);
+#else
 	location_t gcc_loc = { loc.filename, loc.linnum };
 	input_location = gcc_loc;
+#endif
     } else {
 	// do nothing
     }
@@ -119,19 +165,24 @@ ObjectFile::setDeclLoc(tree t, const Loc & loc)
     // DWARF2 will often crash if the DECL_SOURCE_FILE is not set.  It's
     // easier the error here.
     assert(loc.filename);
+#ifdef USE_MAPPED_LOCATION
+    DECL_SOURCE_LOCATION (t) = cvtLocToloc_t(loc);
+#else
     DECL_SOURCE_FILE (t) = loc.filename;
     DECL_SOURCE_LINE (t) = loc.linnum;
+#endif
 }
 
 void
 ObjectFile::setDeclLoc(tree t, Dsymbol * decl)
 {
-    while (decl) {
-	if (decl->loc.filename) {
-	    setDeclLoc(t, decl->loc);
+    Dsymbol * dsym = decl;
+    while (dsym) {
+	if (dsym->loc.filename) {
+	    setDeclLoc(t, dsym->loc);
 	    return;
 	}
-	decl = decl->toParent();
+	dsym = dsym->toParent();
     }
 
     // fallback; backend sometimes crashes if not set
@@ -148,6 +199,28 @@ ObjectFile::setDeclLoc(tree t, Dsymbol * decl)
     setDeclLoc(t, l); 
 }
 
+#if D_GCC_VER >= 40
+void
+ObjectFile::setCfunEndLoc(const Loc & loc)
+{
+    tree fn_decl = cfun->decl;
+# ifdef USE_MAPPED_LOCATION
+    if (loc.filename)
+	cfun->function_end_locus = cvtLocToloc_t(loc);
+    else
+	cfun->function_end_locus = DECL_SOURCE_LOCATION(fn_decl);
+# else
+    if (loc.filename) {
+	cfun->function_end_locus.file = loc.filename;
+	cfun->function_end_locus.line = loc.linnum;
+    } else {
+	cfun->function_end_locus.file = DECL_SOURCE_FILE (fn_decl);
+	cfun->function_end_locus.line = DECL_SOURCE_LINE (fn_decl);
+    }
+# endif    
+}
+#endif
+
 void
 ObjectFile::giveDeclUniqueName(tree decl, const char * prefix)
 {
@@ -524,8 +597,14 @@ ObjectFile::initTypeDecl(tree t, tree decl)
 	    /* Not sure if there is a need for separate TYPE_DECLs in
 	       TYPE_NAME and TYPE_STUB_DECL. */
 	    TYPE_STUB_DECL( t ) = build_decl(TYPE_DECL, DECL_NAME(decl), t);
+#if D_GCC_VER >= 43
+	    DECL_ARTIFICIAL( TYPE_STUB_DECL( t ) ) = 1; // dunno...
+	    // code now assumes...
+	    DECL_SOURCE_LOCATION(TYPE_STUB_DECL(t)) = DECL_SOURCE_LOCATION(decl);
+#else
 	    // g++ does this and the debugging code assumes it:
 	    DECL_ARTIFICIAL( TYPE_STUB_DECL( t ) ) = 1;
+#endif	    
 	    break;
 	default:
 	    // nothing
@@ -690,7 +769,7 @@ ObjectFile::outputThunk(tree thunk_decl, tree target_decl, target_ptrdiff_t offs
 	    delta, 0, alias);
 	assemble_end_function(thunk_decl, fnname);
 	current_function_decl = 0;
-	cfun = 0;
+	set_cfun(0);
 #if D_GCC_VER < 40
 	/* Because init_function_start increments this, we must
 	   decrement it.  */
@@ -761,7 +840,7 @@ ObjectFile::doFunctionToCallFunctions(const char * name, Array * functions, bool
 	    if (! exp)
 		exp = call_exp;
 	    else {
-		exp = build(COMPOUND_EXPR, void_type_node, exp, call_exp);
+		exp = build2(COMPOUND_EXPR, void_type_node, exp, call_exp);
 	    }
 	}
 
@@ -910,7 +989,7 @@ dt2node(dt_t * dt)
 	// %% make sure this is the target word type
 	return gen.integerConstant(dt->DTint, Type::tsize_t);
     case DT_xoff:
-	return build(PLUS_EXPR, ptr_type_node,
+	return gen.pointerOffset(
 	    gen.addressOf(check_static_sym(dt->DTsym)),
 	    gen.integerConstant(dt->DTint, Type::tsize_t) );
     case DT_tree:
@@ -1055,7 +1134,7 @@ outdata(Symbol * sym)
 }
 
 void
-obj_includelib(char *name)
+obj_includelib(const char *name)
 {
     d_warning(OPT_Wunknown_pragmas, "pragma(lib) not implemented");
 }
@@ -1099,6 +1178,7 @@ obj_moduleinfo(Symbol *sym)
     tree our_mod_ref = build_decl(VAR_DECL, NULL_TREE, mod_ref_type);
     dkeep(our_mod_ref);
     g.ofile->giveDeclUniqueName(our_mod_ref, "__mod_ref");
+    g.ofile->setDeclLoc(our_mod_ref, g.mod);
 
     tree init = make_node(CONSTRUCTOR);
     CtorEltMaker ce;
@@ -1120,12 +1200,12 @@ obj_moduleinfo(Symbol *sym)
     DECL_EXTERNAL( the_mod_ref ) = 1;
     TREE_PUBLIC( the_mod_ref ) = 1;
 
-    tree m1 = build(MODIFY_EXPR, void_type_node,
+    tree m1 = build2(MODIFY_EXPR, void_type_node,
 	gen.component(our_mod_ref, f0),
 	the_mod_ref);
-    tree m2 = build(MODIFY_EXPR, void_type_node,
+    tree m2 = build2(MODIFY_EXPR, void_type_node,
 	the_mod_ref, gen.addressOf(our_mod_ref));
-    tree exp = build(COMPOUND_EXPR, void_type_node, m1, m2);
+    tree exp = build2(COMPOUND_EXPR, void_type_node, m1, m2);
 
     g.ofile->doSimpleFunction("*__modinit", exp, true);
 }
