@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -11,18 +11,15 @@
 /* NOTE: This file has been patched from the original DMD distribution to
    work with the GDC compiler.
 
-   Modified by David Friedman, December 2006
+   Modified by Michael Parrott, September 2009
+   Using David Friedman's code
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 
-#if _WIN32 || IN_GCC
-#include "mem.h"
-#elif linux
-#include "../root/mem.h"
-#endif
+#include "rmem.h"
 
 #include "init.h"
 #include "declaration.h"
@@ -37,7 +34,7 @@
 #include "parse.h"
 #include "template.h"
 
-extern void obj_includelib(char *name);
+extern void obj_includelib(const char *name);
 void obj_startaddress(Symbol *s);
 
 
@@ -56,27 +53,59 @@ Array *AttribDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 
 int AttribDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-    unsigned i;
     int m = 0;
     Array *d = include(sc, sd);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    m |= s->addMember(sc, sd, m | memnum);
 	}
     }
     return m;
 }
 
+void AttribDeclaration::semanticNewSc(Scope *sc,
+	unsigned stc, enum LINK linkage, enum PROT protection, int explicitProtection,
+	unsigned structalign)
+{
+    if (decl)
+    {
+	Scope *newsc = sc;
+	if (stc != sc->stc ||
+	    linkage != sc->linkage ||
+	    protection != sc->protection ||
+	    explicitProtection != sc->explicitProtection ||
+	    structalign != sc->structalign)
+	{
+	    // create new one for changes
+	    newsc = new Scope(*sc);
+	    newsc->flags &= ~SCOPEfree;
+	    newsc->stc = stc;
+	    newsc->linkage = linkage;
+	    newsc->protection = protection;
+	    newsc->explicitProtection = explicitProtection;
+	    newsc->structalign = structalign;
+	}
+	for (unsigned i = 0; i < decl->dim; i++)
+	{   Dsymbol *s = (Dsymbol *)decl->data[i];
+
+	    s->semantic(newsc);
+	}
+	if (newsc != sc)
+	{
+	    sc->offset = newsc->offset;
+	    newsc->pop();
+	}
+    }
+}
+
 void AttribDeclaration::semantic(Scope *sc)
 {
     Array *d = include(sc, NULL);
 
-    //printf("\tAttribDeclaration::semantic '%s'\n",toChars());
+    //printf("\tAttribDeclaration::semantic '%s', d = %p\n",toChars(), d);
     if (d)
     {
 	for (unsigned i = 0; i < d->dim; i++)
@@ -90,15 +119,12 @@ void AttribDeclaration::semantic(Scope *sc)
 
 void AttribDeclaration::semantic2(Scope *sc)
 {
-    unsigned i;
     Array *d = include(sc, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    s->semantic2(sc);
 	}
     }
@@ -106,15 +132,12 @@ void AttribDeclaration::semantic2(Scope *sc)
 
 void AttribDeclaration::semantic3(Scope *sc)
 {
-    unsigned i;
     Array *d = include(sc, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    s->semantic3(sc);
 	}
     }
@@ -122,15 +145,12 @@ void AttribDeclaration::semantic3(Scope *sc)
 
 void AttribDeclaration::inlineScan()
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    //printf("AttribDeclaration::inlineScan %s\n", s->toChars());
 	    s->inlineScan();
 	}
@@ -141,15 +161,12 @@ void AttribDeclaration::addComment(unsigned char *comment)
 {
     if (comment)
     {
-	unsigned i;
 	Array *d = include(NULL, NULL);
 
 	if (d)
 	{
-	    for (i = 0; i < d->dim; i++)
-	    {   Dsymbol *s;
-
-		s = (Dsymbol *)d->data[i];
+	    for (unsigned i = 0; i < d->dim; i++)
+ 	    {   Dsymbol *s = (Dsymbol *)d->data[i];
 		//printf("AttribDeclaration::addComment %s\n", s->toChars());
 		s->addComment(comment);
 	    }
@@ -160,22 +177,21 @@ void AttribDeclaration::addComment(unsigned char *comment)
 void AttribDeclaration::emitComment(Scope *sc)
 {
     //printf("AttribDeclaration::emitComment(sc = %p)\n", sc);
+    
+    /* A general problem with this, illustrated by BUGZILLA 2516,
++      * is that attributes are not transmitted through to the underlying
++      * member declarations for template bodies, because semantic analysis
++      * is not done for template declaration bodies
++      * (only template instantiations).
++      * Hence, Ddoc omits attributes from template members.
++      */
 
-    /* If generating doc comment, skip this because if we're inside
-     * a template, then include(NULL, NULL) will fail.
-     */
-//    if (sc->docbuf)
-//	return;
-
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    //printf("AttribDeclaration::emitComment %s\n", s->toChars());
 	    s->emitComment(sc);
 	}
@@ -184,15 +200,12 @@ void AttribDeclaration::emitComment(Scope *sc)
 
 void AttribDeclaration::toObjFile(int multiobj)
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    s->toObjFile(multiobj);
 	}
     }
@@ -200,17 +213,14 @@ void AttribDeclaration::toObjFile(int multiobj)
 
 int AttribDeclaration::cvMember(unsigned char *p)
 {
-    unsigned i;
     int nwritten = 0;
     int n;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    n = s->cvMember(p);
 	    if (p)
 		p += n;
@@ -236,7 +246,7 @@ int AttribDeclaration::hasPointers()
     return 0;
 }
 
-char *AttribDeclaration::kind()
+const char *AttribDeclaration::kind()
 {
     return "attribute";
 }
@@ -250,15 +260,12 @@ int AttribDeclaration::oneMember(Dsymbol **ps)
 
 void AttribDeclaration::checkCtorConstInit()
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    s->checkCtorConstInit();
 	}
     }
@@ -268,15 +275,13 @@ void AttribDeclaration::checkCtorConstInit()
  */
 
 void AttribDeclaration::addLocalClass(ClassDeclarations *aclasses)
-{   unsigned i;
+{  
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    s->addLocalClass(aclasses);
 	}
     }
@@ -324,10 +329,38 @@ Dsymbol *StorageClassDeclaration::syntaxCopy(Dsymbol *s)
 void StorageClassDeclaration::semantic(Scope *sc)
 {
     if (decl)
-    {	unsigned stc_save = sc->stc;
+    {
+#if 1
+	unsigned scstc = sc->stc;
 
-	if (stc & (STCauto | STCscope | STCstatic | STCextern))
-	    sc->stc &= ~(STCauto | STCscope | STCstatic | STCextern);
+	/* These sets of storage classes are mutually exclusive,
+	 * so choose the innermost or most recent one.
+	 */
+	if (stc & (STCauto | STCscope | STCstatic | STCextern | STCmanifest))
+	    scstc &= ~(STCauto | STCscope | STCstatic | STCextern | STCmanifest);
+	if (stc & (STCauto | STCscope | STCstatic | STCtls | STCmanifest | STCgshared))
+	    scstc &= ~(STCauto | STCscope | STCstatic | STCtls | STCmanifest | STCgshared);
+	if (stc & (STCconst | STCimmutable | STCmanifest))
+	    scstc &= ~(STCconst | STCimmutable | STCmanifest);
+	if (stc & (STCgshared | STCshared | STCtls))
+	    scstc &= ~(STCgshared | STCshared | STCtls);
+	scstc |= stc;
+
+	semanticNewSc(sc, scstc, sc->linkage, sc->protection, sc->explicitProtection, sc->structalign);
+#else
+	unsigned stc_save = sc->stc;
+
+	/* These sets of storage classes are mutually exclusive,
+	 * so choose the innermost or most recent one.
+	 */
+	if (stc & (STCauto | STCscope | STCstatic | STCextern | STCmanifest))
+	    sc->stc &= ~(STCauto | STCscope | STCstatic | STCextern | STCmanifest);
+	if (stc & (STCauto | STCscope | STCstatic | STCtls | STCmanifest | STCgshared))
+	    sc->stc &= ~(STCauto | STCscope | STCstatic | STCtls | STCmanifest | STCgshared);
+	if (stc & (STCconst | STCimmutable | STCmanifest))
+	    sc->stc &= ~(STCconst | STCimmutable | STCmanifest);
+	if (stc & (STCgshared | STCshared | STCtls))
+	    sc->stc &= ~(STCgshared | STCshared | STCtls);
 	sc->stc |= stc;
 	for (unsigned i = 0; i < decl->dim; i++)
 	{
@@ -336,12 +369,11 @@ void StorageClassDeclaration::semantic(Scope *sc)
 	    s->semantic(sc);
 	}
 	sc->stc = stc_save;
+	#endif
     }
-    else
-	sc->stc = stc;
 }
 
-void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
+void StorageClassDeclaration::stcToCBuffer(OutBuffer *buf, int stc)
 {
     struct SCstring
     {
@@ -356,24 +388,33 @@ void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 	{ STCstatic,       TOKstatic },
 	{ STCextern,       TOKextern },
 	{ STCconst,        TOKconst },
+ //	{ STCinvariant,    TOKimmutable },
+ //	{ STCshared,       TOKshared },
 	{ STCfinal,        TOKfinal },
 	{ STCabstract,     TOKabstract },
 	{ STCsynchronized, TOKsynchronized },
 	{ STCdeprecated,   TOKdeprecated },
 	{ STCoverride,     TOKoverride },
+ //	{ STCnothrow,      TOKnothrow },
+ //	{ STCpure,         TOKpure },
+ //	{ STCref,          TOKref },
+ //	{ STCtls,          TOKtls },
     };
 
-    int written = 0;
     for (int i = 0; i < sizeof(table)/sizeof(table[0]); i++)
     {
 	if (stc & table[i].stc)
 	{
-	    if (written)
-		buf->writeByte(' ');
-	    written = 1;
 	    buf->writestring(Token::toChars(table[i].tok));
+	    buf->writeByte(' ');
 	}
     }
+    
+}
+  
+void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
+{
+    stcToCBuffer(buf, stc);
 
     AttribDeclaration::toCBuffer(buf, hgs);
 }
@@ -400,7 +441,11 @@ void LinkDeclaration::semantic(Scope *sc)
 {
     //printf("LinkDeclaration::semantic(linkage = %d, decl = %p)\n", linkage, decl);
     if (decl)
-    {	enum LINK linkage_save = sc->linkage;
+    {
+#if 1
+	semanticNewSc(sc, sc->stc, linkage, sc->protection, sc->explicitProtection, sc->structalign);
+#else
+	enum LINK linkage_save = sc->linkage;
 
 	sc->linkage = linkage;
 	for (unsigned i = 0; i < decl->dim; i++)
@@ -410,10 +455,7 @@ void LinkDeclaration::semantic(Scope *sc)
 	    s->semantic(sc);
 	}
 	sc->linkage = linkage_save;
-    }
-    else
-    {
-	sc->linkage = linkage;
+    #endif
     }
 }
 
@@ -439,7 +481,7 @@ void LinkDeclaration::semantic3(Scope *sc)
 }
 
 void LinkDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
-{   char *p;
+{   const char *p;
 
     switch (linkage)
     {
@@ -460,7 +502,7 @@ void LinkDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 char *LinkDeclaration::toChars()
 {
-    return "extern ()";
+    return (char *)"extern ()";
 }
 
 /********************************* ProtDeclaration ****************************/
@@ -484,7 +526,11 @@ Dsymbol *ProtDeclaration::syntaxCopy(Dsymbol *s)
 void ProtDeclaration::semantic(Scope *sc)
 {
     if (decl)
-    {	enum PROT protection_save = sc->protection;
+    {
+#if 1
+	semanticNewSc(sc, sc->stc, sc->linkage, protection, 1, sc->structalign);
+#else
+	enum PROT protection_save = sc->protection;
 	int explicitProtection_save = sc->explicitProtection;
 
 	sc->protection = protection;
@@ -497,15 +543,13 @@ void ProtDeclaration::semantic(Scope *sc)
 	}
 	sc->protection = protection_save;
 	sc->explicitProtection = explicitProtection_save;
-    }
-    else
-    {	sc->protection = protection;
-	sc->explicitProtection = 1;
+    #endif
     }
 }
 
-void ProtDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
-{   char *p;
+void ProtDeclaration::protectionToCBuffer(OutBuffer *buf, enum PROT protection)
+{
+    const char *p;
 
     switch (protection)
     {
@@ -519,6 +563,12 @@ void ProtDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 	    break;
     }
     buf->writestring(p);
+    buf->writeByte(' ');
+}
+
+void ProtDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
+{
+    protectionToCBuffer(buf, protection);
     AttribDeclaration::toCBuffer(buf, hgs);
 }
 
@@ -543,7 +593,11 @@ void AlignDeclaration::semantic(Scope *sc)
 {
     //printf("\tAlignDeclaration::semantic '%s'\n",toChars());
     if (decl)
-    {	unsigned salign_save = sc->structalign;
+    {
+#if 1
+	semanticNewSc(sc, sc->stc, sc->linkage, sc->protection, sc->explicitProtection, salign);
+#else
+	unsigned salign_save = sc->structalign;
 
 	sc->structalign = salign;
 	for (unsigned i = 0; i < decl->dim; i++)
@@ -553,9 +607,8 @@ void AlignDeclaration::semantic(Scope *sc)
 	    s->semantic(sc);
 	}
 	sc->structalign = salign_save;
+	#endif
     }
-    else
-	sc->structalign = salign;
 }
 
 
@@ -572,7 +625,6 @@ AnonDeclaration::AnonDeclaration(Loc loc, int isunion, Array *decl)
 {
     this->loc = loc;
     this->isunion = isunion;
-    this->scope = NULL;
     this->sem = 0;
 }
 
@@ -625,7 +677,7 @@ void AnonDeclaration::semantic(Scope *sc)
 
 	sc = sc->push();
 	sc->anonAgg = &aad;
-	sc->stc &= ~(STCauto | STCscope | STCstatic);
+	sc->stc &= ~(STCauto | STCscope | STCstatic | STCtls);
 	sc->inunion = isunion;
 	sc->offset = 0;
 	sc->flags = 0;
@@ -727,9 +779,9 @@ void AnonDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
     buf->writestring("}\n");
 }
 
-char *AnonDeclaration::kind()
+const char *AnonDeclaration::kind()
 {
-    return (char *)(isunion ? "anonymous union" : "anonymous struct");
+    return (isunion ? "anonymous union" : "anonymous struct");
 }
 
 /********************************* PragmaDeclaration ****************************/
@@ -744,6 +796,7 @@ PragmaDeclaration::PragmaDeclaration(Loc loc, Identifier *ident, Expressions *ar
 
 Dsymbol *PragmaDeclaration::syntaxCopy(Dsymbol *s)
 {
+	//printf("PragmaDeclaration::syntaxCopy(%s)\n", toChars());
     PragmaDeclaration *pd;
 
     assert(!s);
@@ -1018,7 +1071,7 @@ int PragmaDeclaration::oneMember(Dsymbol **ps)
     return TRUE;
 }
 
-char *PragmaDeclaration::kind()
+const char *PragmaDeclaration::kind()
 {
     return "pragma";
 }
@@ -1037,8 +1090,35 @@ void PragmaDeclaration::toObjFile(int multiobj)
 	char *name = (char *)mem.malloc(se->len + 1);
 	memcpy(name, se->string, se->len);
 	name[se->len] = 0;
-	obj_includelib(name);
+	#if OMFOBJ
+ 	/* The OMF format allows library names to be inserted
++ 	 * into the object file. The linker will then automatically
++ 	 * search that library, too.
++ 	 */
+  	obj_includelib(name);
+ 	#elif ELFOBJ || MACHOBJ
+ 	/* The format does not allow embedded library names,
+ 	/* The ELF format does not allow embedded library names,
++ 	 * so instead append the library name to the list to be passed
++ 	 * to the linker.
++ 	 */
+ 	global.params.libfiles->push((void *) name);
+ #else
+ 	error("pragma lib not supported");
+ #endif
     }
+    #if DMDV2
+     else if (ident == Id::startaddress)
+     {
+ 	assert(args && args->dim == 1);
+ 	Expression *e = (Expression *)args->data[0];
+ 	Dsymbol *sa = getDsymbol(e);
+ 	FuncDeclaration *f = sa->isFuncDeclaration();
+ 	assert(f);
+ 	Symbol *s = f->toSymbol();
+ 	obj_startaddress(s);
+     }
+ 	#endif
     AttribDeclaration::toObjFile(multiobj);
 }
 
@@ -1101,6 +1181,17 @@ void ConditionalDeclaration::emitComment(Scope *sc)
     {
 	AttribDeclaration::emitComment(sc);
     }
+    else if (sc->docbuf)
+     {
+ 	/* If generating doc comment, be careful because if we're inside
++ 	 * a template, then include(NULL, NULL) will fail.
++ 	 */
+ 	Array *d = decl ? decl : elsedecl;
+ 	for (unsigned i = 0; i < d->dim; i++)
+ 	{   Dsymbol *s = (Dsymbol *)d->data[i];
+ 	    s->emitComment(sc);
+ 	}
+     }
 }
 
 // Decide if 'then' or 'else' code should be included
@@ -1209,6 +1300,7 @@ Dsymbol *StaticIfDeclaration::syntaxCopy(Dsymbol *s)
 
 int StaticIfDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
+	//printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
     /* This is deferred until semantic(), so that
      * expressions in the condition can refer to declarations
      * in the same scope, such as:
@@ -1235,7 +1327,7 @@ void StaticIfDeclaration::semantic(Scope *sc)
 {
     Array *d = include(sc, sd);
 
-    //printf("\tStaticIfDeclaration::semantic '%s'\n",toChars());
+    //printf("\tStaticIfDeclaration::semantic '%s', d = %p\n",toChars(), d);
     if (d)
     {
 	if (!addisdone)
@@ -1252,7 +1344,7 @@ void StaticIfDeclaration::semantic(Scope *sc)
     }
 }
 
-char *StaticIfDeclaration::kind()
+const char *StaticIfDeclaration::kind()
 {
     return "static if";
 }
@@ -1263,8 +1355,11 @@ char *StaticIfDeclaration::kind()
 CompileDeclaration::CompileDeclaration(Loc loc, Expression *exp)
     : AttribDeclaration(NULL)
 {
+	//printf("CompileDeclaration(loc = %d)\n", loc.linnum);
+	this->loc = loc;
     this->exp = exp;
     this->sd = NULL;
+    this->compiled = 0;
 }
 
 Dsymbol *CompileDeclaration::syntaxCopy(Dsymbol *s)
@@ -1276,20 +1371,29 @@ Dsymbol *CompileDeclaration::syntaxCopy(Dsymbol *s)
 
 int CompileDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
+    //printf("CompileDeclaration::addMember(sc = %p, memnum = %d)\n", sc, memnum);
     this->sd = sd;
+    if (memnum == 0)
+    {	/* No members yet, so parse the mixin now
+ 	 */
+ 	compileIt(sc);
+ 	memnum |= AttribDeclaration::addMember(sc, sd, memnum);
+ 	compiled = 1;
+    }
     return memnum;
 }
 
-void CompileDeclaration::semantic(Scope *sc)
+void CompileDeclaration::compileIt(Scope *sc)
 {
-    //printf("CompileDeclaration::semantic()\n");
+     //printf("CompileDeclaration::compileIt(loc = %d)\n", loc.linnum);
     exp = exp->semantic(sc);
     exp = resolveProperties(sc, exp);
     exp = exp->optimize(WANTvalue | WANTinterpret);
     if (exp->op != TOKstring)
-    {	error("argument to mixin must be a string, not (%s)", exp->toChars());
-	return;
+    {	exp->error("argument to mixin must be a string, not (%s)", exp->toChars());
     }
+    else
+    {
     StringExp *se = (StringExp *)exp;
     se = se->toUTF8(sc);
     Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
@@ -1297,14 +1401,23 @@ void CompileDeclaration::semantic(Scope *sc)
     p.nextToken();
     decl = p.parseDeclDefs(0);
     if (p.token.value != TOKeof)
-    {
-	error("incomplete mixin declaration (%s)", se->toChars());
+	exp->error("incomplete mixin declaration (%s)", se->toChars());
     }
-
-    AttribDeclaration::addMember(sc, sd, 0);
-    AttribDeclaration::semantic(sc);
 }
 
+    void CompileDeclaration::semantic(Scope *sc)
+ {
+     //printf("CompileDeclaration::semantic()\n");
+ 
+     if (!compiled)
+     {
+ 	compileIt(sc);
+  	AttribDeclaration::addMember(sc, sd, 0);
+ 	compiled = 1;
+     }
+      AttribDeclaration::semantic(sc);
+  }
+
 void CompileDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     buf->writestring("mixin(");
