@@ -1,5 +1,5 @@
 
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -10,11 +10,7 @@
 #include <stdio.h>
 #include <assert.h>
 
-#if _WIN32 || IN_GCC
-#include "mem.h"
-#else
-#include "../root/mem.h"
-#endif
+#include "rmem.h"
 
 #include "expression.h"
 #include "mtype.h"
@@ -33,20 +29,45 @@ Expression *Expression::implicitCastTo(Scope *sc, Type *t)
 {
     //printf("implicitCastTo(%s) => %s\n", type->toChars(), t->toChars());
     if (implicitConvTo(t))
-    {
+    {	TY tyfrom = type->toBasetype()->ty;
+ 	TY tyto = t->toBasetype()->ty;
 	if (global.params.warnings &&
-	    Type::impcnvWarn[type->toBasetype()->ty][t->toBasetype()->ty] &&
+	    Type::impcnvWarn[tyfrom][tyto] &&
 	    op != TOKint64)
 	{
 	    Expression *e = optimize(WANTflags | WANTvalue);
 
 	    if (e->op == TOKint64)
 		return e->implicitCastTo(sc, t);
+		
+		if (tyfrom == Tint32 &&
+ 		(op == TOKadd || op == TOKmin ||
+ 		 op == TOKand || op == TOKor || op == TOKxor)
+ 	       )
+ 	    {
+ 		/* This is really only a semi-kludge fix,
++ 		 * we really should look at the operands of op
++ 		 * and see if they are narrower types.
++ 		 * For example, b=b|b and b=b|7 and s=b+b should be allowed,
++ 		 * but b=b|i should be an error.
++ 		 */
+ 		;
+ 	    }
+ 	    else
+ 	    {
 
-	    fprintf(stdmsg, "warning - ");
-	    error("implicit conversion of expression (%s) of type %s to %s can cause loss of data",
+	    warning("implicit conversion of expression (%s) of type %s to %s can cause loss of data",
 		toChars(), type->toChars(), t->toChars());
+		}
 	}
+	#if DMDV2
+ 	if (match == MATCHconst && t == type->constOf())
+ 	{
+ 	    Expression *e = copy();
+ 	    e->type = t;
+ 	    return e;
+ 	}
+ 	#endif
 	return castTo(sc, t);
     }
 
@@ -55,12 +76,13 @@ Expression *Expression::implicitCastTo(Scope *sc, Type *t)
 	return e->implicitCastTo(sc, t);
 
 #if 0
+printf("ty = %d\n", type->ty);
 print();
 type->print();
 printf("to:\n");
 t->print();
 printf("%p %p type: %s to: %s\n", type->deco, t->deco, type->deco, t->deco);
-//printf("%p %p %p\n", type->next->arrayOf(), type, t);
+//printf("%p %p %p\n", type->nextOf()->arrayOf(), type, t);
 fflush(stdout);
 #endif
     if (!t->deco)
@@ -93,6 +115,7 @@ MATCH Expression::implicitConvTo(Type *t)
     printf("Expression::implicitConvTo(this=%s, type=%s, t=%s)\n",
 	toChars(), type->toChars(), t->toChars());
 #endif
+//static int nest; if (++nest == 10) halt();
     if (!type)
     {	error("%s is not an expression", toChars());
 	type = Type::terror;
@@ -111,16 +134,16 @@ MATCH Expression::implicitConvTo(Type *t)
     Type *tb = t->toBasetype();
     if (tb->ty == Tdelegate)
     {	TypeDelegate *td = (TypeDelegate *)tb;
-	TypeFunction *tf = (TypeFunction *)td->next;
+	TypeFunction *tf = (TypeFunction *)td->nextOf();
 
 	if (!tf->varargs &&
 	    !(tf->arguments && tf->arguments->dim)
 	   )
 	{
-	    match = type->implicitConvTo(tf->next);
+	    match = type->implicitConvTo(tf->nextOf());
 	    if (match)
 		return match;
-	    if (tf->next->toBasetype()->ty == Tvoid)
+	    if (tf->nextOf()->toBasetype()->ty == Tvoid)
 		return MATCHconvert;
 	}
     }
@@ -206,7 +229,7 @@ MATCH IntegerExp::implicitConvTo(Type *t)
 
 	case Tchar:
 	case Tuns8:
-	    //printf("value = %llu %llu\n", (integer_t)(unsigned char)value, value);
+	    //printf("value = %llu %llu\n", (dinteger_t)(unsigned char)value, value);
 	    if ((unsigned char)value != value)
 		goto Lno;
 	    goto Lyes;
@@ -346,8 +369,8 @@ Lno:
 MATCH NullExp::implicitConvTo(Type *t)
 {
 #if 0
-    printf("NullExp::implicitConvTo(this=%s, type=%s, t=%s)\n",
-	toChars(), type->toChars(), t->toChars());
+    printf("NullExp::implicitConvTo(this=%s, type=%s, t=%s, committed = %d)\n",
+ 	toChars(), type->toChars(), t->toChars(), committed);
 #endif
     if (this->type->equals(t))
 	return MATCHexact;
@@ -364,6 +387,39 @@ MATCH NullExp::implicitConvTo(Type *t)
     return Expression::implicitConvTo(t);
 }
 
+#if DMDV2
+ MATCH StructLiteralExp::implicitConvTo(Type *t)
+ {
+ #if 0
+     printf("StructLiteralExp::implicitConvTo(this=%s, type=%s, t=%s)\n",
+ 	toChars(), type->toChars(), t->toChars());
+ #endif
+     MATCH m = Expression::implicitConvTo(t);
+     if (m != MATCHnomatch)
+ 	return m;
+     if (type->ty == t->ty && type->ty == Tstruct &&
+ 	((TypeStruct *)type)->sym == ((TypeStruct *)t)->sym)
+     {
+ 	m = MATCHconst;
+ 	for (int i = 0; i < elements->dim; i++)
+ 	{   Expression *e = (Expression *)elements->data[i];
+ 	    Type *te = e->type;
+ 	    if (t->mod == 0)
+ 		te = te->mutableOf();
+ 	    else
+ 	    {	assert(t->mod == MODinvariant);
+ 		te = te->invariantOf();
+ 	    }
+ 	    MATCH m2 = e->implicitConvTo(te);
+ 	    //printf("\t%s => %s, match = %d\n", e->toChars(), te->toChars(), m2);
+ 	    if (m2 < m)
+ 		m = m2;
+ 	}
+     }
+     return m;
+ }
+ #endif //V2
+
 MATCH StringExp::implicitConvTo(Type *t)
 {   MATCH m;
 
@@ -419,6 +475,11 @@ MATCH StringExp::implicitConvTo(Type *t)
 MATCH ArrayLiteralExp::implicitConvTo(Type *t)
 {   MATCH result = MATCHexact;
 
+#if 0
+     printf("ArrayLiteralExp::implicitConvTo(this=%s, type=%s, t=%s)\n",
+ 	toChars(), type->toChars(), t->toChars());
+ #endif
+
     Type *typeb = type->toBasetype();
     Type *tb = t->toBasetype();
     if ((tb->ty == Tarray || tb->ty == Tsarray) &&
@@ -432,7 +493,7 @@ MATCH ArrayLiteralExp::implicitConvTo(Type *t)
 
 	for (int i = 0; i < elements->dim; i++)
 	{   Expression *e = (Expression *)elements->data[i];
-	    MATCH m = (MATCH)e->implicitConvTo(tb->next);
+	    MATCH m = (MATCH)e->implicitConvTo(tb->nextOf());
 	    if (m < result)
 		result = m;			// remember worst match
 	    if (result == MATCHnomatch)
@@ -459,7 +520,7 @@ MATCH AssocArrayLiteralExp::implicitConvTo(Type *t)
 	    if (result == MATCHnomatch)
 		break;				// no need to check for worse
 	    e = (Expression *)values->data[i];
-	    m = (MATCH)e->implicitConvTo(tb->next);
+	    m = (MATCH)e->implicitConvTo(tb->nextOf());
 	    if (m < result)
 		result = m;			// remember worst match
 	    if (result == MATCHnomatch)
@@ -548,10 +609,10 @@ MATCH DelegateExp::implicitConvTo(Type *t)
 	FuncDeclaration *f;
 
 	t = t->toBasetype();
-	if (type->ty == Tdelegate && type->next->ty == Tfunction &&
-	    t->ty == Tdelegate && t->next->ty == Tfunction)
-	{
-	    if (func && func->overloadExactMatch(t->next))
+	if (type->ty == Tdelegate && type->nextOf()->ty == Tfunction &&
+ 	    t->ty == Tdelegate && t->nextOf()->ty == Tfunction)
+  	{
+ 	    if (func && func->overloadExactMatch(t->nextOf()))
 		result = MATCHexact;
 	}
     }
@@ -980,7 +1041,7 @@ Expression *TupleExp::castTo(Scope *sc, Type *t)
 Expression *ArrayLiteralExp::castTo(Scope *sc, Type *t)
 {
 #if 0
-    printf("ArrayLiteralExp::castTo(this=%s, type=%s, t=%s)\n",
+    printf("ArrayLiteralExp::castTo(this=%s, type=%s, => %s)\n",
 	toChars(), type->toChars(), t->toChars());
 #endif
     if (type == t)
@@ -1051,22 +1112,20 @@ L1:
 
 Expression *SymOffExp::castTo(Scope *sc, Type *t)
 {
-    Type *tb;
-
 #if 0
     printf("SymOffExp::castTo(this=%s, type=%s, t=%s)\n",
 	toChars(), type->toChars(), t->toChars());
 #endif
     Expression *e = this;
 
-    tb = t->toBasetype();
-    type = type->toBasetype();
-    if (tb != type)
+    Type *tb = t->toBasetype();
+    Type *typeb = type->toBasetype();
+    if (tb != typeb)
     {
 	// Look for pointers to functions where the functions are overloaded.
 	FuncDeclaration *f;
 
-	if (type->ty == Tpointer && type->next->ty == Tfunction &&
+	if (typeb->ty == Tpointer && typeb->next->ty == Tfunction &&
 	    tb->ty == Tpointer && tb->next->ty == Tfunction)
 	{
 	    f = var->isFuncDeclaration();
@@ -1075,15 +1134,47 @@ Expression *SymOffExp::castTo(Scope *sc, Type *t)
 		f = f->overloadExactMatch(tb->next);
 		if (f)
 		{
+			#if DMDV2
+		    if (tb->ty == Tdelegate)
+		    {
+			if (f->needThis() && hasThis(sc))
+			{
+			    e = new DelegateExp(loc, new ThisExp(loc), f);
+			    e = e->semantic(sc);
+			}
+			else if (f->isNested())
+			{
+			    e = new DelegateExp(loc, new IntegerExp(0), f);
+			    e = e->semantic(sc);
+			}
+			else if (f->needThis())
+			{   error("no 'this' to create delegate for %s", f->toChars());
+			    e = new ErrorExp();
+			}
+			else
+			{   error("cannot cast from function pointer to delegate");
+			    e = new ErrorExp();
+			}
+		    }
+		    else
+			#endif
+		    {
 		    e = new SymOffExp(loc, f, 0);
 		    e->type = t;
+		    }
+			#if DMDV2
+		    f->tookAddressOf++;
+			#endif
 		    return e;
 		}
 	    }
 	}
 	e = Expression::castTo(sc, t);
     }
+    else
+    {
     e->type = t;
+    }
     return e;
 }
 
@@ -1166,7 +1257,7 @@ Expression *BinExp::scaleFactor(Scope *sc)
 	// Replace (ptr + int) with (ptr + (int * stride))
 	Type *t = Type::tptrdiff_t;
 
-	stride = t1b->next->size();
+	stride = t1b->nextOf()->size(loc);
 	if (!t->equals(t2b))
 	    e2 = e2->castTo(sc, t);
 	if (t1b->next->isbit())
@@ -1179,13 +1270,13 @@ Expression *BinExp::scaleFactor(Scope *sc)
 	e2->type = t;
 	type = e1->type;
     }
-    else if (t2b->ty && t1b->isintegral())
+    else if (t2b->ty == Tpointer && t1b->isintegral())
     {   // Need to adjust operator by the stride
 	// Replace (int + ptr) with (ptr + (int * stride))
 	Type *t = Type::tptrdiff_t;
 	Expression *e;
 
-	stride = t2b->next->size();
+	stride = t2b->nextOf()->size(loc);
 	if (!t->equals(t1b))
 	    e = e1->castTo(sc, t);
 	else
@@ -1354,13 +1445,17 @@ Expression *BinExp::typeCombine(Scope *sc)
 	    goto Lincompatible;
     }
     else if ((t1->ty == Tsarray || t1->ty == Tarray) &&
-	     e2->op == TOKnull && t2->ty == Tpointer && t2->next->ty == Tvoid)
-    {
+	     e2->op == TOKnull && t2->ty == Tpointer && t2->nextOf()->ty == Tvoid)
+     {	/*  (T[n] op void*)
+! 	 *  (T[] op void*)
+! 	 */
 	goto Lx1;
     }
     else if ((t2->ty == Tsarray || t2->ty == Tarray) &&
-	     e1->op == TOKnull && t1->ty == Tpointer && t1->next->ty == Tvoid)
-    {
+	     e1->op == TOKnull && t1->ty == Tpointer && t1->nextOf()->ty == Tvoid)
+     {	/*  (void* op T[n])
+! 	 *  (void* op T[])
+! 	 */
 	goto Lx2;
     }
     else if ((t1->ty == Tsarray || t1->ty == Tarray) && t1->implicitConvTo(t2))
@@ -1402,7 +1497,7 @@ Expression *BinExp::typeCombine(Scope *sc)
 		/* Pick 'tightest' type
 		 */
 		ClassDeclaration *cd1 = tc1->sym->baseClass;
-		ClassDeclaration *cd2 = tc1->sym->baseClass;
+		ClassDeclaration *cd2 = tc2->sym->baseClass;
 
 		if (cd1 && cd2)
 		{   t1 = cd1->type;
@@ -1429,25 +1524,55 @@ Expression *BinExp::typeCombine(Scope *sc)
 	goto Lt1;
     }
     else if (t1->ty == Tsarray && t2->ty == Tsarray &&
-	     e2->implicitConvTo(t1->next->arrayOf()))
+	     e2->implicitConvTo(t1->nextOf()->arrayOf()))
     {
      Lx1:
-	t = t1->next->arrayOf();
+	t = t1->nextOf()->arrayOf();
 	e1 = e1->castTo(sc, t);
 	e2 = e2->castTo(sc, t);
     }
     else if (t1->ty == Tsarray && t2->ty == Tsarray &&
-	     e1->implicitConvTo(t2->next->arrayOf()))
+	     e1->implicitConvTo(t2->nextOf()->arrayOf()))
     {
      Lx2:
-	t = t2->next->arrayOf();
+	t = t2->nextOf()->arrayOf();
 	e1 = e1->castTo(sc, t);
 	e2 = e2->castTo(sc, t);
     }
+    else if (t1->isintegral() && t2->isintegral())
+     {
+ 	assert(0);
+     }
+     else if (e1->op == TOKslice && t1->ty == Tarray &&
+ 	     e2->implicitConvTo(t1->nextOf()))
+     {	// T[] op T
+ 	e2 = e2->castTo(sc, t1->nextOf());
+ 	t = t1->nextOf()->arrayOf();
+     }
+     else if (e2->op == TOKslice && t2->ty == Tarray &&
+ 	     e1->implicitConvTo(t2->nextOf()))
+     {	// T op T[]
+ 	e1 = e1->castTo(sc, t2->nextOf());
+ 	t = t2->nextOf()->arrayOf();
+ 
+ 	//printf("test %s\n", e->toChars());
+ 	e1 = e1->optimize(WANTvalue);
+ 	if (isCommutative() && e1->isConst())
+ 	{   /* Swap operands to minimize number of functions generated
+ 	     */
+ 	    //printf("swap %s\n", e->toChars());
+ 	    Expression *tmp = e1;
+ 	    e1 = e2;
+ 	    e2 = tmp;
+ 	}
+     }
     else
     {
      Lincompatible:
 	incompatibleTypes();
+	type = Type::terror;
+ 	e1 = new ErrorExp();
+ 	e2 = new ErrorExp();
     }
 Lret:
     if (!type)
@@ -1473,9 +1598,10 @@ Lt2:
  */
 
 Expression *Expression::integralPromotions(Scope *sc)
-{   Expression *e;
-
-    e = this;
+{
+     Expression *e = this;
+  
+     //printf("integralPromotions %s %s\n", e->toChars(), e->type->toChars());
     switch (type->toBasetype()->ty)
     {
 	case Tvoid:
