@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -11,7 +11,8 @@
 /* NOTE: This file has been patched from the original DMD distribution to
    work with the GDC compiler.
 
-   Modified by David Friedman, December 2006
+   Modified by Michael Parrott, September 2009
+   Using David Friedman's code
 */
 
 #include <stdio.h>
@@ -19,7 +20,7 @@
 #include <assert.h>
 
 #include "root.h"
-#include "mem.h"
+#include "rmem.h"
 
 #include "enum.h"
 #include "init.h"
@@ -153,7 +154,7 @@ ClassDeclaration::ClassDeclaration(Loc loc, Identifier *id, BaseClasses *basecla
 		Type::typeinfotypelist = this;
 	    }
 
-#if V2
+#if DMDV2
 	    if (id == Id::TypeInfo_Const)
 	    {	if (Type::typeinfoconst)
 		    Type::typeinfoconst->error("%s", msg);
@@ -229,19 +230,18 @@ void ClassDeclaration::semantic(Scope *sc)
     //{ static int n;  if (++n == 20) *(char*)0=0; }
 
     if (!ident)		// if anonymous class
-    {	char *id = "__anonclass";
+    {	const char *id = "__anonclass";
 
 	ident = Identifier::generateId(id);
     }
 
-    if (!scope)
-    {
+    if (!sc)
+ 	sc = scope;
 	if (!parent && sc->parent && !sc->parent->isModule())
 	    parent = sc->parent;
 
 	type = type->semantic(loc, sc);
-	handle = handle->semantic(loc, sc);
-    }
+	handle = type;
     if (!members)			// if forward reference
     {	//printf("\tclass '%s' is forward referenced\n", toChars());
 	return;
@@ -336,6 +336,11 @@ void ClassDeclaration::semantic(Scope *sc)
 			goto L7;
 		    }
 		}
+		if (!tc->sym->symtab || tc->sym->sizeok == 0)
+ 		{   // Try to resolve forward reference
+ 		    if (sc->mustsemantic && tc->sym->scope)
+ 			tc->sym->semantic(NULL);
+ 		}
 		if (!tc->sym->symtab || tc->sym->scope || tc->sym->sizeok == 0)
 		{
 		    //error("forward reference of base class %s", baseClass->toChars());
@@ -486,24 +491,36 @@ void ClassDeclaration::semantic(Scope *sc)
 	    if (storage_class & STCstatic)
 		error("static class cannot inherit from nested class %s", baseClass->toChars());
 	    if (toParent2() != baseClass->toParent2())
-		error("super class %s is nested within %s, not %s",
-			baseClass->toChars(),
-			baseClass->toParent2()->toChars(),
-			toParent2()->toChars());
+		{
+ 		if (toParent2())
+ 		{
+ 		    error("is nested within %s, but super class %s is nested within %s",
+ 			toParent2()->toChars(),
+ 			baseClass->toChars(),
+ 			baseClass->toParent2()->toChars());
+ 		}
+ 		else
+ 		{
+ 		    error("is not nested, but super class %s is nested within %s",
+  			baseClass->toChars(),
+ 			baseClass->toParent2()->toChars());
+ 		}
+ 		isnested = 0;
+ 	    }
 	}
 	else if (!(storage_class & STCstatic))
 	{   Dsymbol *s = toParent2();
 	    if (s)
 	    {
-		ClassDeclaration *cd = s->isClassDeclaration();
+		AggregateDeclaration *ad = s->isClassDeclaration();
 		FuncDeclaration *fd = s->isFuncDeclaration();
 
 
-		if (cd || fd)
+		if (ad || fd)
 		{   isnested = 1;
 		    Type *t;
-		    if (cd)
-			t = cd->type;
+		    if (ad)
+			t = ad->handle;
 		    else if (fd)
 		    {	AggregateDeclaration *ad = fd->isMember2();
 			if (ad)
@@ -517,7 +534,7 @@ void ClassDeclaration::semantic(Scope *sc)
 		    else
 			assert(0);
 		    assert(!vthis);
-		    vthis = new ThisDeclaration(t);
+		    vthis = new ThisDeclaration(loc, t);
 		    members->push(vthis);
 		}
 	    }
@@ -537,7 +554,16 @@ void ClassDeclaration::semantic(Scope *sc)
     sc->inunion = 0;
 
     if (isCOMclass())
-	sc->linkage = LINKwindows;
+	{
+ #if _WIN32
+  	sc->linkage = LINKwindows;
+ #else
+ 	/* This enables us to use COM objects under Linux and
++ 	 * work with things like XPCOM
++ 	 */
+ 	sc->linkage = LINKc;
+ #endif
+     }
     sc->protection = PROTpublic;
     sc->explicitProtection = 0;
     sc->structalign = 8;
@@ -549,7 +575,7 @@ void ClassDeclaration::semantic(Scope *sc)
 //	    sc->offset += PTRSIZE;	// room for uplevel context pointer
     }
     else
-    {	sc->offset = PTRSIZE * 2;	// allow room for vptr[] and monitor
+    {	sc->offset = PTRSIZE * 2;	// allow room for __vptr and __monitor
 	alignsize = PTRSIZE;
     }
     structsize = sc->offset;
@@ -589,7 +615,7 @@ void ClassDeclaration::semantic(Scope *sc)
      * They must be in this class, not in a base class.
      */
     ctor = (CtorDeclaration *)search(0, Id::ctor, 0);
-    if (ctor && ctor->toParent() != this)
+    if (ctor && (ctor->toParent() != this || !ctor->isCtorDeclaration()))
 	ctor = NULL;
 
 //    dtor = (DtorDeclaration *)search(Id::dtor, 0);
@@ -610,7 +636,7 @@ void ClassDeclaration::semantic(Scope *sc)
     if (!ctor && baseClass && baseClass->ctor)
     {
 	//printf("Creating default this(){} for class %s\n", toChars());
-	ctor = new CtorDeclaration(0, 0, NULL, 0);
+	ctor = new CtorDeclaration(loc, 0, NULL, 0);
 	ctor->fbody = new CompoundStatement(0, new Statements());
 	members->push(ctor);
 	ctor->addMember(sc, this, 1);
@@ -765,7 +791,11 @@ Dsymbol *ClassDeclaration::search(Loc loc, Identifier *ident, int flags)
 
     //printf("%s.ClassDeclaration::search('%s')\n", toChars(), ident->toChars());
     if (scope)
-	semantic(scope);
+	{	Scope *sc = scope;
+	sc->mustsemantic++;
+	semantic(sc);
+	sc->mustsemantic--;
+    }
 
     if (!members || !symtab || scope)
     {	error("is forward referenced when looking for '%s'", ident->toChars());
@@ -807,7 +837,7 @@ Dsymbol *ClassDeclaration::search(Loc loc, Identifier *ident, int flags)
  * Return 1 if function is hidden (not findable through search).
  */
 
-#if V2
+#if DMDV2
 int isf(void *param, FuncDeclaration *fd)
 {
     //printf("param = %p, fd = %p %s\n", param, fd, fd->toChars());
@@ -816,7 +846,7 @@ int isf(void *param, FuncDeclaration *fd)
 
 int ClassDeclaration::isFuncHidden(FuncDeclaration *fd)
 {
-    //printf("ClassDeclaration::isFuncHidden(%s)\n", fd->toChars());
+    //printf("ClassDeclaration::isFuncHidden(class = %s, fd = %s)\n", toChars(), fd->toChars());
     Dsymbol *s = search(0, fd->ident, 4|2);
     if (!s)
     {	//printf("not found\n");
@@ -846,7 +876,9 @@ FuncDeclaration *ClassDeclaration::findFunc(Identifier *ident, TypeFunction *tf)
     {
 	for (size_t i = 0; i < vtbl->dim; i++)
 	{
-	    FuncDeclaration *fd = (FuncDeclaration *)vtbl->data[i];
+	    FuncDeclaration *fd = ((Dsymbol*)vtbl->data[i])->isFuncDeclaration();
+ 	    if (!fd)
+ 		continue;		// the first entry might be a ClassInfo
 
 	    //printf("\t[%d] = %s\n", i, fd->toChars());
 	    if (ident == fd->ident &&
@@ -947,7 +979,7 @@ int ClassDeclaration::vtblOffset()
 /****************************************
  */
 
-char *ClassDeclaration::kind()
+const char *ClassDeclaration::kind()
 {
     return "class";
 }
@@ -989,10 +1021,13 @@ void InterfaceDeclaration::semantic(Scope *sc)
     //printf("InterfaceDeclaration::semantic(%s), type = %p\n", toChars(), type);
     if (inuse)
 	return;
-    if (!scope)
-    {	type = type->semantic(loc, sc);
-	handle = handle->semantic(loc, sc);
-    }
+    if (!sc)
+	sc = scope;
+    if (!parent && sc->parent && !sc->parent->isModule())
+	parent = sc->parent;
+
+    type = type->semantic(loc, sc);
+    handle = type;
     if (!members)			// if forward reference
     {	//printf("\tinterface '%s' is forward referenced\n", toChars());
 	return;
@@ -1251,7 +1286,7 @@ int InterfaceDeclaration::isCOMinterface()
 /*******************************************
  */
 
-char *InterfaceDeclaration::kind()
+const char *InterfaceDeclaration::kind()
 {
     return "interface";
 }
