@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -11,7 +11,8 @@
 /* NOTE: This file has been patched from the original DMD distribution to
    work with the GDC compiler.
 
-   Modified by David Friedman, September 2004
+   Modified by Michael Parrott, September 2009
+   Using David Friedman's code
 */
 
 #include <stdio.h>
@@ -23,8 +24,9 @@
 #include <complex.h>
 #endif
 
-#include "mem.h"
+#include "rmem.h"
 #include "root.h"
+#include "port.h"
 
 #include "mtype.h"
 #include "expression.h"
@@ -37,6 +39,10 @@
 
 static real_t zero;	// work around DMC bug for now
 
+#if __FreeBSD__
+#define fmodl fmod	// hack for now, fix later
+#endif
+
 #define LOG 0
 
 Expression *expType(Type *type, Expression *e)
@@ -463,7 +469,7 @@ Expression *Div(Type *type, Expression *e1, Expression *e2)
 	n2 = e2->toInteger();
 	if (n2 == 0)
 	{   e2->error("divide by 0");
-	    e2 = new IntegerExp(0, 1, e2->type);
+	    e2 = new IntegerExp(loc, 1, e2->type);
 	    n2 = 1;
 	}
 	if (e1->type->isunsigned() || e2->type->isunsigned())
@@ -526,7 +532,7 @@ Expression *Mod(Type *type, Expression *e1, Expression *e2)
 	n2 = e2->toInteger();
 	if (n2 == 0)
 	{   e2->error("divide by 0");
-	    e2 = new IntegerExp(0, 1, e2->type);
+	    e2 = new IntegerExp(loc, 1, e2->type);
 	    n2 = 1;
 	}
 	if (e1->type->isunsigned() || e2->type->isunsigned())
@@ -550,7 +556,7 @@ Expression *Shr(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
     Loc loc = e1->loc;
     unsigned count;
-    integer_t value;
+    dinteger_t value;
 
     value = e1->toInteger();
     count = e2->toInteger();
@@ -599,7 +605,7 @@ Expression *Ushr(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
     Loc loc = e1->loc;
     unsigned count;
-    integer_t value;
+    dinteger_t value;
 
     value = e1->toInteger();
     count = e2->toInteger();
@@ -635,26 +641,21 @@ Expression *Ushr(Type *type, Expression *e1, Expression *e2)
 }
 
 Expression *And(Type *type, Expression *e1, Expression *e2)
-{   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() & e2->toInteger(), type);
+{
+    Expression *e;
+    e = new IntegerExp(e1->loc, e1->toInteger() & e2->toInteger(), type);
     return e;
 }
 
 Expression *Or(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() | e2->toInteger(), type);
+    e = new IntegerExp(e1->loc, e1->toInteger() | e2->toInteger(), type);
     return e;
 }
 
 Expression *Xor(Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
-    Loc loc = e1->loc;
-
-    e = new IntegerExp(loc, e1->toInteger() ^ e2->toInteger(), type);
+    e = new IntegerExp(e1->loc, e1->toInteger() ^ e2->toInteger(), type);
     return e;
 }
 
@@ -703,7 +704,11 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
     {	StringExp *es1 = (StringExp *)e1;
 	StringExp *es2 = (StringExp *)e2;
 
-	assert(es1->sz == es2->sz);
+	if (es1->sz != es2->sz)
+ 	{
+ 	    assert(global.errors);
+ 	    return EXP_CANT_INTERPRET;
+ 	}
 	if (es1->len == es2->len &&
 	    memcmp(es1->string, es2->string, es1->sz * es1->len) == 0)
 	    cmp = 1;
@@ -825,7 +830,7 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
 # if IN_GCC
 	if (r1.isNan() || r2.isNan())	// if unordered
 # else
-	if (isnan(r1) || isnan(r2))	// if unordered
+	if (Port::isNan(r1) || Port::isNan(r2))	// if unordered
 # endif
 	{
 	    cmp = 0;
@@ -857,9 +862,13 @@ Expression *Identity(enum TOK op, Type *type, Expression *e1, Expression *e2)
     Loc loc = e1->loc;
     int cmp;
 
-    if (e1->op == TOKnull && e2->op == TOKnull)
+    if (e1->op == TOKnull)
     {
-	cmp = 1;
+	cmp = (e2->op == TOKnull);
+    }
+    else if (e2->op == TOKnull)
+    {
+	cmp = 0;
     }
     else if (e1->op == TOKsymoff && e2->op == TOKsymoff)
     {
@@ -882,7 +891,7 @@ Expression *Identity(enum TOK op, Type *type, Expression *e1, Expression *e2)
 Expression *Cmp(enum TOK op, Type *type, Expression *e1, Expression *e2)
 {   Expression *e;
     Loc loc = e1->loc;
-    integer_t n;
+    dinteger_t n;
     real_t r1;
     real_t r2;
 
@@ -926,6 +935,7 @@ Expression *Cmp(enum TOK op, Type *type, Expression *e1, Expression *e2)
 #else
 	if (isnan(r1) || isnan(r2))	// if unordered
 #endif
+	//if (Port::isNan(r1) || Port::isNan(r2))	// if unordered
 	{
 	    switch (op)
 	    {
@@ -1053,7 +1063,7 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
     else if (type->isintegral())
     {
 	if (e1->type->isfloating())
-	{   integer_t result;
+	{   dinteger_t result;
 #ifdef IN_GCC
 	    Type * rt = e1->type;
 	    if (rt->iscomplex())
@@ -1136,7 +1146,7 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
     else
     {
 	error(loc, "cannot cast %s to %s", e1->type->toChars(), type->toChars());
-	e = new IntegerExp(loc, 0, type);
+	e = new IntegerExp(loc, 0, Type::tint32);
     }
     return e;
 }
@@ -1261,7 +1271,7 @@ Expression *Slice(Type *type, Expression *e1, Expression *lwr, Expression *upr)
 	if (iupr > es1->len || ilwr > iupr)
 	    e1->error("string slice [%"PRIuMAX" .. %"PRIuMAX"] is out of bounds", ilwr, iupr);
 	else
-	{   integer_t value;
+	{   dinteger_t value;
 	    void *s;
 	    size_t len = iupr - ilwr;
 	    int sz = es1->sz;
@@ -1307,8 +1317,11 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 {   Expression *e = EXP_CANT_INTERPRET;
     Loc loc = e1->loc;
     Type *t;
+    Type *t1 = e1->type->toBasetype();
+    Type *t2 = e2->type->toBasetype();
 
     //printf("Cat(e1 = %s, e2 = %s)\n", e1->toChars(), e2->toChars());
+    //printf("\tt1 = %s, t2 = %s\n", t1->toChars(), t2->toChars())
 
     if (e1->op == TOKnull && (e2->op == TOKint64 || e2->op == TOKstructliteral))
     {	e = e2;
@@ -1325,7 +1338,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	    StringExp *es;
 	    size_t len = 1;
 	    int sz = tn->size();
-	    integer_t v = e->toInteger();
+	    dinteger_t v = e->toInteger();
 
 	    s = mem.malloc((len + 1) * sz);
 	    switch (sz)
@@ -1364,7 +1377,14 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	size_t len = es1->len + es2->len;
 	int sz = es1->sz;
 
-	assert(sz == es2->sz);
+	if (sz != es2->sz)
+ 	{
+ 	    /* Can happen with:
+! 	     *   auto s = "foo"d ~ "bar"c;
+! 	     */
+ 	    assert(global.errors);
+ 	    return e;
+ 	}
 	s = mem.malloc((len + 1) * sz);
 	memcpy(s, es1->string, es1->len * sz);
 	memcpy((unsigned char *)s + es1->len * sz, es2->string, es2->len * sz);
@@ -1392,7 +1412,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	Type *t;
 	size_t len = es1->len + 1;
 	int sz = es1->sz;
-	integer_t v = e2->toInteger();
+	dinteger_t v = e2->toInteger();
 
 	s = mem.malloc((len + 1) * sz);
 	memcpy(s, es1->string, es1->len * sz);
@@ -1424,7 +1444,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	Type *t;
 	size_t len = 1 + es2->len;
 	int sz = es2->sz;
-	integer_t v = e1->toInteger();
+	dinteger_t v = e1->toInteger();
 
 	s = mem.malloc((len + 1) * sz);
 	switch (sz)
@@ -1465,18 +1485,50 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 	else
 	    e->type = type;
     }
-    else if (e1->op == TOKarrayliteral &&
+    else if (e1->op == TOKarrayliteral && e2->op == TOKnull &&
+ 	t1->nextOf()->equals(t2->nextOf()))
+     {
+ 	e = e1;
+ 	goto L3;
+     }
+     else if (e1->op == TOKnull && e2->op == TOKarrayliteral &&
+ 	t1->nextOf()->equals(t2->nextOf()))
+     {
+ 	e = e2;
+      L3:
+ 	// Concatenate the array with null
+ 	ArrayLiteralExp *es = (ArrayLiteralExp *)e;
+ 
+ 	es = new ArrayLiteralExp(es->loc, (Expressions *)es->elements->copy());
+ 	e = es;
+ 
+ 	if (type->toBasetype()->ty == Tsarray)
+ 	{
+	    e->type = new TypeSArray(t1->nextOf(), new IntegerExp(loc, es->elements->dim, Type::tindex));
+ 	    e->type = e->type->semantic(loc, NULL);
+ 	}
+ 	else
+ 	    e->type = type;
+     }
+    else if ((e1->op == TOKarrayliteral || e1->op == TOKnull) &&
 	e1->type->toBasetype()->nextOf()->equals(e2->type))
     {
-	ArrayLiteralExp *es1 = (ArrayLiteralExp *)e1;
+	ArrayLiteralExp *es1;
+ 	if (e1->op == TOKarrayliteral)
+ 	{   es1 = (ArrayLiteralExp *)e1;
 
 	es1 = new ArrayLiteralExp(es1->loc, (Expressions *)es1->elements->copy());
 	es1->elements->push(e2);
+	}
+ 	else
+ 	{
+ 	    es1 = new ArrayLiteralExp(e1->loc, e2);
+ 	}
 	e = es1;
 
 	if (type->toBasetype()->ty == Tsarray)
 	{
-	    e->type = new TypeSArray(e2->type, new IntegerExp(0, es1->elements->dim, Type::tindex));
+	    e->type = new TypeSArray(e2->type, new IntegerExp(loc, es1->elements->dim, Type::tindex));
 	    e->type = e->type->semantic(loc, NULL);
 	}
 	else
@@ -1493,7 +1545,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 
 	if (type->toBasetype()->ty == Tsarray)
 	{
-	    e->type = new TypeSArray(e1->type, new IntegerExp(0, es2->elements->dim, Type::tindex));
+	    e->type = new TypeSArray(e1->type, new IntegerExp(loc, es2->elements->dim, Type::tindex));
 	    e->type = e->type->semantic(loc, NULL);
 	}
 	else
