@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -11,7 +11,8 @@
 /* NOTE: This file has been patched from the original DMD distribution to
    work with the GDC compiler.
 
-   Modified by David Friedman, December 2006
+   Modified by Michael Parrott, September 2009
+   Using David Friedman's code
 */
 
 #include <stdio.h>
@@ -40,13 +41,14 @@ Declaration::Declaration(Identifier *id)
     protection = PROTundefined;
     linkage = LINKdefault;
     attributes = NULL;
+    inuse = 0;
 }
 
 void Declaration::semantic(Scope *sc)
 {
 }
 
-char *Declaration::kind()
+const char *Declaration::kind()
 {
     return "declaration";
 }
@@ -77,6 +79,11 @@ int Declaration::isDataseg()
     return FALSE;
 }
 
+int Declaration::isThreadlocal()
+{
+    return FALSE;
+}
+
 int Declaration::isCodeseg()
 {
     return FALSE;
@@ -87,6 +94,76 @@ enum PROT Declaration::prot()
     return protection;
 }
 
+/*************************************
++  * Check to see if declaration can be modified in this context (sc).
++  * Issue error if not.
++  */
+ 
+ #if DMDV2
+ void Declaration::checkModify(Loc loc, Scope *sc, Type *t)
+ {
+     if (sc->incontract && isParameter())
+ 	error(loc, "cannot modify parameter '%s' in contract", toChars());
+ 
+     if (isCtorinit())
+     {	// It's only modifiable if inside the right constructor
+ 	Dsymbol *s = sc->func;
+ 	while (1)
+ 	{
+ 	    FuncDeclaration *fd = NULL;
+ 	    if (s)
+ 		fd = s->isFuncDeclaration();
+ 	    if (fd &&
+ 		((fd->isCtorDeclaration() && storage_class & STCfield) ||
+ 		 (fd->isStaticCtorDeclaration() && !(storage_class & STCfield))) &&
+ 		fd->toParent() == toParent()
+ 	       )
+ 	    {
+ 		VarDeclaration *v = isVarDeclaration();
+ 		assert(v);
+ 		v->ctorinit = 1;
+ 		//printf("setting ctorinit\n");
+ 	    }
+ 	    else
+ 	    {
+ 		if (s)
+ 		{   s = s->toParent2();
+ 		    continue;
+ 		}
+ 		else
+		{
+ 		    const char *p = isStatic() ? "static " : "";
+ 		    error(loc, "can only initialize %sconst %s inside %sconstructor",
+ 			p, toChars(), p);
+ 		}
+ 	    }
+ 	    break;
+ 	}
+     }
+     else
+     {
+ 	VarDeclaration *v = isVarDeclaration();
+ 	if (v && v->canassign == 0)
+ 	{
+ 	    const char *p = NULL;
+ 	    if (isConst())
+ 		p = "const";
+ 	    else if (isInvariant())
+ 		p = "mutable";
+ 	    else if (storage_class & STCmanifest)
+ 		p = "enum";
+ 	    else if (!t->isAssignable())
+ 		p = "struct with immutable members";
+ 	    if (p)
+ 	    {	error(loc, "cannot modify %s", p);
+ 	    }
+ 	}
+     }
+ }
+ #endif
+ 
+
+
 /********************************* TupleDeclaration ****************************/
 
 TupleDeclaration::TupleDeclaration(Loc loc, Identifier *id, Objects *objects)
@@ -104,7 +181,7 @@ Dsymbol *TupleDeclaration::syntaxCopy(Dsymbol *s)
     return NULL;
 }
 
-char *TupleDeclaration::kind()
+const char *TupleDeclaration::kind()
 {
     return "tuple";
 }
@@ -190,7 +267,6 @@ TypedefDeclaration::TypedefDeclaration(Loc loc, Identifier *id, Type *basetype,
     this->hbasetype = NULL;
 #endif
     this->sem = 0;
-    this->inuse = 0;
     this->loc = loc;
     this->sinit = NULL;
 }
@@ -242,6 +318,7 @@ void TypedefDeclaration::semantic(Scope *sc)
 	    attributes = sc->attributes;
 	if (sc->parent->isFuncDeclaration() && init)
 	    semantic2(sc);
+	    storage_class |= sc->stc & STCdeprecated;
     }
     else if (sem == 1)
     {
@@ -268,7 +345,7 @@ void TypedefDeclaration::semantic2(Scope *sc)
     }
 }
 
-char *TypedefDeclaration::kind()
+const char *TypedefDeclaration::kind()
 {
     return "typedef";
 }
@@ -329,6 +406,7 @@ AliasDeclaration::AliasDeclaration(Loc loc, Identifier *id, Dsymbol *s)
 
 Dsymbol *AliasDeclaration::syntaxCopy(Dsymbol *s)
 {
+	//printf("AliasDeclaration::syntaxCopy()\n");
     assert(!s);
     AliasDeclaration *sa;
     if (type)
@@ -392,10 +470,24 @@ void AliasDeclaration::semantic(Scope *sc)
      * try to alias y to 3.
      */
     s = type->toDsymbol(sc);
-    if (s)
+    if (s
+#if DMDV2
+`	&& ((s->getType() && type->equals(s->getType())) || s->isEnumMember())
+#endif
+	)
 	goto L2;			// it's a symbolic alias
 
-    //printf("alias type is %s\n", type->toChars());
+    #if DMDV2
+    if (storage_class & STCref)
+    {	// For 'ref' to be attached to function types, and picked
+	// up by Type::resolve(), it has to go into sc.
+	sc = sc->push();
+	sc->stc |= STCref;
+	type->resolve(loc, sc, &e, &t, &s);
+	sc = sc->pop();
+    }
+    else
+	#endif
     type->resolve(loc, sc, &e, &t, &s);
     if (s)
     {
@@ -418,7 +510,9 @@ void AliasDeclaration::semantic(Scope *sc)
 	}
     }
     else if (t)
+    {
 	type = t;
+	 }
     if (overnext)
 	ScopeDsymbol::multiplyDefined(0, this, overnext);
     this->inSemantic = 0;
@@ -456,6 +550,7 @@ void AliasDeclaration::semantic(Scope *sc)
 	    s = NULL;
 	}
     }
+    if (!aliassym)
     aliassym = s;
     this->inSemantic = 0;
 }
@@ -477,13 +572,20 @@ int AliasDeclaration::overloadInsert(Dsymbol *s)
     }
 }
 
-char *AliasDeclaration::kind()
+const char *AliasDeclaration::kind()
 {
     return "alias";
 }
 
 Type *AliasDeclaration::getType()
 {
+	//printf("AliasDeclaration::getType() %s\n", type->toChars());
+#if 0
+    if (!type->deco && scope)
+	semantic(scope);
+    if (type && !type->deco)
+	error("forward reference to alias %s\n", toChars());
+#endif
     return type;
 }
 
@@ -491,10 +593,10 @@ Dsymbol *AliasDeclaration::toAlias()
 {
     //printf("AliasDeclaration::toAlias('%s', this = %p, aliassym = %p, kind = '%s')\n", toChars(), this, aliassym, aliassym ? aliassym->kind() : "");
     assert(this != aliassym);
-    //static int count; if (++count == 10) *(char*)0=0;
+    //static int count; if (++count == 75) exit(0); //*(char*)0=0;
     if (inSemantic)
     {	error("recursive alias declaration");
-//	return this;
+		aliassym = new TypedefDeclaration(loc, ident, Type::terror, NULL);
     }
     Dsymbol *s = aliassym ? aliassym->toAlias() : this;
     return s;
@@ -553,8 +655,9 @@ VarDeclaration::VarDeclaration(Loc loc, Type *type, Identifier *id, Initializer
     this->loc = loc;
     offset = 0;
     noauto = 0;
-    nestedref = 0;
-    inuse = 0;
+    #if DMDV1
+      nestedref = 0;
+ 	 #endif
     ctorinit = 0;
     aliassym = NULL;
     onstack = 0;
@@ -606,10 +709,14 @@ Dsymbol *VarDeclaration::syntaxCopy(Dsymbol *s)
 
 void VarDeclaration::semantic(Scope *sc)
 {
-    //printf("VarDeclaration::semantic('%s', parent = '%s')\n", toChars(), sc->parent->toChars());
-    //printf("type = %s\n", type->toChars());
-    //printf("linkage = %d\n", sc->linkage);
+    #if 0
+    printf("VarDeclaration::semantic('%s', parent = '%s')\n", toChars(), sc->parent->toChars());
+    printf(" type = %s\n", type ? type->toChars() : "null");
+    printf(" stc = x%x\n", sc->stc);
+    printf(" storage_class = x%x\n", storage_class);
+    printf("linkage = %d\n", sc->linkage);
     //if (strcmp(toChars(), "mul") == 0) halt();
+    #endif
 
     storage_class |= sc->stc;
     if (storage_class & STCextern && init)
@@ -635,6 +742,7 @@ void VarDeclaration::semantic(Scope *sc)
 	    originalType = type;
 	type = type->semantic(loc, sc);
     }
+    //printf(" semantic type = %s\n", type ? type->toChars() : "null");
 
     type->checkDeprecated(loc, sc);
     linkage = sc->linkage;
@@ -646,7 +754,14 @@ void VarDeclaration::semantic(Scope *sc)
     else
 	attributes = sc->attributes;
     //printf("sc->stc = %x\n", sc->stc);
-    //printf("storage_class = %x\n", storage_class);
+    //printf("storage_class = x%x\n", storage_class);
+    
+    #if DMDV2
+    if (storage_class & STCgshared && global.params.safe && !sc->module->safe)
+    {
+	error("__gshared not allowed in safe mode; use shared");
+    }
+	#endif
 
     Dsymbol *parent = toParent();
     FuncDeclaration *fd = parent->isFuncDeclaration();
@@ -750,6 +865,16 @@ void VarDeclaration::semantic(Scope *sc)
 	    aad = parent->isAggregateDeclaration();
 	if (aad)
 	{
+	#if DMDV2
+	    assert(!(storage_class & (STCextern | STCstatic | STCtls | STCgshared)));
+
+	    if (storage_class & (STCconst | STCimmutable) && init)
+	    {
+		if (!type->toBasetype()->isTypeBasic())
+		    storage_class |= STCstatic;
+	    }
+	    else
+	#endif
 	    aad->addField(sc, this);
 	}
 
@@ -758,7 +883,8 @@ void VarDeclaration::semantic(Scope *sc)
 	{
 	    error("field not allowed in interface");
 	}
-
+/* Templates cannot add fields to aggregates
+ 	 */
 	TemplateInstance *ti = parent->isTemplateInstance();
 	if (ti)
 	{
@@ -779,6 +905,14 @@ void VarDeclaration::semantic(Scope *sc)
 	    }
 	}
     }
+    
+    #if DMDV2
+    if ((storage_class & (STCref | STCparameter | STCforeach)) == STCref &&
+	ident != Id::This)
+    {
+	error("only parameters or foreach declarations can be ref");
+    }
+	#endif
 
     if (type->isauto() && !noauto)
     {
@@ -812,8 +946,11 @@ void VarDeclaration::semantic(Scope *sc)
 	    Expression *e1;
 	    e1 = new VarExp(loc, this);
 	    e = new AssignExp(loc, e1, e);
-	    e->type = e1->type;
-	    init = new ExpInitializer(loc, e/*->type->defaultInit()*/);
+	    #if DMDV2
+ 	    e->op = TOKconstruct;
+ 		 #endif
+ 	    e->type = e1->type;		// don't type check this, it would fail
+ 	    init = new ExpInitializer(loc, e);
 	    return;
 	}
 	else if (type->ty == Ttypedef)
@@ -832,10 +969,16 @@ void VarDeclaration::semantic(Scope *sc)
 	{
 	    init = getExpInitializer();
 	}
+	#if DMDV2
+	// Default initializer is always a blit
+	op = TOKblit;
+	#endif
     }
 
     if (init)
     {
+    	sc = sc->push();
+ 	sc->stc &= ~(STCconst | STCinvariant | STCpure);
 	ArrayInitializer *ai = init->isArrayInitializer();
 	if (ai && tb->ty == Taarray)
 	{
@@ -863,9 +1006,6 @@ void VarDeclaration::semantic(Scope *sc)
 	    // possibilities.
 	    if (fd && !isStatic() && !isConst() && !init->isVoidInitializer())
 	    {
-		Expression *e1;
-		Type *t;
-		sinteger_t dim;
 
 		//printf("fd = '%s', var = '%s'\n", fd->toChars(), toChars());
 		if (!ei)
@@ -884,15 +1024,15 @@ void VarDeclaration::semantic(Scope *sc)
 		    init = ei;
 		}
 
-		e1 = new VarExp(loc, this);
-
-		t = type->toBasetype();
+		Expression *e1 = new VarExp(loc, this);
+  
+ 		Type *t = type->toBasetype();
 		if (t->ty == Tsarray)
 		{
 		    ei->exp = ei->exp->semantic(sc);
 		    if (!ei->exp->implicitConvTo(type))
 		    {
-			dim = ((TypeSArray *)t)->dim->toInteger();
+			sinteger_t dim = ((TypeSArray *)t)->dim->toInteger();
 			// If multidimensional static array, treat as one large array
 			while (1)
 			{
@@ -908,6 +1048,45 @@ void VarDeclaration::semantic(Scope *sc)
 		else if (t->ty == Tstruct)
 		{
 		    ei->exp = ei->exp->semantic(sc);
+		    #if DMDV2
+		    /* Look to see if initializer is a call to the constructor
+		     */
+		    StructDeclaration *sd = ((TypeStruct *)t)->sym;
+		    if (sd->ctor &&		// there are constructors
+			ei->exp->type->ty == Tstruct &&	// rvalue is the same struct
+			((TypeStruct *)ei->exp->type)->sym == sd &&
+			ei->exp->op == TOKstar)
+		    {
+			/* Look for form of constructor call which is:
+			 *    *__ctmp.ctor(arguments...)
+			 */
+			PtrExp *pe = (PtrExp *)ei->exp;
+			if (pe->e1->op == TOKcall)
+			{   CallExp *ce = (CallExp *)pe->e1;
+			    if (ce->e1->op == TOKdotvar)
+			    {	DotVarExp *dve = (DotVarExp *)ce->e1;
+				if (dve->var->isCtorDeclaration())
+				{   /* It's a constructor call, currently constructing
+				     * a temporary __ctmp.
+				     */
+				    /* Before calling the constructor, initialize
+				     * variable with a bit copy of the default
+				     * initializer
+				     */
+				    Expression *e = new AssignExp(loc, new VarExp(loc, this), t->defaultInit(loc));
+				    e->op = TOKblit;
+				    e->type = t;
+				    ei->exp = new CommaExp(loc, e, ei->exp);
+
+				    /* Replace __ctmp being constructed with e1
+				     */
+				    dve->e1 = e1;
+				    return;
+				}
+			    }
+			}
+		    }
+				#endif
 		    if (!ei->exp->implicitConvTo(type))
 			ei->exp = new CastExp(loc, ei->exp, type);
 		}
@@ -960,6 +1139,12 @@ void VarDeclaration::semantic(Scope *sc)
 		{
 		    if (global.gag == 0)
 			global.errors = errors;	// act as if nothing happened
+			#if DMDV2
+		    /* Save scope for later use, to try again
+		     */
+		    scope = new Scope(*sc);
+		    scope->setNoFree();
+			#endif
 		}
 		else if (ei)
 		{
@@ -968,11 +1153,21 @@ void VarDeclaration::semantic(Scope *sc)
 		    {
 			ei->exp = e;		// no errors, keep result
 		    }
+		    #if DMDV2
+		    else
+		    {
+			/* Save scope for later use, to try again
+			 */
+			scope = new Scope(*sc);
+			scope->setNoFree();
+		    }
+			#endif
 		}
 		else
 		    init = i2;		// no errors, keep result
 	    }
 	}
+	sc = sc->pop();
     }
 }
 
@@ -984,7 +1179,7 @@ ExpInitializer *VarDeclaration::getExpInitializer()
 	ei = init->isExpInitializer();
     else
     {
-	Expression *e = type->defaultInit();
+	Expression *e = type->defaultInit(loc);
 	if (e)
 	    ei = new ExpInitializer(loc, e);
 	else
@@ -1011,7 +1206,7 @@ void VarDeclaration::semantic2(Scope *sc)
     }
 }
 
-char *VarDeclaration::kind()
+const char *VarDeclaration::kind()
 {
     return "variable";
 }
@@ -1026,16 +1221,23 @@ Dsymbol *VarDeclaration::toAlias()
 
 void VarDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
-    if (storage_class & STCconst)
-	buf->writestring("const ");
-    if (storage_class & STCstatic)
-	buf->writestring("static ");
+    StorageClassDeclaration::stcToCBuffer(buf, storage_class);
+  
+    /* If changing, be sure and fix CompoundDeclarationStatement::toCBuffer()
+     * too.
+     */
     if (type)
 	type->toCBuffer(buf, ident, hgs);
     else
 	buf->writestring(ident->toChars());
     if (init)
     {	buf->writestring(" = ");
+#if DMDV2
+	ExpInitializer *ie = init->isExpInitializer();
+	if (ie && (ie->exp->op == TOKconstruct || ie->exp->op == TOKblit))
+	    ((AssignExp *)ie->exp)->e2->toCBuffer(buf, hgs);
+	else
+#endif
 	init->toCBuffer(buf, hgs);
     }
     buf->writeByte(';');
@@ -1062,15 +1264,18 @@ void VarDeclaration::checkCtorConstInit()
 }
 
 /************************************
- * Check to see if variable is a reference to an enclosing function
- * or not.
- */
+!  * Check to see if this variable is actually in an enclosing function
+!  * rather than the current one.
+   */
 
 void VarDeclaration::checkNestedReference(Scope *sc, Loc loc)
 {
+	//printf("VarDeclaration::checkNestedReference() %s\n", toChars());
     if (parent && !isDataseg() && parent != sc->parent)
     {
+    	// The function that this variable is in
 	FuncDeclaration *fdv = toParent()->isFuncDeclaration();
+	// The current function
 	FuncDeclaration *fdthis = sc->parent->isFuncDeclaration();
 
 	if (fdv && fdthis)
@@ -1086,6 +1291,7 @@ void VarDeclaration::checkNestedReference(Scope *sc, Loc loc)
 
 /*******************************
  * Does symbol go into data segment?
+ * Includes extern variables.
  */
 
 int VarDeclaration::isDataseg()
@@ -1108,6 +1314,7 @@ int VarDeclaration::isDataseg()
 
 int VarDeclaration::hasPointers()
 {
+	//printf("VarDeclaration::hasPointers() %s, ty = %d\n", toChars(), type->ty);
     return (!isDataseg() && type->hasPointers());
 }
 
@@ -1210,7 +1417,7 @@ void TypeInfoDeclaration::semantic(Scope *sc)
 
 /***************************** TypeInfoConstDeclaration **********************/
 
-#if V2
+#if DMDV2
 TypeInfoConstDeclaration::TypeInfoConstDeclaration(Type *tinfo)
     : TypeInfoDeclaration(tinfo, 0)
 {
@@ -1219,13 +1426,22 @@ TypeInfoConstDeclaration::TypeInfoConstDeclaration(Type *tinfo)
 
 /***************************** TypeInfoInvariantDeclaration **********************/
 
-#if V2
+#if DMDV2
 TypeInfoInvariantDeclaration::TypeInfoInvariantDeclaration(Type *tinfo)
     : TypeInfoDeclaration(tinfo, 0)
 {
 }
 #endif
 
+/***************************** TypeInfoSharedDeclaration **********************/
+
+#if DMDV2
+TypeInfoSharedDeclaration::TypeInfoSharedDeclaration(Type *tinfo)
+    : TypeInfoDeclaration(tinfo, 0)
+{
+}
+#endif
+
 /***************************** TypeInfoStructDeclaration **********************/
 
 TypeInfoStructDeclaration::TypeInfoStructDeclaration(Type *tinfo)
@@ -1314,8 +1530,8 @@ TypeInfoTupleDeclaration::TypeInfoTupleDeclaration(Type *tinfo)
 
 // For the "this" parameter to member functions
 
-ThisDeclaration::ThisDeclaration(Type *t)
-   : VarDeclaration(0, t, Id::This, NULL)
+ThisDeclaration::ThisDeclaration(Loc loc, Type *t)
+    : VarDeclaration(loc, t, Id::This, NULL)
 {
     noauto = 1;
 }
@@ -1325,5 +1541,3 @@ Dsymbol *ThisDeclaration::syntaxCopy(Dsymbol *s)
     assert(0);		// should never be produced by syntax
     return NULL;
 }
-
-
