@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -12,7 +12,7 @@
 #include <string.h>
 #include <assert.h>
 
-#include "mem.h"
+#include "rmem.h"
 
 #include "mars.h"
 #include "dsymbol.h"
@@ -28,6 +28,7 @@
 #include "init.h"
 #include "import.h"
 #include "template.h"
+#include "attrib.h"
 
 /****************************** Dsymbol ******************************/
 
@@ -41,6 +42,7 @@ Dsymbol::Dsymbol()
     this->isym = NULL;
     this->loc = 0;
     this->comment = NULL;
+    this->scope = NULL;
 }
 
 Dsymbol::Dsymbol(Identifier *ident)
@@ -53,6 +55,7 @@ Dsymbol::Dsymbol(Identifier *ident)
     this->isym = NULL;
     this->loc = 0;
     this->comment = NULL;
+    this->scope = NULL;
 }
 
 int Dsymbol::equals(Object *o)
@@ -191,7 +194,7 @@ char *Dsymbol::locToChars()
     return loc.toChars();
 }
 
-char *Dsymbol::kind()
+const char *Dsymbol::kind()
 {
     return "symbol";
 }
@@ -221,6 +224,17 @@ Dsymbol *Dsymbol::pastMixin()
     return s;
 }
 
+TemplateInstance *Dsymbol::inTemplateInstance()
+ {
+     for (Dsymbol *parent = this->parent; parent; parent = parent->parent)
+     {
+ 	TemplateInstance *ti = parent->isTemplateInstance();
+ 	if (ti)
+ 	    return ti;
+     }
+     return NULL;
+ }
+
 /**********************************
  * Use this instead of toParent() when looking for the
  * 'this' pointer of the enclosing function/class.
@@ -240,26 +254,64 @@ int Dsymbol::isAnonymous()
     return ident ? 0 : 1;
 }
 
+/*************************************
+ * Set scope for future semantic analysis so we can
+ * deal better with forward references.
+ */
+
+void Dsymbol::setScope(Scope *sc)
+{
+    if (!sc->nofree)
+	sc->setNoFree();		// may need it even after semantic() finishes
+    scope = sc;
+}
+
+/*************************************
+ * Does semantic analysis on the public face of declarations.
+ */
+
 void Dsymbol::semantic(Scope *sc)
 {
     error("%p has no semantic routine", this);
 }
 
+/*************************************
+ * Does semantic analysis on initializers and members of aggregates.
+ */
+
 void Dsymbol::semantic2(Scope *sc)
 {
     // Most Dsymbols have no further semantic analysis needed
 }
 
+/*************************************
+ * Does semantic analysis on function bodies.
+ */
+
 void Dsymbol::semantic3(Scope *sc)
 {
     // Most Dsymbols have no further semantic analysis needed
 }
 
+/*************************************
+ * Look for function inlining possibilities.
+ */
+
 void Dsymbol::inlineScan()
 {
-    // Most Dsymbols have no further semantic analysis needed
+    // Most Dsymbols aren't functions
 }
 
+/*********************************************
++  * Search for ident as member of s.
++  * Input:
++  *	flags:	1	don't find private members
++  *		2	don't give error messages
++  *		4	return NULL if ambiguous
++  * Returns:
++  *	NULL if not found
++  */
+
 Dsymbol *Dsymbol::search(Loc loc, Identifier *ident, int flags)
 {
     //printf("Dsymbol::search(this=%p,%s, ident='%s')\n", this, toChars(), ident->toChars());
@@ -305,7 +357,7 @@ Dsymbol *Dsymbol::searchX(Loc loc, Scope *sc, Identifier *id)
 		return NULL;
 	    }
 	    ti->tempdecl = td;
-	    if (!ti->semanticdone)
+	    if (!ti->semanticRun)
 		ti->semantic(sc);
 	    sm = ti->toAlias();
 	    break;
@@ -379,7 +431,9 @@ LabelDsymbol *Dsymbol::isLabel()		// is this a LabelDsymbol()?
 
 AggregateDeclaration *Dsymbol::isMember()	// is this a member of an AggregateDeclaration?
 {
-    Dsymbol *parent = toParent();
+    //printf("Dsymbol::isMember() %s\n", toChars());
+      Dsymbol *parent = toParent();
+     //printf("parent is %s %s\n", parent->kind(), parent->toChars());
     return parent ? parent->isAggregateDeclaration() : NULL;
 }
 
@@ -432,6 +486,7 @@ void Dsymbol::error(const char *format, ...)
 	    fprintf(stdmsg, "%s: ", p);
 	mem.free(p);
 
+	fprintf(stdmsg, "Error: ");
 	if (isAnonymous())
 	    fprintf(stdmsg, "%s ", kind());
 	else
@@ -462,6 +517,7 @@ void Dsymbol::error(Loc loc, const char *format, ...)
 	    fprintf(stdmsg, "%s: ", p);
 	mem.free(p);
 
+	fprintf(stdmsg, "Error: ");
 	fprintf(stdmsg, "%s %s ", kind(), toPrettyChars());
 
 	va_list ap;
@@ -492,6 +548,9 @@ void Dsymbol::checkDeprecated(Loc loc, Scope *sc)
 	{
 	    if (sc->scopesym && sc->scopesym->isDeprecated())
 		return;
+		// If inside a StorageClassDeclaration that is deprecated
+ 	    if (sc->stc & STCdeprecated)
+ 		return;
 	}
 
 	error(loc, "is deprecated");
@@ -607,12 +666,12 @@ Dsymbol *ScopeDsymbol::syntaxCopy(Dsymbol *s)
 }
 
 Dsymbol *ScopeDsymbol::search(Loc loc, Identifier *ident, int flags)
-{   Dsymbol *s;
-    int i;
+{   
 
     //printf("%s->ScopeDsymbol::search(ident='%s', flags=x%x)\n", toChars(), ident->toChars(), flags);
+    //if (strcmp(ident->toChars(),"c") == 0) *(char*)0=0;
     // Look in symbols declared in this module
-    s = symtab ? symtab->lookup(ident) : NULL;
+    Dsymbol *s = symtab ? symtab->lookup(ident) : NULL;
     if (s)
     {
 	//printf("\ts = '%s.%s'\n",toChars(),s->toChars());
@@ -620,7 +679,7 @@ Dsymbol *ScopeDsymbol::search(Loc loc, Identifier *ident, int flags)
     else if (imports)
     {
 	// Look in imported modules
-	for (i = 0; i < imports->dim; i++)
+	for (int i = 0; i < imports->dim; i++)
 	{   ScopeDsymbol *ss = (ScopeDsymbol *)imports->data[i];
 	    Dsymbol *s2;
 
@@ -629,6 +688,8 @@ Dsymbol *ScopeDsymbol::search(Loc loc, Identifier *ident, int flags)
 		continue;
 
 	    //printf("\tscanning import '%s', prots = %d, isModule = %p, isImport = %p\n", ss->toChars(), prots[i], ss->isModule(), ss->isImport());
+	    /* Don't find private members if ss is a module
++ 	     */
 	    s2 = ss->search(loc, ident, ss->isModule() ? 1 : 0);
 	    if (!s)
 		s = s2;
@@ -636,6 +697,10 @@ Dsymbol *ScopeDsymbol::search(Loc loc, Identifier *ident, int flags)
 	    {
 		if (s->toAlias() == s2->toAlias())
 		{
+			/* After following aliases, we found the same symbol,
++ 		     * so it's not an ambiguity.
++ 		     * But if one alias is deprecated, prefer the other.
++ 		     */
 		    if (s->isDeprecated())
 			s = s2;
 		}
@@ -685,7 +750,7 @@ void ScopeDsymbol::importScope(ScopeDsymbol *s, enum PROT protection)
 	    {   ScopeDsymbol *ss;
 
 		ss = (ScopeDsymbol *) imports->data[i];
-		if (ss == s)
+		if (ss == s)			// if already imported
 		{
 		    if (protection > prots[i])
 			prots[i] = protection;	// upgrade access
@@ -758,7 +823,7 @@ Dsymbol *ScopeDsymbol::nameCollision(Dsymbol *s)
     return sprev;
 }
 
-char *ScopeDsymbol::kind()
+const char *ScopeDsymbol::kind()
 {
     return "ScopeDsymbol";
 }
@@ -770,7 +835,7 @@ char *ScopeDsymbol::kind()
  * Returns NULL if not found
  */
 
-#if V2
+#if DMDV2
 FuncDeclaration *ScopeDsymbol::findGetMembers()
 {
     Dsymbol *s = search_function(this, Id::getmembers);
@@ -852,7 +917,8 @@ Dsymbol *ArrayScopeSymbol::search(Loc loc, Identifier *ident, int flags)
     L1:
 
 	if (td)
- 	{
+ 	{   /* $ gives the number of elements in the tuple
+	     */
 	    VarDeclaration *v = new VarDeclaration(loc, Type::tsize_t, Id::dollar, NULL);
 	    Expression *e = new IntegerExp(0, td->objects->dim, Type::tsize_t);
 	    v->init = new ExpInitializer(0, e);
@@ -861,7 +927,8 @@ Dsymbol *ArrayScopeSymbol::search(Loc loc, Identifier *ident, int flags)
 	}
 
 	if (type)
- 	{
+ 	{   /* $ gives the number of type entries in the type tuple
+ 	     */
 	    VarDeclaration *v = new VarDeclaration(loc, Type::tsize_t, Id::dollar, NULL);
 	    Expression *e = new IntegerExp(0, type->arguments->dim, Type::tsize_t);
 	    v->init = new ExpInitializer(0, e);
@@ -870,22 +937,30 @@ Dsymbol *ArrayScopeSymbol::search(Loc loc, Identifier *ident, int flags)
 	}
 
 	if (exp->op == TOKindex)
-	{
+	{   /* array[index] where index is some function of $
+ 	     */
 	    IndexExp *ie = (IndexExp *)exp;
 
 	    pvar = &ie->lengthVar;
 	    ce = ie->e1;
 	}
 	else if (exp->op == TOKslice)
-	{
+	{   /* array[lwr .. upr] where lwr or upr is some function of $
+ 	     */
 	    SliceExp *se = (SliceExp *)exp;
 
 	    pvar = &se->lengthVar;
 	    ce = se->e1;
 	}
 	else
+		/* Didn't find $, look in enclosing scope(s).
+ 	     */
 	    return NULL;
 
+	/* If we are indexing into an array that is really a type
+ 	 * tuple, rewrite this as an index into a type tuple and
+ 	 * try again.
+ 	 */
 	if (ce->op == TOKtype)
 	{
 	    Type *t = ((TypeExp *)ce)->type;
@@ -895,8 +970,13 @@ Dsymbol *ArrayScopeSymbol::search(Loc loc, Identifier *ident, int flags)
 	    }
 	}
 
-	if (!*pvar)
-	{
+	/* *pvar is lazily initialized, so if we refer to $
+	 * multiple times, it gets set only once.
+	 */
+	if (!*pvar)		// if not already initialized
+	{   /* Create variable v and set it to the value of $,
+	     * which will be a constant.
+	     */
 	    VarDeclaration *v = new VarDeclaration(loc, Type::tsize_t, Id::dollar, NULL);
 
 	    if (ce->op == TOKstring)
@@ -944,13 +1024,13 @@ DsymbolTable::~DsymbolTable()
 }
 
 Dsymbol *DsymbolTable::lookup(Identifier *ident)
-{   StringValue *sv;
+{
 
 #ifdef DEBUG
     assert(ident);
     assert(tab);
 #endif
-    sv = tab->lookup((char*)ident->string, ident->len);
+    StringValue *sv = tab->lookup((char*)ident->string, ident->len);
     return (Dsymbol *)(sv ? sv->ptrvalue : NULL);
 }
 
