@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -49,6 +49,7 @@ struct HdrGenState;
 struct BinExp;
 struct InterState;
 struct Symbol;		// back end symbol
+struct OverloadSet;
 
 enum TOK;
 
@@ -73,6 +74,7 @@ void argsToCBuffer(OutBuffer *buf, Expressions *arguments, HdrGenState *hgs);
 void expandTuples(Expressions *exps);
 FuncDeclaration *hasThis(Scope *sc);
 Expression *fromConstInitializer(int result, Expression *e);
+int arrayExpressionCanThrow(Expressions *exps);
 
 struct Expression : Object
 {
@@ -92,12 +94,13 @@ struct Expression : Object
     char *toChars();
     virtual void dump(int indent);
     void error(const char *format, ...);
+    void warning(const char *format, ...);
     virtual void rvalue();
 
     static Expression *combine(Expression *e1, Expression *e2);
     static Expressions *arraySyntaxCopy(Expressions *exps);
 
-    virtual integer_t toInteger();
+    virtual dinteger_t toInteger();
     virtual uinteger_t toUInteger();
     virtual real_t toReal();
     virtual real_t toImaginary();
@@ -135,6 +138,7 @@ struct Expression : Object
     virtual int isBool(int result);
     virtual int isBit();
     virtual int checkSideEffect(int flag);
+    virtual int canThrow();
 
     virtual int inlineCost(InlineCostState *ics);
     virtual Expression *doInline(InlineDoState *ids);
@@ -144,6 +148,10 @@ struct Expression : Object
     virtual int isCommutative();
     virtual Identifier *opId();
     virtual Identifier *opId_r();
+    
+    // For array ops
+    virtual void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    virtual Expression *buildArrayLoop(Arguments *fparams);
 
     // Back end
     virtual elem *toElem(IRState *irs);
@@ -152,16 +160,16 @@ struct Expression : Object
 
 struct IntegerExp : Expression
 {
-    integer_t value;
+    dinteger_t value;
 
-    IntegerExp(Loc loc, integer_t value, Type *type);
-    IntegerExp(integer_t value);
+    IntegerExp(Loc loc, dinteger_t value, Type *type);
+    IntegerExp(dinteger_t value);
     int equals(Object *o);
     Expression *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
     char *toChars();
     void dump(int indent);
-    integer_t toInteger();
+    dinteger_t toInteger();
     real_t toReal();
     real_t toImaginary();
     complex_t toComplex();
@@ -175,6 +183,13 @@ struct IntegerExp : Expression
     dt_t **toDt(dt_t **pdt);
 };
 
+struct ErrorExp : IntegerExp
+{
+    ErrorExp();
+
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
 struct RealExp : Expression
 {
     real_t value;
@@ -184,7 +199,7 @@ struct RealExp : Expression
     Expression *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
     char *toChars();
-    integer_t toInteger();
+    dinteger_t toInteger();
     uinteger_t toUInteger();
     real_t toReal();
     real_t toImaginary();
@@ -207,7 +222,7 @@ struct ComplexExp : Expression
     Expression *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
     char *toChars();
-    integer_t toInteger();
+    dinteger_t toInteger();
     uinteger_t toUInteger();
     real_t toReal();
     real_t toImaginary();
@@ -436,20 +451,12 @@ struct StructLiteralExp : Expression
     Expression *inlineScan(InlineScanState *iss);
 };
 
-struct TypeDotIdExp : Expression
-{
-    Identifier *ident;
-
-    TypeDotIdExp(Loc loc, Type *type, Identifier *ident);
-    Expression *syntaxCopy();
-    Expression *semantic(Scope *sc);
-    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    elem *toElem(IRState *irs);
-};
+Expression *typeDotIdExp(Loc loc, Type *type, Identifier *ident);
 
 struct TypeExp : Expression
 {
     TypeExp(Loc loc, Type *type);
+    Expression *syntaxCopy();
     Expression *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Expression *optimize(int result);
@@ -568,6 +575,19 @@ struct VarExp : Expression
     //Expression *inlineScan(InlineScanState *iss);
 };
 
+#if DMDV2
+ // Overload Set
+ 
+ struct OverExp : Expression
+ {
+     OverloadSet *vars;
+ 
+     OverExp(OverloadSet *s);
+     int isLvalue();
+     Expression *toLvalue(Scope *sc, Expression *e);
+ };
+ #endif
+
 // Function/Delegate literal
 
 struct FuncExp : Expression
@@ -617,6 +637,19 @@ struct TypeidExp : Expression
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
+#if DMDV2
+ struct TraitsExp : Expression
+ {
+     Identifier *ident;
+     Objects *args;
+ 
+     TraitsExp(Loc loc, Identifier *ident, Objects *args);
+     Expression *syntaxCopy();
+     Expression *semantic(Scope *sc);
+     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+ };
+ #endif
+
 struct HaltExp : Expression
 {
     HaltExp(Loc loc);
@@ -689,6 +722,7 @@ struct BinExp : Expression
     Expression *interpretCommon(InterState *istate, Expression *(*fp)(Type *, Expression *, Expression *));
     Expression *interpretCommon2(InterState *istate, Expression *(*fp)(TOK, Type *, Expression *, Expression *));
     Expression *interpretAssignCommon(InterState *istate, Expression *(*fp)(Type *, Expression *, Expression *), int post = 0);
+    Expression *arrayOp(Scope *sc);
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -765,6 +799,8 @@ struct DotVarExp : UnaExp
     Expression *semantic(Scope *sc);
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
+    Expression *optimize(int result);
+    Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     void dump(int indent);
     elem *toElem(IRState *irs);
@@ -825,6 +861,7 @@ struct CallExp : UnaExp
     elem *toElem(IRState *irs);
     void scanForNestedRef(Scope *sc);
     Expression *toLvalue(Scope *sc, Expression *e);
+    Expression *modifiableLvalue(Scope *sc, Expression *e);
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -859,6 +896,8 @@ struct NegExp : UnaExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -881,6 +920,8 @@ struct ComExp : UnaExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -956,6 +997,8 @@ struct SliceExp : UnaExp
     void dump(int indent);
     elem *toElem(IRState *irs);
     void scanForNestedRef(Scope *sc);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -1053,6 +1096,8 @@ struct AssignExp : BinExp
     Expression *checkToBoolean();
     Expression *interpret(InterState *istate);
     Identifier *opId();    // For operator overloading
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
     elem *toElem(IRState *irs);
 };
 
@@ -1062,25 +1107,33 @@ struct op##AssignExp : BinExp					\
     op##AssignExp(Loc loc, Expression *e1, Expression *e2);	\
     Expression *semantic(Scope *sc);				\
     Expression *interpret(InterState *istate);			\
+    X(void buildArrayIdent(OutBuffer *buf, Expressions *arguments);) \
+    X(Expression *buildArrayLoop(Arguments *fparams);)		\
 								\
     Identifier *opId();    /* For operator overloading */	\
 								\
     elem *toElem(IRState *irs);					\
 };
 
+#define X(a) a
 ASSIGNEXP(Add)
 ASSIGNEXP(Min)
-ASSIGNEXP(Cat)
 ASSIGNEXP(Mul)
 ASSIGNEXP(Div)
 ASSIGNEXP(Mod)
-ASSIGNEXP(Shl)
-ASSIGNEXP(Shr)
-ASSIGNEXP(Ushr)
 ASSIGNEXP(And)
 ASSIGNEXP(Or)
 ASSIGNEXP(Xor)
+#undef X
 
+#define X(a)
+ 
+ ASSIGNEXP(Shl)
+ ASSIGNEXP(Shr)
+ ASSIGNEXP(Ushr)
+ ASSIGNEXP(Cat)
+ 
+ #undef X
 #undef ASSIGNEXP
 
 struct AddExp : BinExp
@@ -1089,6 +1142,8 @@ struct AddExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1104,6 +1159,8 @@ struct MinExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1132,6 +1189,8 @@ struct MulExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1147,6 +1206,8 @@ struct DivExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1161,6 +1222,8 @@ struct ModExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     Identifier *opId();
@@ -1217,6 +1280,8 @@ struct AndExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1232,6 +1297,8 @@ struct OrExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1247,6 +1314,8 @@ struct XorExp : BinExp
     Expression *semantic(Scope *sc);
     Expression *optimize(int result);
     Expression *interpret(InterState *istate);
+    void buildArrayIdent(OutBuffer *buf, Expressions *arguments);
+    Expression *buildArrayLoop(Arguments *fparams);
 
     // For operator overloading
     int isCommutative();
@@ -1371,6 +1440,33 @@ struct CondExp : BinExp
     elem *toElem(IRState *irs);
 };
 
+#if DMDV2
+ /****************************************************************/
+ 
+ struct DefaultInitExp : Expression
+ {
+     enum TOK subop;		// which of the derived classes this is
+ 
+     DefaultInitExp(Loc loc, enum TOK subop, int size);
+     virtual Expression *resolve(Loc loc, Scope *sc) = 0;
+     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+ };
+ 
+ struct FileInitExp : DefaultInitExp
+ {
+     FileInitExp(Loc loc);
+     Expression *semantic(Scope *sc);
+     Expression *resolve(Loc loc, Scope *sc);
+ };
+ 
+ struct LineInitExp : DefaultInitExp
+ {
+     LineInitExp(Loc loc);
+     Expression *semantic(Scope *sc);
+     Expression *resolve(Loc loc, Scope *sc);
+ };
+ #endif
+
 
 /****************************************************************/
 
