@@ -1,6 +1,5 @@
-
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -42,6 +41,7 @@ FuncDeclaration::FuncDeclaration(Loc loc, Loc endloc, Identifier *id, enum STC s
     : Declaration(id)
 {
     //printf("FuncDeclaration(id = '%s', type = %p)\n", id->toChars(), type);
+    //printf("storage_class = x%x\n", storage_class);
     this->storage_class = storage_class;
     this->type = type;
     this->loc = loc;
@@ -71,16 +71,24 @@ FuncDeclaration::FuncDeclaration(Loc loc, Loc endloc, Identifier *id, enum STC s
     inlineAsm = 0;
     cantInterpret = 0;
     semanticRun = 0;
-    nestedFrameRef = 0;
+    #if DMDV1
+      nestedFrameRef = 0;
+ 	 #endif
     fes = NULL;
     introducing = 0;
     tintro = NULL;
+    /* The type given for "infer the return type" is a TypeFunction with
++      * NULL for the return type.
++      */
     inferRetType = (type && type->nextOf() == NULL);
-    scope = NULL;
     hasReturnExp = 0;
     nrvo_can = 1;
     nrvo_var = NULL;
     shidden = NULL;
+    #if DMDV2
+     builtin = BUILTINunknown;
+     tookAddressOf = 0;
+	 #endif
 }
 
 Dsymbol *FuncDeclaration::syntaxCopy(Dsymbol *s)
@@ -108,17 +116,32 @@ void FuncDeclaration::semantic(Scope *sc)
     StructDeclaration *sd;
     ClassDeclaration *cd;
     InterfaceDeclaration *id;
+    Dsymbol *pd;
 
 #if 0
     printf("FuncDeclaration::semantic(sc = %p, this = %p, '%s', linkage = %d)\n", sc, this, toPrettyChars(), sc->linkage);
     if (isFuncLiteralDeclaration())
 	printf("\tFuncLiteralDeclaration()\n");
-    printf("sc->parent = %s\n", sc->parent->toChars());
-    printf("type: %s\n", type->toChars());
+    printf("sc->parent = %s, parent = %s\n", sc->parent->toChars(), parent ? parent->toChars() : "");
+    printf("type: %p, %s\n", type, type->toChars());
 #endif
 
-    if (type->nextOf())
-	type = type->semantic(loc, sc);
+	if (semanticRun && isFuncLiteralDeclaration())
+     {
+ 	/* Member functions that have return types that are
+ 	 * forward references can have semantic() run more than
+ 	 * once on them.
+ 	 * See test\interface2.d, test20
+ 	 */
+ 	return;
+     }
+     assert(semanticRun <= 1);
+     semanticRun = 1;
+
+    if (!type->deco)
+    {
+  	type = type->semantic(loc, sc);
+    }
     //type->print();
     if (type->ty != Tfunction)
     {
@@ -142,6 +165,9 @@ void FuncDeclaration::semantic(Scope *sc)
 	attributes = sc->attributes;
     //printf("function storage_class = x%x\n", storage_class);
     Dsymbol *parent = toParent();
+    
+    if (ident == Id::ctor && !isCtorDeclaration())
+ 	error("_ctor is reserved for constructors");
 
     if (isConst() || isAuto() || isScope())
 	error("functions cannot be const or auto");
@@ -209,6 +235,9 @@ void FuncDeclaration::semantic(Scope *sc)
 	storage_class |= STCabstract;
 
 	if (isCtorDeclaration() ||
+	#if DMDV2
+ 	    isPostBlitDeclaration() ||
+	#endif
 	    isDtorDeclaration() ||
 	    isInvariantDeclaration() ||
 	    isUnitTestDeclaration() || isNewDeclaration() || isDelete())
@@ -216,6 +245,17 @@ void FuncDeclaration::semantic(Scope *sc)
 	if (fbody)
 	    error("function body is not abstract in interface %s", id->toChars());
     }
+    /* Template member functions aren't virtual:
++      *   interface TestInterface { void tpl(T)(); }
++      * and so won't work in interfaces
++      */
+     if ((pd = toParent()) != NULL &&
+ 	pd->isTemplateInstance() &&
+ 	(pd = toParent2()) != NULL &&
+ 	(id = pd->isInterfaceDeclaration()) != NULL)
+     {
+ 	error("template member function not allowed in interface %s", id->toChars());
+     }
 
     cd = parent->isClassDeclaration();
     if (cd)
@@ -275,8 +315,11 @@ void FuncDeclaration::semantic(Scope *sc)
 	vi = findVtblIndex(&cd->vtbl, cd->baseClass ? cd->baseClass->vtbl.dim : 0);
 	switch (vi)
 	{
-	    case -1:	// didn't find one
-		// This is an 'introducing' function.
+	    case -1:
+ 		/* Didn't find one, so
+! 		 * This is an 'introducing' function which gets a new
+! 		 * slot in the vtbl[].
+! 		 */
 
 		// Verify this doesn't override previous final function
 		if (cd->baseClass)
@@ -292,6 +335,8 @@ void FuncDeclaration::semantic(Scope *sc)
 
 		if (isFinal())
 		{
+			if (isOverride())
+ 			error("does not override any function");
 		    cd->vtblFinal.push(this);
 		}
 		else
@@ -315,9 +360,9 @@ void FuncDeclaration::semantic(Scope *sc)
 		if (fdv->isFinal())
 		    error("cannot override final function %s", fdv->toPrettyChars());
 
-#if V2
-		if (!isOverride() && global.params.warnings)
-		    error("overrides base class function %s, but is not marked with 'override'", fdv->toPrettyChars());
+#if DMDV2
+		if (!isOverride())
+ 		    warning(loc, "overrides base class function %s, but is not marked with 'override'", fdv->toPrettyChars());
 #endif
 
 		if (fdv->toParent() == parent)
@@ -331,7 +376,7 @@ void FuncDeclaration::semantic(Scope *sc)
 #if !BREAKABI
 			&& !isDtorDeclaration()
 #endif
-#if V2
+#if DMDV2
 			&& !isPostBlitDeclaration()
 #endif
 			)
@@ -598,7 +643,6 @@ void FuncDeclaration::semantic2(Scope *sc)
 
 void FuncDeclaration::semantic3(Scope *sc)
 {   TypeFunction *f;
-    AggregateDeclaration *ad;
     VarDeclaration *argptr = NULL;
     VarDeclaration *_arguments = NULL;
 
@@ -611,24 +655,23 @@ void FuncDeclaration::semantic3(Scope *sc)
     }
     //printf("FuncDeclaration::semantic3('%s.%s', sc = %p, loc = %s)\n", parent->toChars(), toChars(), sc, loc.toChars());
     //fflush(stdout);
+    //printf("storage class = x%x %x\n", sc->stc, storage_class);
     //{ static int x; if (++x == 2) *(char*)0=0; }
     //printf("\tlinkage = %d\n", sc->linkage);
 
     //printf(" sc->incontract = %d\n", sc->incontract);
-    if (semanticRun)
+    if (semanticRun >= 3)
 	return;
-    semanticRun = 1;
+    semanticRun = 3;
 
     if (!type || type->ty != Tfunction)
 	return;
     f = (TypeFunction *)(type);
-    size_t nparams = Argument::dim(f->parameters);
 
     // Check the 'throws' clause
     if (fthrows)
-    {	int i;
-
-	for (i = 0; i < fthrows->dim; i++)
+    {
+ 	for (int i = 0; i < fthrows->dim; i++)
 	{
 	    Type *t = (Type *)fthrows->data[i];
 
@@ -667,7 +710,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 	sc2->noctor = 0;
 
 	// Declare 'this'
-	ad = isThis();
+	AggregateDeclaration *ad = isThis();
 	if (ad)
 	{   VarDeclaration *v;
 
@@ -680,7 +723,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 	    {
 		assert(!isNested());	// can't be both member and nested
 		assert(ad->handle);
-		v = new ThisDeclaration(ad->handle);
+		v = new ThisDeclaration(loc, ad->handle);
 		v->storage_class |= STCparameter | STCin;
 		v->semantic(sc2);
 		if (!sc2->insert(v))
@@ -691,9 +734,11 @@ void FuncDeclaration::semantic3(Scope *sc)
 	}
 	else if (isNested())
 	{
-	    VarDeclaration *v;
-
-	    v = new ThisDeclaration(Type::tvoid->pointerTo());
+	    /* The 'this' for a nested function is the link to the
+! 	     * enclosing function's stack frame.
+! 	     * Note that nested functions and member functions are disjoint.
+! 	     */
+ 	    VarDeclaration *v = new ThisDeclaration(loc, Type::tvoid->pointerTo());
 	    v->storage_class |= STCparameter | STCin;
 	    v->semantic(sc2);
 	    if (!sc2->insert(v))
@@ -749,6 +794,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 	    for (size_t i = 0; i < f->parameters->dim; i++)
 	    {	Argument *arg = (Argument *)f->parameters->data[i];
 
+		//printf("[%d] arg->type->ty = %d %s\n", i, arg->type->ty, arg->type->toChars());
 		if (arg->type->ty == Ttuple)
 		{   TypeTuple *t = (TypeTuple *)arg->type;
 		    size_t dim = Argument::dim(t->arguments);
@@ -760,8 +806,10 @@ void FuncDeclaration::semantic3(Scope *sc)
 	    }
 	}
 
-	// Declare all the function parameters as variables
-	if (nparams)
+	/* Declare all the function parameters as variables
+ 	 * and install them in parameters[]
+ 	 */
+ 	size_t nparams = Argument::dim(f->parameters);
 	{   /* parameters[] has all the tuples removed, as the back end
 	     * doesn't know about tuples
 	     */
@@ -776,13 +824,10 @@ void FuncDeclaration::semantic3(Scope *sc)
 		    /* Generate identifier for un-named parameter,
 		     * because we need it later on.
 		     */
-		    OutBuffer buf;
-		    buf.printf("_param_%"PRIuSIZE, i);
-		    char *name = (char *)buf.extractData();
-		    id = new Identifier(name, TOKidentifier);
-		    arg->ident = id;
+		    arg->ident = id = Identifier::generateId("_param_", i);
 		}
-		VarDeclaration *v = new VarDeclaration(loc, arg->type, id, NULL);
+		Type *vtype = arg->type;
+ 		VarDeclaration *v = new VarDeclaration(loc, vtype, id, NULL);
 		//printf("declaring parameter %s of type %s\n", v->toChars(), v->type->toChars());
 		v->storage_class |= STCparameter;
 		if (f->varargs == 2 && i + 1 == nparams)
@@ -819,11 +864,11 @@ void FuncDeclaration::semantic3(Scope *sc)
 			assert(narg->ident);
 			VarDeclaration *v = sc2->search(0, narg->ident, NULL)->isVarDeclaration();
 			assert(v);
-			Expression *e = new VarExp(0, v);
+			Expression *e = new VarExp(v->loc, v);
 			exps->data[j] = (void *)e;
 		    }
 		    assert(arg->ident);
-		    TupleDeclaration *v = new TupleDeclaration(0, arg->ident, exps);
+		    TupleDeclaration *v = new TupleDeclaration(loc, arg->ident, exps);
 		    //printf("declaring tuple %s\n", v->toChars());
 		    v->isexp = 1;
 		    if (!sc2->insert(v))
@@ -879,6 +924,12 @@ void FuncDeclaration::semantic3(Scope *sc)
 
 		v = new VarDeclaration(loc, type->nextOf(), outId, NULL);
 		v->noauto = 1;
+		#if DMDV2
+ 		if (f->isref)
+ 		{
+ 		    v->storage_class |= STCref | STCforeach;
+ 		}
+ 		#endif
 		sc2->incontract--;
 		v->semantic(sc2);
 		sc2->incontract++;
@@ -929,8 +980,12 @@ void FuncDeclaration::semantic3(Scope *sc)
 		}
 		else
 		{   // Call invariant virtually
-		    ThisExp *v = new ThisExp(0);
+		    Expression *v = new ThisExp(0);
 		    v->type = vthis->type;
+		    #if STRUCTTHISREF
+ 		    if (ad->isStructDeclaration())
+ 			v = v->addressOf(sc);
+ 			#endif
 		    e = new AssertExp(0, v);
 		}
 		if (e)
@@ -983,7 +1038,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 		f = (TypeFunction *)type;
 	    }
 
-	    int offend = fbody ? fbody->fallOffEnd() : TRUE;
+	    int offend = fbody ? fbody->blockExit() & BEfallthru : TRUE;
 
 	    if (isStaticCtorDeclaration())
 	    {	/* It's a static constructor. Ensure that all
@@ -1065,10 +1120,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 		    if (offend)
 		    {   Expression *e;
 
-			if (global.params.warnings)
-			{   fprintf(stdmsg, "warning - ");
-			    error("no return at end of function");
-			}
+			warning(loc, "no return at end of function");
 
 			if (global.params.useAssert &&
 			    !global.params.useInline)
@@ -1078,7 +1130,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 			    e = new AssertExp(
 				  endloc,
 				  new IntegerExp(0),
-				  new StringExp(loc, "missing return expression")
+				  new StringExp(loc, (char *)"missing return expression")
 				);
 			}
 			else
@@ -1194,7 +1246,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 		{   // Call invariant virtually
 		    ThisExp *v = new ThisExp(0);
 		    v->type = vthis->type;
-		    Expression *se = new StringExp(0, "null this");
+		    Expression *se = new StringExp(0, (char *)"null this");
 		    se = se->semantic(sc);
 		    se->type = Type::tchar->arrayOf();
 		    e = new AssertExp(loc, v, se);
@@ -1233,7 +1285,7 @@ void FuncDeclaration::semantic3(Scope *sc)
 	sc2->callSuper = 0;
 	sc2->pop();
     }
-    semanticRun = 2;
+    semanticRun = 4;
 }
 
 void FuncDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
@@ -1477,7 +1529,7 @@ int fp1(void *param, FuncDeclaration *f)
 	return 1;
     }
 
-#if V2
+#if DMDV2
     /* Allow covariant matches, if it's just a const conversion
      * of the return type
      */
@@ -1564,12 +1616,11 @@ int fp2(void *param, FuncDeclaration *f)
 
     if (f != m->lastf)		// skip duplicates
     {
-	TypeFunction *tf;
-
-	m->anyf = f;
+   m->anyf = f;
+	TypeFunction *tf = (TypeFunction *)f->type;
 	tf = (TypeFunction *)f->type;
 	match = (MATCH) tf->callMatch(arguments);
-	//printf("match = %d\n", match);
+	//printf("1match = %d\n", match);
 	if (match != MATCHnomatch)
 	{
 	    if (match > m->last)
@@ -1604,7 +1655,8 @@ int fp2(void *param, FuncDeclaration *f)
 }
 
 
-void overloadResolveX(Match *m, FuncDeclaration *fstart, Expressions *arguments)
+void overloadResolveX(Match *m, FuncDeclaration *fstart,
+ 	Expression *ethis, Expressions *arguments)
 {
     Param2 p;
     p.m = m;
@@ -1630,7 +1682,7 @@ void overloadResolveX(Match *m, FuncDeclaration *fstart, Expressions *arguments)
 	fa = d->isFuncAliasDeclaration();
 	if (fa)
 	{
-	    overloadResolveX(m, fa->funcalias, arguments);
+	    overloadResolveX(m, fa->funcalias, NULL, arguments);
 	    next = fa->overnext;
 	}
 	else if ((f = d->isFuncDeclaration()) != NULL)
@@ -1645,7 +1697,7 @@ void overloadResolveX(Match *m, FuncDeclaration *fstart, Expressions *arguments)
 		m->anyf = f;
 		tf = (TypeFunction *)f->type;
 		match = (MATCH) tf->callMatch(arguments);
-		//printf("match = %d\n", match);
+		//printf("2match = %d\n", match);
 		if (match != MATCHnomatch)
 		{
 		    if (match > m->last)
@@ -1694,7 +1746,7 @@ void overloadResolveX(Match *m, FuncDeclaration *fstart, Expressions *arguments)
 }
 #endif
 
-FuncDeclaration *FuncDeclaration::overloadResolve(Loc loc, Expressions *arguments)
+FuncDeclaration *FuncDeclaration::overloadResolve(Loc loc, Expression *ethis, Expressions *arguments, int flags)
 {
     TypeFunction *tf;
     Match m;
@@ -1717,7 +1769,7 @@ if (arguments)
 
     memset(&m, 0, sizeof(m));
     m.last = MATCHnomatch;
-    overloadResolveX(&m, this, arguments);
+    overloadResolveX(&m, this, NULL, arguments);
 
     if (m.count == 1)		// exactly one match
     {
@@ -1904,8 +1956,16 @@ int FuncDeclaration::isMain()
 
 int FuncDeclaration::isWinMain()
 {
-    return ident == Id::WinMain &&
-	linkage != LINKc && !isMember();
+    //printf("FuncDeclaration::isWinMain() %s\n", toChars());
+ #if 0
+     int x = ident == Id::WinMain &&
+ 	linkage != LINKc && !isMember();
+     printf("%s\n", x ? "yes" : "no");
+     return x;
+ #else
+      return ident == Id::WinMain &&
+  	linkage != LINKc && !isMember();
+ #endif
 }
 
 int FuncDeclaration::isDllMain()
@@ -1943,6 +2003,23 @@ int FuncDeclaration::isVirtual()
 	toParent()->isClassDeclaration();
 }
 
+int FuncDeclaration::isFinal()
+ {
+     ClassDeclaration *cd;
+ #if 0
+     printf("FuncDeclaration::isFinal(%s)\n", toChars());
+     printf("%p %d %d %d %d\n", isMember(), isStatic(), protection == PROTprivate, isCtorDeclaration(), linkage != LINKd);
+     printf("result is %d\n",
+ 	isMember() &&
+ 	!(isStatic() || protection == PROTprivate || protection == PROTpackage) &&
+ 	(cd = toParent()->isClassDeclaration()) != NULL &&
+ 	cd->storage_class & STCfinal);
+ #endif
+     return isMember() &&
+ 	(Declaration::isFinal() ||
+ 	 ((cd = toParent()->isClassDeclaration()) != NULL && cd->storage_class & STCfinal));
+ }
+
 int FuncDeclaration::isAbstract()
 {
     return storage_class & STCabstract;
@@ -2000,7 +2077,7 @@ int FuncDeclaration::addPostInvariant()
  * Generate a FuncDeclaration for a runtime library function.
  */
 
-FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, char *name,
+FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, const char *name,
     Type *t1, Type *t2, Type *t3)
 {
     return genCfunc(treturn, Lexer::idPool(name), t1, t2, t3);
@@ -2051,7 +2128,7 @@ FuncDeclaration *FuncDeclaration::genCfunc(Type *treturn, Identifier *id,
     return fd;
 }
 
-char *FuncDeclaration::kind()
+const char *FuncDeclaration::kind()
 {
     return "function";
 }
@@ -2061,7 +2138,7 @@ char *FuncDeclaration::kind()
  * created for them.
  */
 
-#if V2
+#if DMDV2
 int FuncDeclaration::needsClosure()
 {
     /* Need a closure for all the closureVars[] if any of the
@@ -2071,7 +2148,11 @@ int FuncDeclaration::needsClosure()
      * 1) is a virtual function
      * 2) has its address taken
      * 3) has a parent that escapes
-     * escapes.
+     *
+     * Note that since a non-virtual function can be called by
+     * a virtual one, if that non-virtual function accesses a closure
+     * var, the closure still has to be taken. Hence, we check for isThis()
+     * instead of isVirtual(). (thanks to David Friedman)
      */
 
     //printf("FuncDeclaration::needsClosure() %s\n", toChars());
@@ -2085,14 +2166,14 @@ int FuncDeclaration::needsClosure()
 	    assert(f != this);
 
 	    //printf("\t\tf = %s, %d, %d\n", f->toChars(), f->isVirtual(), f->tookAddressOf);
-	    if (f->isVirtual() || f->tookAddressOf)
+	    if (f->isThis() || f->tookAddressOf)
 		goto Lyes;	// assume f escapes this function's scope
 
 	    // Look to see if any parents of f that are below this escape
-	    for (Dsymbol *s = f->parent; s != this; s = s->parent)
+	    for (Dsymbol *s = f->parent; s && s != this; s = s->parent)
 	    {
 		f = s->isFuncDeclaration();
-		if (f && (f->isVirtual() || f->tookAddressOf))
+		if (f && (f->isThis() || f->tookAddressOf))
 		    goto Lyes;
 	    }
 	}
@@ -2117,7 +2198,7 @@ FuncAliasDeclaration::FuncAliasDeclaration(FuncDeclaration *funcalias)
     this->funcalias = funcalias;
 }
 
-char *FuncAliasDeclaration::kind()
+const char *FuncAliasDeclaration::kind()
 {
     return "function alias";
 }
@@ -2129,7 +2210,7 @@ FuncLiteralDeclaration::FuncLiteralDeclaration(Loc loc, Loc endloc, Type *type,
 	enum TOK tok, ForeachStatement *fes)
     : FuncDeclaration(loc, endloc, NULL, STCundefined, type)
 {
-    char *id;
+    const char *id;
 
     if (fes)
 	id = "__foreachbody";
@@ -2162,7 +2243,12 @@ int FuncLiteralDeclaration::isNested()
     return (tok == TOKdelegate);
 }
 
-char *FuncLiteralDeclaration::kind()
+int FuncLiteralDeclaration::isVirtual()
+ {
+     return FALSE;
+ }
+ 
+ const char *FuncLiteralDeclaration::kind()
 {
     // GCC requires the (char*) casts
     return (tok == TOKdelegate) ? (char*)"delegate" : (char*)"function";
@@ -2190,7 +2276,7 @@ CtorDeclaration::CtorDeclaration(Loc loc, Loc endloc, Arguments *arguments, int
 {
     this->arguments = arguments;
     this->varargs = varargs;
-    //printf("CtorDeclaration() %s\n", toChars());
+    //printf("CtorDeclaration(loc = %s) %s\n", loc.toChars(), toChars());
 }
 
 Dsymbol *CtorDeclaration::syntaxCopy(Dsymbol *s)
@@ -2233,6 +2319,9 @@ void CtorDeclaration::semantic(Scope *sc)
     else
 	tret = cd->type; //->referenceTo();
     type = new TypeFunction(arguments, tret, varargs, LINKd);
+    
+    if (!originalType)
+ 	originalType = type;
 
     sc->flags |= SCOPEctor;
     type = type->semantic(loc, sc);
@@ -2259,14 +2348,14 @@ void CtorDeclaration::semantic(Scope *sc)
 	cd->defaultCtor = this;
 }
 
-char *CtorDeclaration::kind()
+const char *CtorDeclaration::kind()
 {
     return "constructor";
 }
 
 char *CtorDeclaration::toChars()
 {
-    return "this";
+    return (char *)"this";
 }
 
 int CtorDeclaration::isVirtual()
@@ -2281,7 +2370,7 @@ int CtorDeclaration::addPreInvariant()
 
 int CtorDeclaration::addPostInvariant()
 {
-    return (vthis && global.params.useInvariants);
+    return (isThis() && vthis && global.params.useInvariants);
 }
 
 
@@ -2292,6 +2381,80 @@ void CtorDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
     bodyToCBuffer(buf, hgs);
 }
 
+/********************************* PostBlitDeclaration ****************************/
+
+#if DMDV2
+PostBlitDeclaration::PostBlitDeclaration(Loc loc, Loc endloc)
+    : FuncDeclaration(loc, endloc, Id::_postblit, STCundefined, NULL)
+{
+}
+
+PostBlitDeclaration::PostBlitDeclaration(Loc loc, Loc endloc, Identifier *id)
+    : FuncDeclaration(loc, endloc, id, STCundefined, NULL)
+{
+}
+
+Dsymbol *PostBlitDeclaration::syntaxCopy(Dsymbol *s)
+{
+    assert(!s);
+    PostBlitDeclaration *dd = new PostBlitDeclaration(loc, endloc, ident);
+    return FuncDeclaration::syntaxCopy(dd);
+}
+
+
+void PostBlitDeclaration::semantic(Scope *sc)
+{
+    //printf("PostBlitDeclaration::semantic() %s\n", toChars());
+    //printf("ident: %s, %s, %p, %p\n", ident->toChars(), Id::dtor->toChars(), ident, Id::dtor);
+    parent = sc->parent;
+    Dsymbol *parent = toParent();
+    StructDeclaration *ad = parent->isStructDeclaration();
+    if (!ad)
+    {
+	error("post blits are only for struct/union definitions, not %s %s", parent->kind(), parent->toChars());
+    }
+    else if (ident == Id::_postblit)
+	ad->postblits.push(this);
+    type = new TypeFunction(NULL, Type::tvoid, FALSE, LINKd);
+
+    sc = sc->push();
+    sc->stc &= ~STCstatic;		// not static
+    sc->linkage = LINKd;
+
+    FuncDeclaration::semantic(sc);
+
+    sc->pop();
+}
+
+int PostBlitDeclaration::overloadInsert(Dsymbol *s)
+{
+    return FALSE;	// cannot overload postblits
+}
+
+int PostBlitDeclaration::addPreInvariant()
+{
+    return FALSE;
+}
+
+int PostBlitDeclaration::addPostInvariant()
+{
+    return (isThis() && vthis && global.params.useInvariants);
+}
+
+int PostBlitDeclaration::isVirtual()
+{
+    return FALSE;
+}
+
+void PostBlitDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
+{
+    if (hgs->hdrgen)
+	return;
+    buf->writestring("=this()");
+    bodyToCBuffer(buf, hgs);
+}
+#endif
+
 /********************************* DtorDeclaration ****************************/
 
 DtorDeclaration::DtorDeclaration(Loc loc, Loc endloc)
@@ -2343,7 +2506,7 @@ int DtorDeclaration::overloadInsert(Dsymbol *s)
 
 int DtorDeclaration::addPreInvariant()
 {
-    return (vthis && global.params.useInvariants);
+    return (isThis() && vthis && global.params.useInvariants);
 }
 
 int DtorDeclaration::addPostInvariant()
@@ -2351,6 +2514,16 @@ int DtorDeclaration::addPostInvariant()
     return FALSE;
 }
 
+const char *DtorDeclaration::kind()
+{	
+    return "destructor";
+}
+
+char *DtorDeclaration::toChars()
+{
+    return (char *)"~this";
+}
+
 int DtorDeclaration::isVirtual()
 {
     /* This should be FALSE so that dtor's don't get put into the vtbl[],
@@ -2394,6 +2567,34 @@ void StaticCtorDeclaration::semantic(Scope *sc)
     //printf("StaticCtorDeclaration::semantic()\n");
 
     type = new TypeFunction(NULL, Type::tvoid, FALSE, LINKd);
+    
+    /* If the static ctor appears within a template instantiation,
++      * it could get called multiple times by the module constructors
++      * for different modules. Thus, protect it with a gate.
++      */
+     if (inTemplateInstance())
+     {
+ 	/* Add this prefix to the function:
++ 	 *	static int gate;
++ 	 *	if (++gate != 1) return;
++ 	 * Note that this is not thread safe; should not have threads
++ 	 * during static construction.
++ 	 */
+ 	Identifier *id = Lexer::idPool("__gate");
+ 	VarDeclaration *v = new VarDeclaration(0, Type::tint32, id, NULL);
+ 	v->storage_class = STCstatic;
+ 	Statements *sa = new Statements();
+ 	Statement *s = new DeclarationStatement(0, v);
+ 	sa->push(s);
+ 	Expression *e = new IdentifierExp(0, id);
+ 	e = new AddAssignExp(0, e, new IntegerExp(1));
+ 	e = new EqualExp(TOKnotequal, 0, e, new IntegerExp(1));
+ 	s = new IfStatement(0, NULL, e, new ReturnStatement(0, NULL), NULL);
+ 	sa->push(s);
+ 	if (fbody)
+ 	    sa->push(fbody);
+ 	fbody = new CompoundStatement(0, sa);
+     }
 
     FuncDeclaration::semantic(sc);
 
@@ -2450,6 +2651,7 @@ StaticDtorDeclaration::StaticDtorDeclaration(Loc loc, Loc endloc)
     : FuncDeclaration(loc, endloc,
       Identifier::generateId("_staticDtor"), STCstatic, NULL)
 {
+	vgate = NULL;
 }
 
 Dsymbol *StaticDtorDeclaration::syntaxCopy(Dsymbol *s)
@@ -2472,6 +2674,36 @@ void StaticDtorDeclaration::semantic(Scope *sc)
     {
     }
     type = new TypeFunction(NULL, Type::tvoid, FALSE, LINKd);
+    
+    /* If the static ctor appears within a template instantiation,
++      * it could get called multiple times by the module constructors
++      * for different modules. Thus, protect it with a gate.
++      */
+     if (inTemplateInstance())
+     {
+ 	/* Add this prefix to the function:
++ 	 *	static int gate;
++ 	 *	if (--gate != 0) return;
++ 	 * Increment gate during constructor execution.
++ 	 * Note that this is not thread safe; should not have threads
++ 	 * during static destruction.
++ 	 */
+ 	Identifier *id = Lexer::idPool("__gate");
+ 	VarDeclaration *v = new VarDeclaration(0, Type::tint32, id, NULL);
+ 	v->storage_class = STCstatic;
+ 	Statements *sa = new Statements();
+ 	Statement *s = new DeclarationStatement(0, v);
+ 	sa->push(s);
+ 	Expression *e = new IdentifierExp(0, id);
+ 	e = new AddAssignExp(0, e, new IntegerExp(-1));
+ 	e = new EqualExp(TOKnotequal, 0, e, new IntegerExp(0));
+ 	s = new IfStatement(0, NULL, e, new ReturnStatement(0, NULL), NULL);
+ 	sa->push(s);
+ 	if (fbody)
+ 	    sa->push(fbody);
+ 	fbody = new CompoundStatement(0, sa);
+ 	vgate = v;
+     }
 
     FuncDeclaration::semantic(sc);
 
@@ -2548,7 +2780,7 @@ void InvariantDeclaration::semantic(Scope *sc)
     ad = parent->isAggregateDeclaration();
     if (!ad)
     {
-	error("invariants only are for struct/union/class definitions");
+	error("invariants are only for struct/union/class definitions");
 	return;
     }
     else if (ad->inv && ad->inv != this)
@@ -2601,12 +2833,7 @@ void InvariantDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 static Identifier *unitTestId()
 {
-    static int n;
-    char buffer[10 + sizeof(n)*3 + 1];
-
-    sprintf(buffer,"__unittest%d", n);
-    n++;
-    return Lexer::idPool(buffer);
+    return Lexer::uniqueId("__unittest");
 }
 
 UnitTestDeclaration::UnitTestDeclaration(Loc loc, Loc endloc)
@@ -2628,10 +2855,12 @@ void UnitTestDeclaration::semantic(Scope *sc)
 {
     if (global.params.useUnitTests)
     {
-	Type *tret;
 
 	type = new TypeFunction(NULL, Type::tvoid, FALSE, LINKd);
-	FuncDeclaration::semantic(sc);
+	Scope *sc2 = sc->push();
+ 	sc2->linkage = LINKd;
+ 	FuncDeclaration::semantic(sc2);
+	sc2->pop();
     }
 
     // We're going to need ModuleInfo even if the unit tests are not
@@ -2715,24 +2944,24 @@ void NewDeclaration::semantic(Scope *sc)
     type = type->semantic(loc, sc);
     assert(type->ty == Tfunction);
 
-    // Check that there is at least one argument of type uint
+    // Check that there is at least one argument of type size_t
     TypeFunction *tf = (TypeFunction *)type;
     if (Argument::dim(tf->parameters) < 1)
     {
-	error("at least one argument of type uint expected");
+	error("at least one argument of type size_t expected");
     }
     else
     {
 	Argument *a = Argument::getNth(tf->parameters, 0);
-	if (!a->type->equals(Type::tuns32)  &&
+	if (!a->type->equals(Type::tsize_t)  &&
 	    (! global.params.isX86_64 || !a->type->equals(Type::tuns64)))
-	    error("first argument must be type uint, not %s", a->type->toChars());
+ 	    error("first argument must be type size_t, not %s", a->type->toChars());
     }
 
     FuncDeclaration::semantic(sc);
 }
 
-char *NewDeclaration::kind()
+const char *NewDeclaration::kind()
 {
     return "allocator";
 }
@@ -2816,7 +3045,7 @@ void DeleteDeclaration::semantic(Scope *sc)
     FuncDeclaration::semantic(sc);
 }
 
-char *DeleteDeclaration::kind()
+const char *DeleteDeclaration::kind()
 {
     return "deallocator";
 }
