@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -384,7 +384,7 @@ Initializer *ArrayInitializer::semantic(Scope *sc, Type *t)
 	value.data[i] = (void *)val;
 	length++;
 	if (length == 0)
-	    error("array dimension overflow");
+	    error(loc, "array dimension overflow");
 	if (length > dim)
 	    dim = length;
     }
@@ -402,24 +402,71 @@ Expression *ArrayInitializer::toExpression()
 {   Expressions *elements;
     Expression *e;
 
-    //printf("ArrayInitializer::toExpression()\n");
+    //printf("ArrayInitializer::toExpression(), dim = %d\n", dim);
     //static int i; if (++i == 2) halt();
+    
+    size_t edim;
+    Type *t = NULL;
+    if (type)
+    {
+	t = type->toBasetype();
+	switch (t->ty)
+	{
+	   case Tsarray:
+	       edim = ((TypeSArray *)t)->dim->toInteger();
+	       break;
+
+	   case Tpointer:
+	   case Tarray:
+	       edim = dim;
+	       break;
+
+	   default:
+	       assert(0);
+	}
+    }
+    else
+	edim = value.dim;
+
     elements = new Expressions();
-    for (size_t i = 0; i < value.dim; i++)
+    elements->setDim(edim);
+    for (size_t i = 0, j = 0; i < value.dim; i++, j++)
     {
 	if (index.data[i])
-	    goto Lno;
+	    j = ((Expression *)index.data[i])->toInteger();
+ 	assert(j < edim);
 	Initializer *iz = (Initializer *)value.data[i];
 	if (!iz)
 	    goto Lno;
 	Expression *ex = iz->toExpression();
 	if (!ex)
+	{
 	    goto Lno;
-	elements->push(ex);
     }
-    e = new ArrayLiteralExp(loc, elements);
+    elements->data[j] = ex;
+    }
+
+    /* Fill in any missing elements with the default initializer
+     */
+    {
+    Expression *init = NULL;
+    for (size_t i = 0; i < edim; i++)
+    {
+	if (!elements->data[i])
+	{
+	    if (!type)
+		goto Lno;
+	    if (!init)
+		init = t->next->defaultInit();
+	    elements->data[i] = init;
+	}
+    }
+
+    Expression *e = new ArrayLiteralExp(loc, elements);
+
     e->type = type;
     return e;
+    }
 
 Lno:
     delete elements;
@@ -427,7 +474,6 @@ Lno:
     return NULL;
 }
 
-
 /********************************
  * If possible, convert array initializer to associative array initializer.
  */
@@ -574,7 +620,19 @@ Type *ExpInitializer::inferType(Scope *sc)
     //printf("ExpInitializer::inferType() %s\n", toChars());
     exp = exp->semantic(sc);
     exp = resolveProperties(sc, exp);
-    return exp->type;
+    #if DMDV2
+    // Give error for overloaded function addresses
+    if (exp->op == TOKsymoff)
+    {   SymOffExp *se = (SymOffExp *)exp;
+	if (se->hasOverloads && !se->var->isFuncDeclaration()->isUnique())
+	    exp->error("cannot infer type from overloaded function symbol %s", exp->toChars());
+    }
+	#endif
+
+    Type *t = exp->type;
+    if (!t)
+	t = Initializer::inferType(sc);
+    return t;
 }
 
 Expression *ExpInitializer::toExpression()
