@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -24,25 +24,9 @@
 //#include <wchar.h>
 #include <stdlib.h>
 #include <assert.h>
-#include <sys/time.h>
+#include <time.h>	// for time() and ctime()
 
-#ifdef IN_GCC
-
-#include <time.h>
-#include "mem.h"
-
-#else
-
-#if __GNUC__
-#include <time.h>
-#endif
-
-#if _WIN32
-#include "..\root\mem.h"
-#else
-#include "../root/mem.h"
-#endif
-#endif
+#include "rmem.h"
 
 #include "stringtable.h"
 
@@ -92,7 +76,7 @@ static void cmtable_init()
 
 /************************* Token **********************************************/
 
-char *Token::tochars[TOKMAX];
+const char *Token::tochars[TOKMAX];
 
 void *Token::operator new(size_t size)
 {   Token *t;
@@ -114,8 +98,8 @@ void Token::print()
 }
 #endif
 
-char *Token::toChars()
-{   char *p;
+const char *Token::toChars()
+ {   const char *p;
     static char buffer[3 + 3 * sizeof(value) + 1];
 
     p = buffer;
@@ -244,8 +228,8 @@ char *Token::toChars()
     return p;
 }
 
-char *Token::toChars(enum TOK value)
-{   char *p;
+const char *Token::toChars(enum TOK value)
+{   const char *p;
     static char buffer[3 + 3 * sizeof(value) + 1];
 
     p = tochars[value];
@@ -398,6 +382,15 @@ Token *Lexer::peek(Token *ct)
     return t;
 }
 
+/***********************
+  * Look ahead at next token's value.
+  */
+ 
+ TOK Lexer::peekNext()
+ {
+     return peek(&token)->value;
+ }
+
 /*********************************
  * tk is on the opening (.
  * Look ahead and return token that is past the closing ).
@@ -472,7 +465,7 @@ int Lexer::isValidIdentifier(char *p)
     while (p[idx])
     {   dchar_t dc;
 
-	char *q = utf_decodeChar((unsigned char *)p, len, &idx, &dc);
+	const char *q = utf_decodeChar((unsigned char *)p, len, &idx, &dc);
 	if (q)
 	    goto Linvalid;
 
@@ -573,7 +566,7 @@ void Lexer::scan(Token *t)
 		t->value = hexStringConstant(t);
 		return;
 
-#if V2
+#if DMDV2
 	    case 'q':
 		if (p[1] == '"')
 		{
@@ -632,7 +625,7 @@ void Lexer::scan(Token *t)
 	    case 'a':  	case 'b':   case 'c':   case 'd':   case 'e':
 	    case 'f':  	case 'g':   case 'h':   case 'i':   case 'j':
 	    case 'k':  	            case 'm':   case 'n':   case 'o':
-#if V2
+#if DMDV2
 	    case 'p':  	/*case 'q': case 'r':*/ case 's':   case 't':
 #else
 	    case 'p':  	case 'q': /*case 'r':*/ case 's':   case 't':
@@ -681,26 +674,28 @@ void Lexer::scan(Token *t)
 			sprintf(time, "%.8s", p + 11);
 			sprintf(timestamp, "%.24s", p);
 		    }
-
+			#if DMDV1
 		    if (mod && id == Id::FILE)
 		    {
 			t->ustring = (unsigned char *)(loc.filename ? loc.filename : mod->ident->toChars());
-			goto Lstring;
+			goto Lstr;
 		    }
 		    else if (mod && id == Id::LINE)
 		    {
 			t->value = TOKint64v;
 			t->uns64value = loc.linnum;
 		    }
-		    else if (id == Id::DATE)
+		    else
+ 			#endif //if V1
+ 		    if (id == Id::DATE)
 		    {
 			t->ustring = (unsigned char *)date;
-			goto Lstring;
+			goto Lstr;
 		    }
 		    else if (id == Id::TIME)
 		    {
 			t->ustring = (unsigned char *)time;
-			goto Lstring;
+			goto Lstr;
 		    }
 		    else if (id == Id::VENDOR)
 		    {
@@ -709,12 +704,12 @@ void Lexer::scan(Token *t)
 #else
 			t->ustring = (unsigned char *)"Digital Mars D";
 #endif
-			goto Lstring;
+			goto Lstr;
 		    }
 		    else if (id == Id::TIMESTAMP)
 		    {
 			t->ustring = (unsigned char *)timestamp;
-		     Lstring:
+		     Lstr:
 			t->value = TOKstring;
 		     Llen:
 			t->postfix = 0;
@@ -724,7 +719,7 @@ void Lexer::scan(Token *t)
 		    {	unsigned major = 0;
 			unsigned minor = 0;
 
-			for (char *p = global.version + 1; 1; p++)
+			for (const char *p = global.version + 1; 1; p++)
 			{
 			    char c = *p;
 			    if (isdigit(c))
@@ -739,7 +734,7 @@ void Lexer::scan(Token *t)
 			t->value = TOKint64v;
 			t->uns64value = major * 1000 + minor;
 		    }
-#if V2
+#if DMDV2
 		    else if (id == Id::EOFX)
 		    {
 			t->value = TOKeof;
@@ -1281,7 +1276,9 @@ unsigned Lexer::escapeSequence()
 			}
 		    }
 		    if (ndigits != 2 && !utf_isValidDchar(v))
-			error("invalid UTF character \\U%08x", v);
+			{	error("invalid UTF character \\U%08x", v);
+ 			v = '?';	// recover with valid UTF character
+ 		    }
 		    c = v;
 		}
 		else
@@ -1485,7 +1482,7 @@ TOK Lexer::hexStringConstant(Token *t)
 }
 
 
-#if V2
+#if DMDV2
 /**************************************
  * Lex delimited strings:
  *	q"(foo(xxx))"   // "foo(xxx)"
@@ -2192,13 +2189,15 @@ done:
 		break;
 	    if (d >= r)
 		break;
-	    if (n && n * r + d <= n)
+	   uinteger_t n2 = n * r;
+ 	    //printf("n2 / r = %llx, n = %llx\n", n2/r, n);
+	    if (n2 / r != n || n2 + d < n)
 	    {
 		error ("integer overflow");
 		break;
 	    }
 
-	    n = n * r + d;
+	    n = n2 + d;
 	    p++;
 	}
 #endif
@@ -2446,6 +2445,10 @@ done:
 #ifdef IN_GCC
 	    real_t::parse((char *)stringbuffer.data, real_t::Double);
 #else	    
+		/* Should do our own strtod(), since dmc and linux gcc
++ 	     * accept 2.22507e-308, while apple gcc will only take
++ 	     * 2.22508e-308. Not sure who is right.
++ 	     */
 	    strtod((char *)stringbuffer.data, NULL);
 #endif
 	    result = TOKfloat64v;
@@ -2609,7 +2612,7 @@ unsigned Lexer::decodeUTF()
     unsigned char *s = p;
     size_t len;
     size_t idx;
-    char *msg;
+    const char *msg;
 
     c = *s;
     assert(c & 0x80);
@@ -2640,10 +2643,12 @@ unsigned Lexer::decodeUTF()
 
 void Lexer::getDocComment(Token *t, unsigned lineComment)
 {
-    OutBuffer buf;
+    /* ct tells us which kind of comment it is: '/', '*', or '+'
+!      */
     unsigned char ct = t->ptr[2];
+    /* Start of comment text skips over / * *, / + +, or / / /
++      */
     unsigned char *q = t->ptr + 3;	// start of comment text
-    int linestart = 0;
 
     unsigned char *qend = p;
     if (ct == '*' || ct == '+')
@@ -2667,6 +2672,12 @@ void Lexer::getDocComment(Token *t, unsigned lineComment)
 		break;
 	}
     }
+    
+    /* Comment is now [q .. qend].
++      * Canonicalize it into buf[].
++      */
+     OutBuffer buf;
+     int linestart = 0;
 
     for (; q < qend; q++)
     {
@@ -2744,11 +2755,13 @@ void Lexer::getDocComment(Token *t, unsigned lineComment)
 }
 
 /********************************************
- * Combine two document comments into one.
+ * Combine two document comments into one,
+ * separated by a newline.
  */
 
 unsigned char *Lexer::combineComments(unsigned char *c1, unsigned char *c2)
 {
+	//printf("Lexer::combineComments('%s', '%s')\n", c1, c2);
     unsigned char *c = c2;
 
     if (c1)
@@ -2759,6 +2772,12 @@ unsigned char *Lexer::combineComments(unsigned char *c1, unsigned char *c2)
 
 	    c = (unsigned char *)mem.malloc(len1 + 1 + len2 + 1);
 	    memcpy(c, c1, len1);
+	    if (len1 && c1[len1 - 1] != '\n')
+ 	    {	c[len1] = '\n';
+ 		len1++;
+ 	    }
+ 	    memcpy(c + len1, c2, len2);
+ 	    c[len1 + len2] = 0;
 	    c[len1] = '\n';
 	    memcpy(c + len1 + 1, c2, len2);
 	    c[len1 + 1 + len2] = 0;
@@ -2807,7 +2826,7 @@ Identifier *Lexer::uniqueId(const char *s)
  */
 
 struct Keyword
-{   char *name;
+{   const char *name;
     enum TOK value;
 };
 
@@ -2926,11 +2945,16 @@ static Keyword keywords[] =
     // Added after 1.0
     {	"ref",		TOKref		},
     {	"macro",	TOKmacro	},
-#if V2
+#if DMDV2
     {	"pure",		TOKpure		},
     {	"nothrow",	TOKnothrow	},
+    {	"__thread",	TOKtls		},
     {	"__traits",	TOKtraits	},
     {	"__overloadset", TOKoverloadset	},
+    {	"__FILE__",	TOKfile		},
+    {	"__LINE__",	TOKline		},
+    {	"shared",	TOKshared	},
+    {	"immutable",	TOKimmutable	},
 #endif
 };
 
@@ -2956,7 +2980,7 @@ void Lexer::initKeywords()
     cmtable_init();
 
     for (u = 0; u < nkeywords; u++)
-    {	char *s;
+    {	const char *s;
 
 	//printf("keyword[%d] = '%s'\n",u, keywords[u].name);
 	s = keywords[u].name;
@@ -2983,7 +3007,7 @@ void Lexer::initKeywords()
     Token::tochars[TOKxorass]		= "^=";
     Token::tochars[TOKassign]		= "=";
     Token::tochars[TOKconstruct]	= "=";
-#if V2
+#if DMDV2
     Token::tochars[TOKblit]		= "=";
 #endif
     Token::tochars[TOKlt]		= "<";
@@ -3059,7 +3083,6 @@ void Lexer::initKeywords()
     Token::tochars[TOKdotvar]		= "dotvar";
     Token::tochars[TOKdottype]		= "dottype";
     Token::tochars[TOKsymoff]		= "symoff";
-    Token::tochars[TOKtypedot]		= "typedot";
     Token::tochars[TOKarraylength]	= "arraylength";
     Token::tochars[TOKarrayliteral]	= "arrayliteral";
     Token::tochars[TOKassocarrayliteral] = "assocarrayliteral";
@@ -3070,4 +3093,6 @@ void Lexer::initKeywords()
     Token::tochars[TOKdeclaration]	= "declaration";
     Token::tochars[TOKdottd]		= "dottd";
     Token::tochars[TOKon_scope_exit]	= "scope(exit)";
+    Token::tochars[TOKon_scope_success]	= "scope(success)";
+    Token::tochars[TOKon_scope_failure]	= "scope(failure)";
 }
