@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -16,7 +16,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "mem.h"
+#include "rmem.h"
 #include "root.h"
 #include "stringtable.h"
 
@@ -24,7 +24,7 @@
 #include "lib.h"
 #include "melf.h"
 
-#define LOG 1
+#define LOG 0
 
 Library::Library()
 {
@@ -115,7 +115,7 @@ struct ObjModule
     unsigned offset;		// offset from start of library
     char *name;			// module name (file name)
     int name_offset;		// if not -1, offset into string table of name
-    long file_time;		// file time
+    time_t file_time;		// file time
     unsigned user_id;
     unsigned group_id;
     unsigned file_mode;
@@ -222,68 +222,138 @@ void Library::scanObjModule(ObjModule *om)
     size_t buflen = om->length;
     int reason = 0;
 
-    if (buflen < sizeof(elf_header_f32))
-    {
-      Lcorrupt:
-	error("corrupt ELF object module %s %d", om->name, reason);
-	return;
-    }
-
-    elf_header_f32 *eh = (elf_header_f32 *)buf;
-    if (memcmp(buf, elf, 4))
-    {	reason = 1;
-	goto Lcorrupt;
-    }
-    if (eh->EHtype != EH_TYPE_RELFILE)
-	return;				// not relocatable object module
-
-#if 0
-    /* String table section
-     */
-    elf_sht *string_section = (elf_sht *)(buf + eh->EHshtoff +
-	eh->EHshtentsz * eh->EHshtstrndx);
-    if (string_section->SHtype != SHT_STRTAB)
-    {
-	//printf("buf = %p, EHshtentsz = %d, EHshtstrndx = %d\n", buf, eh->EHshtentsz, eh->EHshtstrndx);
-	//printf("SHtype = %d, SHT_STRTAB = %d\n", string_section->SHtype, SHT_STRTAB);
-	reason = 2;
-	goto Lcorrupt;
-    }
-    printf("strtab SHfileoff = x%x\n", string_section->SHfileoff);
-    char *string_tab = (char *)(buf + string_section->SHfileoff);
-#endif
-
-    /* For each Section
-     */
-    for (unsigned u = 0; u < eh->EHshtnum; u++)
-    {	elf_sht *section = (elf_sht *)(buf + eh->EHshtoff + eh->EHshtentsz * u);
-
-	if (section->SHtype == SHT_SYMTAB)
-	{   /* SHoptidx gives the particular string table section
-	     * used for the symbol names.
-	     */
-	    elf_sht *string_section = (elf_sht *)(buf + eh->EHshtoff +
-		eh->EHshtentsz * section->SHoptidx);
-	    if (string_section->SHtype != SHT_STRTAB)
-	    {
-		reason = 3;
-		goto Lcorrupt;
-	    }
-	    char *string_tab = (char *)(buf + string_section->SHfileoff);
-
-	    for (unsigned offset = 0; offset < section->SHsecsz; offset += sizeof(elf_symtab))
-	    {	elf_symtab *sym = (elf_symtab *)(buf + section->SHfileoff + offset);
-
-		if ((sym->STinfo >> 4) == ST_BIND_GLOBAL &&
-		    sym->STdefsht != SHT_UNDEF)	// not extern
-		{
-		    char *name = string_tab + sym->STname;
-		    //printf("sym STname = x%x\n", sym->STname);
-		    addSymbol(om, name, 1);
-		}
-	    }
-	}
-    }
+    if (buflen < EI_NIDENT + sizeof(Elf32_Hdr))
+      {
+        Lcorrupt:
+  	error("corrupt ELF object module %s %d", om->name, reason);
+  	return;
+      }
+  
+      if (memcmp(buf, elf, 4))
+      {	reason = 1;
+  	goto Lcorrupt;
+      }
+     if (buf[EI_VERSION] != EV_CURRENT)
+     {
+ 	error("ELF object module %s has EI_VERSION = %d, should be %d", om->name, buf[EI_VERSION], EV_CURRENT);
+ 	return;
+     }
+     if (buf[EI_DATA] != ELFDATA2LSB)
+     {
+ 	error("ELF object module %s is byte swapped and unsupported", om->name);
+ 	return;
+     }
+     if (buf[EI_CLASS] == ELFCLASS32)
+     {
+ 	Elf32_Hdr *eh = (Elf32_Hdr *)(buf + EI_NIDENT);
+ 	if (eh->e_type != ET_REL)
+ 	{
+ 	    error("ELF object module %s is not relocatable", om->name);
+  	    return;				// not relocatable object module
+ 	}
+ 	if (eh->e_version != EV_CURRENT)
+ 	    goto Lcorrupt;
+  
+ 	/* For each Section
+  	 */
+ 	for (unsigned u = 0; u < eh->e_shnum; u++)
+ 	{   Elf32_Shdr *section = (Elf32_Shdr *)(buf + eh->e_shoff + eh->e_shentsize * u);
+ 
+ 	    if (section->sh_type == SHT_SYMTAB)
+ 	    {   /* sh_link gives the particular string table section
+! 		 * used for the symbol names.
+! 		 */
+ 		Elf32_Shdr *string_section = (Elf32_Shdr *)(buf + eh->e_shoff +
+ 		    eh->e_shentsize * section->sh_link);
+ 		if (string_section->sh_type != SHT_STRTAB)
+  		{
+ 		    reason = 3;
+  		    goto Lcorrupt;
+  		}
+ 		char *string_tab = (char *)(buf + string_section->sh_offset);
+ 
+ 		for (unsigned offset = 0; offset < section->sh_size; offset += sizeof(Elf32_Sym))
+ 		{   Elf32_Sym *sym = (Elf32_Sym *)(buf + section->sh_offset + offset);
+ 
+ 		    if (((sym->st_info >> 4) == STB_GLOBAL ||
+ 			 (sym->st_info >> 4) == STB_WEAK) &&
+ 			sym->st_shndx != SHT_UNDEF)	// not extern
+ 		    {
+ 			char *name = string_tab + sym->st_name;
+ 			//printf("sym st_name = x%x\n", sym->st_name);
+ 			addSymbol(om, name, 1);
+ 		    }
+ 		}
+ 	    }
+ 	}
+     }
+     else if (buf[EI_CLASS] == ELFCLASS64)
+     {
+ 	Elf64_Ehdr *eh = (Elf64_Ehdr *)(buf + EI_NIDENT);
+ 	if (buflen < EI_NIDENT + sizeof(Elf64_Ehdr))
+ 	    goto Lcorrupt;
+ 	if (eh->e_type != ET_REL)
+ 	{
+ 	    error("ELF object module %s is not relocatable", om->name);
+ 	    return;				// not relocatable object module
+ 	}
+ 	if (eh->e_version != EV_CURRENT)
+ 	    goto Lcorrupt;
+  
+  	/* For each Section
+  	 */
+ 	for (unsigned u = 0; u < eh->e_shnum; u++)
+ 	{   Elf64_Shdr *section = (Elf64_Shdr *)(buf + eh->e_shoff + eh->e_shentsize * u);
+  
+ 	    if (section->sh_type == SHT_SYMTAB)
+ 	    {   /* sh_link gives the particular string table section
+  		 * used for the symbol names.
+  		 */
+ 		Elf64_Shdr *string_section = (Elf64_Shdr *)(buf + eh->e_shoff +
+ 		    eh->e_shentsize * section->sh_link);
+ 		if (string_section->sh_type != SHT_STRTAB)
+  		{
+  		    reason = 3;
+  		    goto Lcorrupt;
+  		}
+ 		char *string_tab = (char *)(buf + string_section->sh_offset);
+  
+ 		for (unsigned offset = 0; offset < section->sh_size; offset += sizeof(Elf64_Sym))
+ 		{   Elf64_Sym *sym = (Elf64_Sym *)(buf + section->sh_offset + offset);
+  
+ 		    if (((sym->st_info >> 4) == STB_GLOBAL ||
+ 			 (sym->st_info >> 4) == STB_WEAK) &&
+ 			sym->st_shndx != SHT_UNDEF)	// not extern
+  		    {
+ 			char *name = string_tab + sym->st_name;
+ 			//printf("sym st_name = x%x\n", sym->st_name);
+  			addSymbol(om, name, 1);
+  		    }
+  		}
+  	    }
+  	}
+     }
+     else
+     {
+ 	error("ELF object module %s is unrecognized class %d", om->name, buf[EI_CLASS]);
+ 	return;
+     }
+ 
+ #if 0
+     /* String table section
+      */
+     Elf32_Shdr *string_section = (Elf32_Shdr *)(buf + eh->e_shoff +
+ 	eh->e_shentsize * eh->e_shstrndx);
+     if (string_section->sh_type != SHT_STRTAB)
+     {
+ 	//printf("buf = %p, e_shentsize = %d, e_shstrndx = %d\n", buf, eh->e_shentsize, eh->e_shstrndx);
+ 	//printf("sh_type = %d, SHT_STRTAB = %d\n", string_section->sh_type, SHT_STRTAB);
+ 	reason = 2;
+ 	goto Lcorrupt;
+     }
+     printf("strtab sh_offset = x%x\n", string_section->sh_offset);
+     char *string_tab = (char *)(buf + string_section->sh_offset);
+ #endif
 }
 
 /***************************************
@@ -293,7 +363,7 @@ void Library::scanObjModule(ObjModule *om)
  * and load the file.
  */
 
-void Library::addObject(char *module_name, void *buf, size_t buflen)
+void Library::addObject(const char *module_name, void *buf, size_t buflen)
 {
     if (!module_name)
 	module_name = "";
@@ -303,7 +373,7 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
     int fromfile = 0;
     if (!buf)
     {	assert(module_name[0]);
-	FileName f(module_name, 0);
+	FileName f((char *)module_name, 0);
 	File file(&f);
 	file.readv();
 	buf = file.buffer;
@@ -315,6 +385,9 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 
     if (buflen < 16)
     {
+    	#if LOG
+ 		printf("buf = %p, buflen = %d\n", buf, buflen);
+ 		#endif
       Lcorrupt:
 	error("corrupt object module %s %d", module_name, reason);
 	return;
@@ -325,6 +398,9 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 	 * Pull each object module out of the library and add it
 	 * to the object module array.
 	 */
+	 #if LOG
+ 	printf("archive, buf = %p, buflen = %d\n", buf, buflen);
+ 	#endif
 	unsigned offset = 8;
 	char *symtab = NULL;
 	unsigned symtab_size = 0;
@@ -342,9 +418,13 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 	    char *endptr = NULL;
 	    unsigned long size = strtoul(header->file_size, &endptr, 10);
 	    if (endptr >= &header->file_size[10] || *endptr != ' ')
-		goto Lcorrupt;
+		{	reason = 2;
+  		goto Lcorrupt;
+ 	    }
 	    if (offset + size > buflen)
-		goto Lcorrupt;
+		{	reason = 3;
+  		goto Lcorrupt;
+ 	    }
 
 	    if (header->object_name[0] == '/' &&
 		header->object_name[1] == ' ')
@@ -353,26 +433,32 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 		 * library, just use the already created symbol table.
 		 */
 		if (symtab)
-		    goto Lcorrupt;
+		{   reason = 4;
+  		    goto Lcorrupt;
+ 		}
 		symtab = (char *)buf + offset;
 		symtab_size = size;
 		if (size < 4)
-		    goto Lcorrupt;
-	    }
+		    {   reason = 5;
+  		    goto Lcorrupt;
+  		}
+ 	    }
 	    else if (header->object_name[0] == '/' &&
 		     header->object_name[1] == '/')
 	    {
 		/* This is the file name table, save it for later.
 		 */
 		if (filenametab)
-		    goto Lcorrupt;
+		{   reason = 6;
+  		    goto Lcorrupt;
+ 		}
 		filenametab = (char *)buf + offset;
 		filenametab_size = size;
 	    }
 	    else
 	    {
 		ObjModule *om = new ObjModule();
-		om->base = (unsigned char *)buf + offset - sizeof(Header);
+		om->base = (unsigned char *)buf + offset /*- sizeof(Header)*/;
 		om->length = size;
 		om->offset = 0;
 		if (header->object_name[0] == '/')
@@ -382,7 +468,9 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 		    unsigned i;
 		    for (i = 0; 1; i++)
 		    {	if (foff + i >= filenametab_size)
-			    goto Lcorrupt;
+			    {   reason = 7;
+  			    goto Lcorrupt;
+ 			}
 			char c = filenametab[foff + i];
 			if (c == '/')
 			    break;
@@ -399,7 +487,9 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 		    assert(om->name);
 		    for (int i = 0; 1; i++)
 		    {	if (i == OBJECT_NAME_SIZE)
-			    goto Lcorrupt;
+			    {   reason = 8;
+  			    goto Lcorrupt;
+ 			}
 			char c = header->object_name[i];
 			if (c == '/')
 			{   om->name[i] = 0;
@@ -419,7 +509,9 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 	    offset += (size + 1) & ~1;
 	}
 	if (offset != buflen)
-	    goto Lcorrupt;
+	    {   reason = 9;
+  	    goto Lcorrupt;
+ 	}
 
 	/* Scan the library's symbol table, and insert it into our own.
 	 * We use this instead of rescanning the object module, because
@@ -430,22 +522,30 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
 	unsigned nsymbols = sgetl(symtab);
 	char *s = symtab + 4 + nsymbols * 4;
 	if (4 + nsymbols * (4 + 1) > symtab_size)
-	    goto Lcorrupt;
+	    {   reason = 10;
+  	    goto Lcorrupt;
+ 	}
 	for (unsigned i = 0; i < nsymbols; i++)
 	{   char *name = s;
 	    s += strlen(name) + 1;
 	    if (s - symtab > symtab_size)
-		goto Lcorrupt;
+		{	reason = 11;
+  		goto Lcorrupt;
+ 	    }
 	    unsigned moff = sgetl(symtab + 4 + i * 4);
+	    //printf("symtab[%d] moff = %x  %x, name = %s\n", i, moff, moff + sizeof(Header), name);
 	    for (unsigned m = mstart; 1; m++)
 	    {	if (m == objmodules.dim)
-		    goto Lcorrupt;		// didn't find it
+		    {   reason = 12;
+  		    goto Lcorrupt;		// didn't find it
+ 		}
 		ObjModule *om = (ObjModule *)objmodules.data[m];
+		//printf("\t%x\n", (char *)om->base - (char *)buf);
 		if (moff + sizeof(Header) == (char *)om->base - (char *)buf)
 		{
 		    addSymbol(om, name, 1);
-		    if (mstart == m)
-			mstart++;
+//		    if (mstart == m)
+ //			mstart++;
 		    break;
 		}
 	    }
@@ -455,7 +555,7 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
     }
 
     if (memcmp(buf, elf, 4) != 0)
-    {	reason = 2;
+    {	reason = 13;
 	goto Lcorrupt;
     }
 
@@ -472,7 +572,7 @@ void Library::addObject(char *module_name, void *buf, size_t buflen)
     {   struct stat statbuf;
 	int i = stat(module_name, &statbuf);
 	if (i == -1)		// error, errno is set
-	{   reason = 3;
+	{   reason = 14;
 	    goto Lcorrupt;
 	}
 	om->file_time = statbuf.st_ctime;
@@ -587,7 +687,7 @@ void Library::WriteLibToBuffer(OutBuffer *libbuf)
     om.base = NULL;
     om.length = hoffset - (8 + sizeof(Header));
     om.offset = 8;
-    om.name = "";
+    om.name = (char*)"";
     ::time(&om.file_time);
     om.user_id = 0;
     om.group_id = 0;
