@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 2008-2008 by Digital Mars
+// Copyright (c) 2008-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -26,7 +26,7 @@ void browse(const char *url)
 
 #endif
 
-#if linux
+#if linux || __FreeBSD__ || __sun&&__SVR4
 
 #include	<sys/types.h>
 #include	<sys/wait.h>
@@ -41,7 +41,7 @@ void browse(const char *url)
     if (browser)
 	browser = strdup(browser);
     else
-	browser = "firefox";
+	browser = "x-www-browser";
 
     args[0] = browser;
     args[1] = url;
@@ -58,3 +58,41 @@ void browse(const char *url)
 
 #endif
 
+#if __APPLE__
+ 
+ #include	<sys/types.h>
+ #include	<sys/wait.h>
+ #include	<unistd.h>
+ 
+ void browse(const char *url)
+ {
+     pid_t childpid;
+     const char *args[5];
+ 
+     char *browser = getenv("BROWSER");
+     if (browser)
+     {	browser = strdup(browser);
+ 	args[0] = browser;
+ 	args[1] = url;
+ 	args[2] = NULL;
+     }
+     else
+     {
+ 	//browser = "/Applications/Safari.app/Contents/MacOS/Safari";
+ 	args[0] = "open";
+ 	args[1] = "-a";
+ 	args[2] = "/Applications/Safari.app";
+ 	args[3] = url;
+ 	args[4] = NULL;
+     }
+ 
+    childpid = fork();
+     if (childpid == 0)
+     {
+ 	execvp(args[0], (char**)args);
+ 	perror(args[0]);		// failed to execute
+ 	return;
+     }
+ }
+ 
+#endif
