@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -15,22 +15,15 @@
 #include <limits.h>
 
 #ifndef IN_GCC
-#if _WIN32
-#include <windows.h>
-long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep);
-#endif
 
-#if __DMC__
-#include <dos.h>
-#endif
-
-#if linux
+#if linux || __APPLE__ || __FreeBSD__ || __sun&&__SVR4
 #include <errno.h>
 #endif
 #endif
 
-#include "mem.h"
+#include "rmem.h"
 #include "root.h"
+#include "async.h"
 
 #include "mars.h"
 #include "module.h"
@@ -41,6 +34,12 @@ long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep);
 #include "lexer.h"
 #include "lib.h"
 
+#if WINDOWS_SEH
+#include <windows.h>
+long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep);
+#endif
+
+int response_expand(int *pargc, char ***pargv);
 void browse(const char *url);
 void getenv_setargv(const char *envvar, int *pargc, char** *pargv);
 
@@ -58,10 +57,11 @@ Global::Global()
     ddoc_ext = "ddoc";
 
 #ifndef IN_GCC
-#if _WIN32
+#if TARGET_WINDOS
     obj_ext  = "obj";
-#elif linux
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
     obj_ext  = "o";
+#elif TARGET_NET
 #else
 #error "fix this"
 #endif
@@ -70,10 +70,11 @@ Global::Global()
 #endif
 
 #ifndef IN_GCC
-#if _WIN32
+#if TARGET_WINDOS
     lib_ext  = "lib";
-#elif linux
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
     lib_ext  = "a";
+#elif TARGET_NET
 #else
 #error "fix this"
 #endif
@@ -81,9 +82,13 @@ Global::Global()
     lib_ext = "a";
 #endif
 
-    copyright = "Copyright (c) 1999-2008 by Digital Mars";
-    written = "written by Walter Bright";
-    version = "v1.030";
+    copyright = "Copyright (c) 1999-2009 by Digital Mars";
+    written = "written by Walter Bright"
+#if TARGET_NET
+    "\nMSIL back-end (alpha release) by Cristian L. Vlasceanu and associates.";
+#endif
+    ;
+    version = "v1.046";
     global.structalign = 8;
 
     memset(&params, 0, sizeof(Param));
@@ -115,6 +120,11 @@ Loc::Loc(Module *mod, unsigned linnum)
     this->filename = mod ? mod->srcfile->toChars() : NULL;
 }
 
+bool Loc::equals(const Loc& loc)
+{
+    return linnum == loc.linnum && FileName::equals(filename, loc.filename);
+}
+
 /**************************************
  * Print error message and exit.
  */
@@ -127,6 +137,28 @@ void error(Loc loc, const char *format, ...)
     va_end( ap );
 }
 
+void error(const char *filename, unsigned linnum, const char *format, ...)
+{   Loc loc;
+    loc.filename = (char *)filename;
+    loc.linnum = linnum;
+    va_list ap;
+    va_start(ap, format);
+    verror(loc, format, ap);
+    va_end( ap );
+}
+
+void warning(Loc loc, const char *format, ...)
+{
+    if (global.params.warnings && !global.gag)
+    {
+	fprintf(stdmsg, "warning - ");
+ 	va_list ap;
+ 	va_start(ap, format);
+ 	verror(loc, format, ap);
+ 	va_end( ap );
+    }
+}
+
 void verror(Loc loc, const char *format, va_list ap)
 {
     if (!global.gag)
@@ -138,9 +170,17 @@ void verror(Loc loc, const char *format, va_list ap)
 	mem.free(p);
 
 	fprintf(stdmsg, "Error: ");
-	vfprintf(stdmsg, format, ap);
+	#if _MSC_VER
+ 	// MS doesn't recognize %zu format
+ 	OutBuffer tmp;
+ 	tmp.vprintf(format, ap);
+ 	fprintf(stdmsg, "%s", tmp.toChars());
+ 	#else
+  	vfprintf(stdmsg, format, ap);
+ 	#endif
 	fprintf(stdmsg, "\n");
 	fflush(stdmsg);
+	halt();
     }
     global.errors++;
 }
@@ -183,7 +223,8 @@ Documentation: http://www.digitalmars.com/d/1.0/index.html\n\
 Usage:\n\
   dmd files.d ... { -switch }\n\
 \n\
-  files.d        D source files\n%s\
+  files.d        D source files\n\
+  @cmdfile       read arguments from cmdfile\n\
   -c             do not link\n\
   -cov           do code coverage analysis\n\
   -D             generate documentation\n\
@@ -195,6 +236,7 @@ Usage:\n\
   -debug=ident   compile in debug code identified by ident\n\
   -debuglib=name    set symbolic debug library to name\n\
   -defaultlib=name  set default library to name\n\
+  -deps=filename write module dependencies to filename\n\
   -g             add symbolic debug info\n\
   -gc            add symbolic debug info, pretend to be C\n\
   -H             generate 'header' file\n\
@@ -224,13 +266,7 @@ Usage:\n\
   -version=level compile in version code >= level\n\
   -version=ident compile in version code identified by ident\n\
   -w             enable warnings\n\
-",
-#if WIN32
-"  @cmdfile       read arguments from cmdfile\n"
-#else
-""
-#endif
-);
+");
 }
 
 int main(int argc, char *argv[])
@@ -242,6 +278,7 @@ int main(int argc, char *argv[])
     Module *m;
     int status = EXIT_SUCCESS;
     int argcstart = argc;
+    int setdebuglib = 0;
 
     // Check for malformed input
     if (argc < 1 || !argv)
@@ -256,10 +293,8 @@ int main(int argc, char *argv[])
 	    goto Largs;
     }
 
-#if __DMC__	// DMC unique support for response files
     if (response_expand(&argc,&argv))	// expand response files
 	error("can't open response file");
-#endif
 
     files.reserve(argc - 1);
 
@@ -282,39 +317,63 @@ int main(int argc, char *argv[])
     global.params.objfiles = new Array();
     global.params.ddocfiles = new Array();
 
-#if _WIN32
+#if TARGET_WINDOS
     global.params.defaultlibname = "phobos";
-#elif linux
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
     global.params.defaultlibname = "phobos";
+#elif TARGET_NET
+#else
+#error "fix this"
 #endif
-    global.params.debuglibname = global.params.defaultlibname;
 
     // Predefine version identifiers
     VersionCondition::addPredefinedGlobalIdent("DigitalMars");
-#if _WIN32
+    
+#if TARGET_WINDOS
     VersionCondition::addPredefinedGlobalIdent("Windows");
-    VersionCondition::addPredefinedGlobalIdent("Win32");
     global.params.isWindows = 1;
-#endif
-#if linux
+#elif TARGET_LINUX
+	VersionCondition::addPredefinedGlobalIdent("Posix");
     VersionCondition::addPredefinedGlobalIdent("linux");
     global.params.isLinux = 1;
-#endif /* linux */
+#elif TARGET_OSX
+     VersionCondition::addPredefinedGlobalIdent("Posix");
+     VersionCondition::addPredefinedGlobalIdent("OSX");
+     global.params.isOSX = 1;
+ 
+     // For legacy compatibility
+     VersionCondition::addPredefinedGlobalIdent("darwin");
+#elif TARGET_FREEBSD
+     VersionCondition::addPredefinedGlobalIdent("Posix");
+     VersionCondition::addPredefinedGlobalIdent("FreeBSD");
+     global.params.isFreeBSD = 1;
+#elif TARGET_SOLARIS
+    VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("Solaris");
+    global.params.isSolaris = 1;
+#else
+#error "fix this"
+#endif
+
+#if TARGET_NET
+    // TARGET_NET macro is NOT mutually-exclusive with TARGET_WINDOS
+    VersionCondition::addPredefinedGlobalIdent("D_NET");
+#endif
+
     VersionCondition::addPredefinedGlobalIdent("X86");
     VersionCondition::addPredefinedGlobalIdent("LittleEndian");
     //VersionCondition::addPredefinedGlobalIdent("D_Bits");
-    VersionCondition::addPredefinedGlobalIdent("D_InlineAsm");
-    VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86");
-#if V2
+#if DMDV2
     VersionCondition::addPredefinedGlobalIdent("D_Version2");
 #endif
     VersionCondition::addPredefinedGlobalIdent("all");
 
 #if _WIN32
     inifile(argv[0], "sc.ini");
-#endif
-#if linux
+#elif linux || __APPLE__ || __FreeBSD__ || __sun&&__SVR4
     inifile(argv[0], "dmd.conf");
+#else
+#error "fix this"
 #endif
     getenv_setargv("DFLAGS", &argc, &argv);
 
@@ -336,8 +395,10 @@ int main(int argc, char *argv[])
 		global.params.link = 0;
 	    else if (strcmp(p + 1, "cov") == 0)
 		global.params.cov = 1;
+		#if TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
 	    else if (strcmp(p + 1, "fPIC") == 0)
 		global.params.pic = 1;
+		#endif
 	    else if (strcmp(p + 1, "multiobj") == 0)
 		global.params.multiobj = 1;
 	    else if (strcmp(p + 1, "g") == 0)
@@ -348,13 +409,19 @@ int main(int argc, char *argv[])
 	    {	error("use -profile instead of -gt\n");
 		global.params.trace = 1;
 	    }
+	    else if (strcmp(p + 1, "m64") == 0)
+ 		global.params.isX86_64 = 1;
 	    else if (strcmp(p + 1, "profile") == 0)
 		global.params.trace = 1;
 	    else if (strcmp(p + 1, "v") == 0)
 		global.params.verbose = 1;
+		#if DMDV2
+ 	    else if (strcmp(p + 1, "vtls") == 0)
+ 		global.params.vtls = 1;
+		#endif
 	    else if (strcmp(p + 1, "v1") == 0)
 	    {
-#if V1
+#if DMDV1
 		global.params.Dversion = 1;
 #else
 		error("use DMD 1.0 series compilers for -v1 switch");
@@ -458,6 +525,10 @@ int main(int argc, char *argv[])
 		global.params.quiet = 1;
 	    else if (strcmp(p + 1, "release") == 0)
 		global.params.release = 1;
+		#if DMDV2
+ 	    else if (strcmp(p + 1, "safe") == 0)
+ 		global.params.safe = 1;
+ 		#endif
 	    else if (strcmp(p + 1, "unittest") == 0)
 		global.params.useUnitTests = 1;
 	    else if (p[1] == 'I')
@@ -549,23 +620,45 @@ int main(int argc, char *argv[])
 	    }
 	    else if (memcmp(p + 1, "debuglib=", 9) == 0)
 	    {
+	    setdebuglib = 1;
 		global.params.debuglibname = p + 1 + 9;
 	    }
+	    else if (memcmp(p + 1, "deps=", 5) == 0)
+ 	    {
+ 		global.params.moduleDepsFile = p + 1 + 5;
+ 		if (!global.params.moduleDepsFile[0])
+ 		    goto Lnoarg;
+ 		global.params.moduleDeps = new OutBuffer;
+ 	    }
 	    else if (memcmp(p + 1, "man", 3) == 0)
 	    {
 #if _WIN32
-#if V1
+#if DMDV1
 		browse("http://www.digitalmars.com/d/1.0/dmd-windows.html");
 #else
 		browse("http://www.digitalmars.com/d/2.0/dmd-windows.html");
 #endif
 #endif
 #if linux
-#if V1
+#if DMDV1
 		browse("http://www.digitalmars.com/d/1.0/dmd-linux.html");
 #else
 		browse("http://www.digitalmars.com/d/2.0/dmd-linux.html");
 #endif
+#endif
+#if __APPLE__
+#if DMDV1
+		browse("http://www.digitalmars.com/d/1.0/dmd-osx.html");
+#else
+ 		browse("http://www.digitalmars.com/d/2.0/dmd-osx.html");
+#endif
+#endif
+#if __FreeBSD__
+#if DMDV1
+ 		browse("http://www.digitalmars.com/d/1.0/dmd-freebsd.html");
+#else
+		browse("http://www.digitalmars.com/d/2.0/dmd-freebsd.html");
+#endif
 #endif
 		exit(EXIT_SUCCESS);
 	    }
@@ -597,9 +690,9 @@ int main(int argc, char *argv[])
 	}
 	else
 	{
-#if !TARGET_LINUX
+#if !TARGET_LINUX && !TARGET_OSX && !TARGET_FREEBSD
 	    char *ext = FileName::ext(p);
-	    if (ext && stricmp(ext, "exe") == 0)
+	    if (ext && FileName::compare(ext, "exe") == 0)
 	    {
 		global.params.objname = p;
 		continue;
@@ -616,6 +709,12 @@ int main(int argc, char *argv[])
     {	usage();
 	return EXIT_FAILURE;
     }
+    
+    if (!setdebuglib)
+ 	global.params.debuglibname = global.params.defaultlibname;
+    #if TARGET_OSX
+     global.params.pic = 1;
+ 	#endif
 
     if (global.params.release)
     {	global.params.useInvariants = 0;
@@ -678,9 +777,31 @@ int main(int argc, char *argv[])
 	    //fatal();
 	}
     }
+    if (global.params.isX86_64)
+     {
+ 	VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86_64");
+ 	VersionCondition::addPredefinedGlobalIdent("X86_64");
+ 	VersionCondition::addPredefinedGlobalIdent("D_LP64");
+ #if TARGET_WINDOS
+ 	VersionCondition::addPredefinedGlobalIdent("Win64");
+ #endif
+     }
+     else
+     {
+ 	VersionCondition::addPredefinedGlobalIdent("D_InlineAsm");
+ 	VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86");
+ 	VersionCondition::addPredefinedGlobalIdent("X86");
+ #if TARGET_WINDOSs
+ 	VersionCondition::addPredefinedGlobalIdent("Win32");
+ #endif
+     }
+     if (global.params.doDocComments)
+ 	VersionCondition::addPredefinedGlobalIdent("D_Ddoc");
     if (global.params.cov)
 	VersionCondition::addPredefinedGlobalIdent("D_Coverage");
-#if V2
+	if (global.params.pic)
+ 	VersionCondition::addPredefinedGlobalIdent("D_PIC");
+#if DMDV2
     if (global.params.useUnitTests)
 	VersionCondition::addPredefinedGlobalIdent("unittest");
 #endif
@@ -754,22 +875,14 @@ int main(int argc, char *argv[])
 	if (ext)
 	{   /* Deduce what to do with a file based on its extension
 	     */
-#if TARGET_LINUX
-	    if (strcmp(ext, global.obj_ext) == 0)
-#else
-	    if (stricmp(ext, global.obj_ext) == 0)
-#endif
+		if (FileName::equals(ext, global.obj_ext))
 	    {
 		global.params.objfiles->push(files.data[i]);
 		libmodules.push(files.data[i]);
 		continue;
 	    }
 
-#if TARGET_LINUX
-	    if (strcmp(ext, global.lib_ext) == 0)
-#else
-	    if (stricmp(ext, global.lib_ext) == 0)
-#endif
+		if (FileName::equals(ext, global.lib_ext))
 	    {
 		global.params.libfiles->push(files.data[i]);
 		libmodules.push(files.data[i]);
@@ -782,20 +895,20 @@ int main(int argc, char *argv[])
 		continue;
 	    }
 
-#if !TARGET_LINUX
-	    if (stricmp(ext, "res") == 0)
+#if TARGET_WINDOS
+	    if (FileName::equals(ext, "res"))
 	    {
 		global.params.resfile = (char *)files.data[i];
 		continue;
 	    }
 
-	    if (stricmp(ext, "def") == 0)
+	    if (FileName::equals(ext, "def"))
 	    {
 		global.params.deffile = (char *)files.data[i];
 		continue;
 	    }
 
-	    if (stricmp(ext, "exe") == 0)
+	    if (FileName::equals(ext, "exe"))
 	    {
 		assert(0);	// should have already been handled
 	    }
@@ -804,11 +917,12 @@ int main(int argc, char *argv[])
 	    /* Examine extension to see if it is a valid
 	     * D source file extension
 	     */
-	    if (stricmp(ext, global.mars_ext) == 0 ||
-		stricmp(ext, "dd") == 0 ||
-		stricmp(ext, "htm") == 0 ||
-		stricmp(ext, "html") == 0 ||
-		stricmp(ext, "xhtml") == 0)
+	    if (FileName::equals(ext, global.mars_ext) ||
+ 		FileName::equals(ext, global.hdr_ext) ||
+ 		FileName::equals(ext, "dd") ||
+ 		FileName::equals(ext, "htm") ||
+ 		FileName::equals(ext, "html") ||
+ 		FileName::equals(ext, "xhtml"))
 	    {
 		ext--;			// skip onto '.'
 		assert(*ext == '.');
@@ -850,11 +964,31 @@ int main(int argc, char *argv[])
 	}
     }
 
-#if _WIN32
+#if WINDOWS_SEH
   __try
   {
 #endif
-    // Read files, parse them
+    // Read files
+#define ASYNCREAD 1
+#if ASYNCREAD
+    // Multi threaded
+    AsyncRead *aw = AsyncRead::create(modules.dim);
+    for (i = 0; i < modules.dim; i++)
+    {
+	m = (Module *)modules.data[i];
+ 	aw->addFile(m->srcfile);
+    }
+    aw->start();
+#else
+    // Single threaded
+    for (i = 0; i < modules.dim; i++)
+    {
+	m = (Module *)modules.data[i];
+	m->read(0);
+    }
+#endif
+
+    // Parse files
     int anydocfiles = 0;
     for (i = 0; i < modules.dim; i++)
     {
@@ -866,7 +1000,12 @@ int main(int argc, char *argv[])
 	m->importedFrom = m;
 	if (!global.params.oneobj || i == 0 || m->isDocFile)
 	    m->deleteObjFile();
-	m->read(0);
+	#if ASYNCREAD
+ 	if (aw->read(i))
+ 	{
+ 	    error("cannot read file %s", m->srcfile->name->toChars());
+ 	}
+	#endif
 	m->parse();
 	if (m->isDocFile)
 	{
@@ -891,6 +1030,9 @@ int main(int argc, char *argv[])
 		global.params.link = 0;
 	}
     }
+    #if ASYNCREAD
+    AsyncRead::dispose(aw);
+	#endif
     if (anydocfiles && modules.dim &&
 	(global.params.oneobj || global.params.objname))
     {
@@ -951,6 +1093,16 @@ int main(int argc, char *argv[])
     }
     if (global.errors)
 	fatal();
+	
+	if (global.params.moduleDeps != NULL)
+    {
+	assert(global.params.moduleDepsFile != NULL);
+
+	File deps(global.params.moduleDepsFile);
+	OutBuffer* ob = global.params.moduleDeps;
+	deps.setbuffer((void*)ob->data, ob->offset);
+	deps.writev();
+    }
 
     // Scan for functions to inline
     if (global.params.useInline)
@@ -1048,7 +1200,7 @@ int main(int argc, char *argv[])
     if (global.params.lib && !global.errors)
 	library->write();
 
-#if _WIN32
+#if WINDOWS_SEH
   }
   __except (__ehfilter(GetExceptionInformation()))
   {
@@ -1204,7 +1356,7 @@ Ldone:
     *pargv = (char **)argv->data;
 }
 
-#if _WIN32
+#if WINDOWS_SEH
 
 long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep)
 {
