@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -19,8 +19,55 @@
 
 #ifdef __DMC__
 #pragma once
-#endif /* __DMC__ */
+#endif
+ 
+/*
+ It is very important to use version control macros correctly - the
+ idea is that host and target are independent. If these are done
+ correctly, cross compilers can be built.
+ The host compiler and host operating system are also different,
+ and are predefined by the host compiler. The ones used in
+ dmd are:
+ 
+ Macros defined by the compiler, not the code:
+ 
+     Compiler:
+ 	__DMC__		Digital Mars compiler
+ 	_MSC_VER	Microsoft compiler
+ 	__GNUC__	Gnu compiler
+ 
+     Host operating system:
+ 	_WIN32		Microsoft NT, Windows 95, Windows 98, Win32s,
+ 			Windows 2000, Win XP, Vista
+ 	_WIN64		Windows for AMD64
+ 	linux		Linux
+ 	__APPLE__	Mac OSX
+  __FreeBSD__	FreeBSD
+  __sun&&__SVR4	Solaris, OpenSolaris (yes, both macros are necessary)
+ 
+ For the target systems, there are the target operating system and
+ the target object file format:
+ 
+     Target operating system:
+ 	TARGET_WINDOS	Covers 32 bit windows and 64 bit windows
+ 	TARGET_LINUX	Covers 32 and 64 bit linux
+ 	TARGET_OSX	Covers 32 and 64 bit Mac OSX
+	TARGET_FREEBSD	Covers 32 and 64 bit FreeBSD
+	TARGET_SOLARIS	Covers 32 and 64 bit Solaris
+	TARGET_NET	Covers .Net
+ 
+     It is expected that the compiler for each platform will be able
+     to generate 32 and 64 bit code from the same compiler binary.
+ 
+     Target object module format:
+ 	OMFOBJ		Intel Object Module Format, used on Windows
+ 	ELFOBJ		Elf Object Module Format, used on linux, FreeBSD and Solaris
+ 	MACHOBJ		Mach-O Object Module Format, used on Mac OSX
+ 
+     There are currently no macros for byte endianness order.
+*/
 
+#include <stdio.h>
 #include <stdint.h>
 #include <stdarg.h>
 #define __STDC_FORMAT_MACROS 1
@@ -35,14 +82,42 @@
 #endif
 
 #ifdef IN_GCC
+//To maintain compatibility with GDC2
+#define V1	1
+#define V2	0
 /* Changes for the GDC compiler by David Friedman */
 #endif
 
-#define V1	1
-#define V2	0	// Version 2.0 features
+#define DMDV1	1
+#define DMDV2	0	// Version 2.0 features
 #define BREAKABI 1	// 0 if not ready to break the ABI just yet
+#define STRUCTTHISREF DMDV2	// if 'this' for struct is a reference, not a pointer
+#define SNAN_DEFAULT_INIT DMDV2	// if floats are default initialized to signalling NaN
+ 
+/* Other targets are TARGET_LINUX, TARGET_OSX, TARGET_FREEBSD and
+ * TARGET_SOLARIS, which are
+ * set on the command line via the compiler makefile.
+ */
+ 
+#if _WIN32
+ #define TARGET_WINDOS 1		// Windows dmd generates Windows targets
+ #define OMFOBJ 1
+#endif
+ 
+#if TARGET_LINUX || TARGET_FREEBSD || TARGET_SOLARIS
+#ifndef ELFOBJ
+ #define ELFOBJ 1
+#endif
+#endif
+ 
+#if TARGET_OSX
+#ifndef MACHOBJ
+ #define MACHOBJ 1
+#endif
+#endif
 
 struct Array;
+struct OutBuffer;
 
 // Put command line switches in here
 struct Param
@@ -55,12 +130,16 @@ struct Param
     char trace;		// insert profiling hooks
     char quiet;		// suppress non-error messages
     char verbose;	// verbose compile
+    char vtls;		// identify thread local variabless
     char symdebug;	// insert debug symbolic information
     char optimize;	// run optimizer
     char cpu;		// target CPU
     char isX86_64;	// generate X86_64 bit code
     char isLinux;	// generate code for linux
+    char isOSX;		// generate code for Mac OSX
     char isWindows;	// generate code for Windows
+    char isFreeBSD;	// generate code for FreeBSD
+    char isSolaris;	// generate code for Solaris
     char scheduler;	// which scheduler to use
     char useDeprecated;	// allow use of deprecated features
     char useAssert;	// generate runtime code for assert()'s
@@ -79,6 +158,7 @@ struct Param
     char nofloat;	// code should not pull in floating point support
     char Dversion;	// D version number
     char ignoreUnsupportedPragmas;	// rather than error on them
+    char safe;		// enforce safe memory model
 
     char *argv0;	// program name
     Array *imppath;	// array of char*'s of where to look for import modules
@@ -104,10 +184,13 @@ struct Param
 
     bool dump_source;
 
-    char *defaultlibname;	// default library for non-debug builds
-    char *debuglibname;		// default library for debug builds
-
-    char *xmlname;		// filename for XML output
+    const char *defaultlibname;	// default library for non-debug builds
+    const char *debuglibname;	// default library for debug builds
+  
+    const char *xmlname;	// filename for XML output
+    
+    char *moduleDepsFile;	// filename for deps output
+    OutBuffer *moduleDeps;	// contents to be written to deps file
 
     // Hidden debug switches
     char debuga;
@@ -134,19 +217,19 @@ struct Param
 
 struct Global
 {
-    char *mars_ext;
-    char *sym_ext;
-    char *obj_ext;
-    char *lib_ext;
-    char *doc_ext;	// for Ddoc generated files
-    char *ddoc_ext;	// for Ddoc macro include files
-    char *hdr_ext;	// for D 'header' import files
-    char *copyright;
-    char *written;
+    const char *mars_ext;
+    const char *sym_ext;
+    const char *obj_ext;
+    const char *lib_ext;
+    const char *doc_ext;	// for Ddoc generated files
+    const char *ddoc_ext;	// for Ddoc macro include files
+    const char *hdr_ext;	// for D 'header' import files
+    const char *copyright;
+    const char *written;
     Array *path;	// Array of char*'s which form the import lookup path
     Array *filePath;	// Array of char*'s which form the file import lookup path
     int structalign;
-    char *version;
+    const char *version;
 
     Param params;
     unsigned errors;	// number of errors reported so far
@@ -157,10 +240,10 @@ struct Global
 
 extern Global global;
 
-#if __GNUC__
-//#define memicmp strncasecmp
-//#define stricmp strcasecmp
-#endif
+/* Set if Windows Structured Exception Handling C extensions are supported.
++  * Apparently, VC has dropped support for these?
++  */
+ #define WINDOWS_SEH	(_WIN32 && __DMC__)
 
 #ifdef __DMC__
  typedef _Complex long double complex_t;
@@ -170,12 +253,13 @@ extern Global global;
  #endif
  #ifdef __APPLE__
   //#include "complex.h"//This causes problems with include the c++ <complex> and not the C "complex.h"
-  #define integer_t dmd_integer_t
  #endif
 #endif
 
-// Be careful not to care about sign when using integer_t
-typedef uint64_t integer_t;
+// Be careful not to care about sign when using dinteger_t
+//typedef uint64_t integer_t;
+typedef uint64_t dinteger_t;	// use this instead of integer_t to
+				// avoid conflicts with system #include's
 
 // Signed and unsigned variants
 typedef int64_t sinteger_t;
@@ -237,7 +321,7 @@ struct Module;
 //typedef unsigned Loc;		// file location
 struct Loc
 {
-    char *filename;
+    const char *filename;
     unsigned linnum;
 
     Loc()
@@ -255,6 +339,7 @@ struct Loc
     Loc(Module *mod, unsigned linnum);
 
     char *toChars();
+    bool equals(const Loc& loc);
 };
 
 #ifndef GCC_SAFE_DMD
@@ -292,12 +377,13 @@ enum MATCH
 {
     MATCHnomatch,	// no match
     MATCHconvert,	// match with conversions
-#if V2
+#if DMDV2
     MATCHconst,		// match with conversion to const
 #endif
     MATCHexact		// exact match
 };
 
+void warning(Loc loc, const char *format, ...);
 void error(Loc loc, const char *format, ...);
 void verror(Loc loc, const char *format, va_list);
 void fatal();
@@ -305,14 +391,15 @@ void err_nomem();
 int runLINK();
 void deleteExeFile();
 int runProgram();
-void inifile(char *argv0, char *inifile);
+void inifile(const char *argv0, const char *inifile);
 void halt();
+void util_progress();
 
 /*** Where to send error messages ***/
 #if IN_GCC
 #define stdmsg stderr
 #else
-#define stdmsg stdout
+#define stdmsg stderr
 #endif
 
 struct Dsymbol;
