@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -12,7 +12,11 @@
 #include <stdlib.h>
 #include <assert.h>
 
-#ifdef _MSC_VER
+#if (defined (__SVR4) && defined (__sun))
+#include <alloca.h>
+#endif 
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #include <malloc.h>
 #endif
 
@@ -20,7 +24,7 @@
 #include "gdc_alloca.h"
 #endif
 
-#include "mem.h"
+#include "rmem.h"
 
 #include "mars.h"
 #include "module.h"
@@ -75,12 +79,13 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
 #ifdef IN_GCC
     strictlyneedmoduleinfo = 0;
 #endif
+	selfimports = 0;
     insearch = 0;
     searchCacheIdent = NULL;
     searchCacheSymbol = NULL;
     searchCacheFlags = 0;
     semanticstarted = 0;
-    semanticdone = 0;
+    semanticRun = 0;
     decldefs = NULL;
     vmoduleinfo = NULL;
     massert = NULL;
@@ -110,6 +115,7 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
 
     srcfilename = FileName::defaultExt(filename, global.mars_ext);
     if (!srcfilename->equalsExt(global.mars_ext) &&
+    	!srcfilename->equalsExt(global.hdr_ext) &&
         !srcfilename->equalsExt("dd"))
     {
 	if (srcfilename->equalsExt("html") ||
@@ -225,7 +231,7 @@ Module::~Module()
 {
 }
 
-char *Module::kind()
+const char *Module::kind()
 {
     return "module";
 }
@@ -661,25 +667,32 @@ void Module::semantic()
     // Add all symbols into module's symbol table
     symtab = new DsymbolTable();
     for (i = 0; i < members->dim; i++)
-    {	Dsymbol *s;
-
-	s = (Dsymbol *)members->data[i];
+    {	Dsymbol *s = (Dsymbol *)members->data[i];
 	s->addMember(NULL, sc->scopesym, 1);
     }
+    
+    /* Set scope for the symbols so that if we forward reference
+     * a symbol, it can possibly be resolved on the spot.
+     * If this works out well, it can be extended to all modules
+     * before any semantic() on any of them.
+     */
+    for (i = 0; i < members->dim; i++)
+    {	Dsymbol *s = (Dsymbol *)members->data[i];
+	s->setScope(sc);
+    }
 
     // Pass 1 semantic routines: do public side of the definition
     for (i = 0; i < members->dim; i++)
-    {	Dsymbol *s;
+    {	Dsymbol *s = (Dsymbol *)members->data[i];
 
-	s = (Dsymbol *)members->data[i];
 	//printf("\tModule('%s'): '%s'.semantic()\n", toChars(), s->toChars());
 	s->semantic(sc);
 	runDeferredSemantic();
     }
 
     sc = sc->pop();
-    sc->pop();
-    semanticdone = semanticstarted;
+    sc->pop();		// 2 pops because Scope::createGlobal() created 2
+    semanticRun = semanticstarted;
     //printf("-Module::semantic(this = %p, '%s'): parent = %p\n", this, toChars(), parent);
 }
 
@@ -718,7 +731,7 @@ void Module::semantic2()
 
     sc = sc->pop();
     sc->pop();
-    semanticdone = semanticstarted;
+    semanticRun = semanticstarted;
     //printf("-Module::semantic2('%s'): parent = %p\n", toChars(), parent);
 }
 
@@ -748,7 +761,7 @@ void Module::semantic3()
 
     sc = sc->pop();
     sc->pop();
-    semanticdone = semanticstarted;
+    semanticRun = semanticstarted;
 }
 
 void Module::inlineScan()
@@ -773,7 +786,7 @@ void Module::inlineScan()
 
 	s->inlineScan();
     }
-    semanticdone = semanticstarted;
+    semanticRun = semanticstarted;
 }
 
 /****************************************************
@@ -905,6 +918,63 @@ void Module::runDeferredSemantic()
     //printf("-Module::runDeferredSemantic('%s'), len = %d\n", toChars(), deferred.dim);
 }
 
+/************************************
++  * Recursively look at every module this module imports,
++  * return TRUE if it imports m.
++  * Can be used to detect circular imports.
++  */
+ 
+ int Module::imports(Module *m)
+ {
+     //printf("%s Module::imports(%s)\n", toChars(), m->toChars());
+     int aimports_dim = aimports.dim;
+ #if 0
+     for (int i = 0; i < aimports.dim; i++)
+     {	Module *mi = (Module *)aimports.data[i];
+ 	printf("\t[%d] %s\n", i, mi->toChars());
+     }
+ #endif
+     for (int i = 0; i < aimports.dim; i++)
+     {	Module *mi = (Module *)aimports.data[i];
+ 	if (mi == m)
+ 	    return TRUE;
+ 	if (!mi->insearch)
+ 	{
+ 	    mi->insearch = 1;
+ 	    int r = mi->imports(m);
+ 	    if (r)
+ 		return r;
+ 	}
+     }
+     return FALSE;
+ }
+ 
+/*************************************
+  * Return !=0 if module imports itself.
+  */
+ 
+int Module::selfImports()
+{
+    //printf("Module::selfImports() %s\n", toChars());
+    if (!selfimports)
+    {
+ 	for (int i = 0; i < amodules.dim; i++)
+ 	{   Module *mi = (Module *)amodules.data[i];
+ 	    //printf("\t[%d] %s\n", i, mi->toChars());
+ 	    mi->insearch = 0;
+ 	}
+ 
+ 	selfimports = imports(this) + 1;
+ 
+ 	for (int i = 0; i < amodules.dim; i++)
+ 	{   Module *mi = (Module *)amodules.data[i];
+ 	    //printf("\t[%d] %s\n", i, mi->toChars());
+ 	    mi->insearch = 0;
+ 	}
+    }
+    return selfimports - 1;
+}
+
 /* =========================== ModuleDeclaration ===================== */
 
 ModuleDeclaration::ModuleDeclaration(Array *packages, Identifier *id)
@@ -940,7 +1010,7 @@ Package::Package(Identifier *ident)
 }
 
 
-char *Package::kind()
+const char *Package::kind()
 {
     return "package";
 }
