@@ -40,10 +40,12 @@ struct TypedefDeclaration;
 struct TypeInfoDeclaration;
 struct Dsymbol;
 struct TemplateInstance;
+struct CppMangleState;
 enum LINK;
 
 struct TypeBasic;
 struct HdrGenState;
+struct Argument;
 
 // Back end
 #if IN_GCC
@@ -104,6 +106,7 @@ enum TY
     Ttypeof,
     Ttuple,
     Tslice,
+    Treturn,
     TMAX
 };
 
@@ -118,10 +121,12 @@ extern int Tindex;
 struct Type : Object
 {
     TY ty;
-    unsigned char mod;	// modifiers (MODconst, MODinvariant)
+    unsigned char mod;	// modifiers MODxxxx
+ 	/* pick this order of numbers so switch statements work better
+! 	 */
 	#define MODconst     1	// type is const
-	#define MODinvariant 2	// type is invariant
-    Type *next;
+	#define MODinvariant 4	// type is invariant
+ 	#define MODshared    2	// type is shared
     char *deco;
     Type *pto;		// merged pointer to this type
     Type *rto;		// reference to this type
@@ -210,6 +215,7 @@ struct Type : Object
     virtual Type *semantic(Loc loc, Scope *sc);
     virtual void toDecoBuffer(OutBuffer *buf);
     Type *merge();
+    Type *merge2();
     virtual void toCBuffer(OutBuffer *buf, Identifier *ident, HdrGenState *hgs);
     virtual void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     void toCBuffer3(OutBuffer *buf, HdrGenState *hgs, int mod);
@@ -237,7 +243,7 @@ struct Type : Object
     virtual Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     virtual unsigned memalign(unsigned salign);
     virtual Expression *defaultInit(Loc loc = 0);
-    virtual int isZeroInit();		// if initializer is 0
+    virtual int isZeroInit(Loc loc = 0);		// if initializer is 0
     virtual dt_t **toDt(dt_t **pdt);
     Identifier *getTypeInfoIdent(int internal);
     virtual MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
@@ -249,9 +255,11 @@ struct Type : Object
     virtual Type *reliesOnTident();
     virtual Expression *toExpression();
     virtual int hasPointers();
+    Type *next;
     Type *nextOf() { return next; }
 
     static void error(Loc loc, const char *format, ...);
+    static void warning(Loc loc, const char *format, ...);
 
     // For backend
     virtual unsigned totym();
@@ -265,8 +273,8 @@ struct Type : Object
 
 struct TypeBasic : Type
 {
-    char *dstring;
-    char *cstring;
+    const char *dstring;
+    const char *cstring;
     unsigned flags;
 
     TypeBasic(TY ty);
@@ -287,7 +295,7 @@ struct TypeBasic : Type
     int isunsigned();
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int builtinTypeInfo();
 
     // For eliminating dynamic_cast
@@ -315,7 +323,7 @@ struct TypeSArray : TypeArray
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     int isString();
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     unsigned memalign(unsigned salign);
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
@@ -342,7 +350,7 @@ struct TypeDArray : TypeArray
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     int isString();
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
@@ -368,12 +376,13 @@ struct TypeAArray : TypeArray
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     Expression *defaultInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
+    int isZeroInit(Loc loc);
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
 
     // Back end
-    Symbol *aaGetSymbol(char *func, int flags);
+    Symbol *aaGetSymbol(const char *func, int flags);
 
     type *toCtype();
 };
@@ -388,7 +397,7 @@ struct TypePointer : Type
     MATCH implicitConvTo(Type *to);
     int isscalar();
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
 
@@ -403,7 +412,7 @@ struct TypeReference : Type
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
 };
 
 enum RET
@@ -446,7 +455,7 @@ struct TypeDelegate : Type
     d_uns64 size(Loc loc);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
@@ -499,6 +508,7 @@ struct TypeInstance : TypeQualified
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     void resolve(Loc loc, Scope *sc, Expression **pe, Type **pt, Dsymbol **ps);
     Type *semantic(Loc loc, Scope *sc);
+    Dsymbol *toDsymbol(Scope *sc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
 };
 
@@ -530,7 +540,7 @@ struct TypeStruct : Type
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     unsigned memalign(unsigned salign);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     dt_t **toDt(dt_t **pdt);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
@@ -545,6 +555,7 @@ struct TypeEnum : Type
     EnumDeclaration *sym;
 
     TypeEnum(EnumDeclaration *sym);
+    Type *syntaxCopy();
     d_uns64 size(Loc loc);
     unsigned alignsize();
     char *toChars();
@@ -561,7 +572,7 @@ struct TypeEnum : Type
     MATCH implicitConvTo(Type *to);
     Type *toBasetype();
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
@@ -596,7 +607,7 @@ struct TypeTypedef : Type
     Type *toBasetype();
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     dt_t **toDt(dt_t **pdt);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
@@ -623,12 +634,20 @@ struct TypeClass : Type
     int isBaseOf(Type *t, target_ptrdiff_t *poffset);
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     int isauto();
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+    int builtinTypeInfo();
+#if DMDV2
+    Type *toHeadMutable();
+    MATCH constConv(Type *to);
+#if TARGET_LINUX || TARGET_OSX
+    void toCppMangle(OutBuffer *buf, CppMangleState *cms);
+#endif
+#endif
 
     type *toCtype();
 
