@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -22,19 +22,13 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
+#if _MSC_VER
+#include <complex>
+#else
 #include <complex.h>
-
-#ifdef __APPLE__
-#define integer_t dmd_integer_t
 #endif
 
-#if IN_GCC
-#include "mem.h"
-#elif linux
-#include "../root/mem.h"
-#elif _WIN32
-#include "..\root\mem.h"
-#endif
+#include "rmem.h"
 
 //#include "port.h"
 #include "mtype.h"
@@ -48,7 +42,7 @@
 static Expression *build_overload(Loc loc, Scope *sc, Expression *ethis, Expression *earg, Identifier *id);
 static void inferApplyArgTypesX(FuncDeclaration *fstart, Arguments *arguments);
 static int inferApplyArgTypesY(TypeFunction *tf, Arguments *arguments);
-static void templateResolve(Match *m, TemplateDeclaration *td, Scope *sc, Loc loc, Objects *targsi, Expressions *arguments);
+static void templateResolve(Match *m, TemplateDeclaration *td, Scope *sc, Loc loc, Objects *targsi, Expression *ethis, Expressions *arguments);
 
 /******************************** Expression **************************/
 
@@ -173,6 +167,7 @@ Identifier *ArrayExp::opId()	{ return Id::index; }
 
 Expression *UnaExp::op_overload(Scope *sc)
 {
+    //printf("UnaExp::op_overload() (%s)\n", toChars());
     AggregateDeclaration *ad;
     Dsymbol *fd;
     Type *t1 = e1->type->toBasetype();
@@ -192,10 +187,11 @@ Expression *UnaExp::op_overload(Scope *sc)
 	{
 	    if (op == TOKarray)
 	    {
-		Expression *e;
+		/* Rewrite op e1[arguments] as:
+		 *    e1.fd(arguments)
+		 */
+		Expression *e = new DotIdExp(loc, e1, fd->ident);
 		ArrayExp *ae = (ArrayExp *)this;
-
-		e = new DotIdExp(loc, e1, fd->ident);
 		e = new CallExp(loc, e, ae->arguments);
 		e = e->semantic(sc);
 		return e;
@@ -206,6 +202,21 @@ Expression *UnaExp::op_overload(Scope *sc)
 		return build_overload(loc, sc, e1, NULL, fd->ident);
 	    }
 	}
+
+#if DMDV2
+	// Didn't find it. Forward to aliasthis
+	if (ad->aliasthis)
+	{
+	    /* Rewrite op(e1) as:
+	     *	op(e1.aliasthis)
+	     */
+	    Expression *e1 = new DotIdExp(loc, this->e1, ad->aliasthis->ident);
+	    Expression *e = copy();
+	    ((UnaExp *)e)->e1 = e1;
+	    e = e->semantic(sc);
+	    return e;
+	}
+#endif
     }
     return NULL;
 }
@@ -279,11 +290,11 @@ Expression *BinExp::op_overload(Scope *sc)
 	    fd = s->isFuncDeclaration();
 	    if (fd)
 	    {
-		overloadResolveX(&m, fd, &args2);
+		overloadResolveX(&m, fd, NULL, &args2);
 	    }
 	    else
 	    {   td = s->isTemplateDeclaration();
-		templateResolve(&m, td, sc, loc, NULL, &args2);
+		templateResolve(&m, td, sc, loc, NULL, NULL, &args2);
 	    }
 	}
 	
@@ -294,11 +305,11 @@ Expression *BinExp::op_overload(Scope *sc)
 	    fd = s_r->isFuncDeclaration();
 	    if (fd)
 	    {
-		overloadResolveX(&m, fd, &args1);
+		overloadResolveX(&m, fd, NULL, &args1);
 	    }
 	    else
 	    {   td = s_r->isTemplateDeclaration();
-		templateResolve(&m, td, sc, loc, NULL, &args1);
+		templateResolve(&m, td, sc, loc, NULL, NULL, &args1);
 	    }
 	}
 
@@ -350,7 +361,6 @@ Expression *BinExp::op_overload(Scope *sc)
 	     *	b.opfunc(a)
 	     * and see which is better.
 	     */
-	    Expression *e;
 	    FuncDeclaration *lastf;
 
 	    if (!argsset)
@@ -368,11 +378,11 @@ Expression *BinExp::op_overload(Scope *sc)
 		fd = s_r->isFuncDeclaration();
 		if (fd)
 		{
-		    overloadResolveX(&m, fd, &args2);
+		    overloadResolveX(&m, fd, NULL, &args2);
 		}
 		else
 		{   td = s_r->isTemplateDeclaration();
-		    templateResolve(&m, td, sc, loc, NULL, &args2);
+		    templateResolve(&m, td, sc, loc, NULL, NULL, &args2);
 		}
 	    }
 	    lastf = m.lastf;
@@ -382,11 +392,11 @@ Expression *BinExp::op_overload(Scope *sc)
 		fd = s->isFuncDeclaration();
 		if (fd)
 		{
-		    overloadResolveX(&m, fd, &args1);
+		    overloadResolveX(&m, fd, NULL, &args1);
 		}
 		else
 		{   td = s->isTemplateDeclaration();
-		    templateResolve(&m, td, sc, loc, NULL, &args1);
+		    templateResolve(&m, td, sc, loc, NULL, NULL, &args1);
 		}
 	    }
 
@@ -403,6 +413,7 @@ Expression *BinExp::op_overload(Scope *sc)
 		m.lastf = m.anyf;
 	    }
 
+	    Expression *e;
 	    if (lastf && m.lastf == lastf ||
 		id_r && m.last == MATCHnomatch)
 		// Rewrite (e1 op e2) as e1.opfunc_r(e2)
@@ -438,6 +449,33 @@ Expression *BinExp::op_overload(Scope *sc)
 	}
     }
 
+#if DMDV2
+    // Try alias this on first operand
+    if (ad1 && ad1->aliasthis)
+    {
+	/* Rewrite (e1 op e2) as:
+	 *	(e1.aliasthis op e2)
+	 */
+	Expression *e1 = new DotIdExp(loc, this->e1, ad1->aliasthis->ident);
+	Expression *e = copy();
+	((BinExp *)e)->e1 = e1;
+	e = e->semantic(sc);
+	return e;
+    }
+
+    // Try alias this on second operand
+    if (ad2 && ad2->aliasthis)
+    {
+	/* Rewrite (e1 op e2) as:
+	 *	(e1 op e2.aliasthis)
+	 */
+	Expression *e2 = new DotIdExp(loc, this->e2, ad2->aliasthis->ident);
+	Expression *e = copy();
+	((BinExp *)e)->e2 = e2;
+	e = e->semantic(sc);
+	return e;
+    }
+#endif
     return NULL;
 }
 
@@ -445,7 +483,7 @@ Expression *BinExp::op_overload(Scope *sc)
  * Utility to build a function call out of this reference and argument.
  */
 
-static Expression *build_overload(Loc loc, Scope *sc, Expression *ethis, Expression *earg, Identifier *id)
+Expression *build_overload(Loc loc, Scope *sc, Expression *ethis, Expression *earg, Identifier *id)
 {
     Expression *e;
 
@@ -514,7 +552,6 @@ void inferApplyArgTypes(enum TOK op, Arguments *arguments, Expression *aggr)
     }
 
     AggregateDeclaration *ad;
-    FuncDeclaration *fd;
 
     Argument *arg = (Argument *)arguments->data[0];
     Type *taggr = aggr->type;
@@ -585,7 +622,7 @@ void inferApplyArgTypes(enum TOK op, Arguments *arguments, Expression *aggr)
 						   : Id::apply);
 	    if (s)
 	    {
-		fd = s->isFuncDeclaration();
+		FuncDeclaration *fd = s->isFuncDeclaration();
 		if (fd) 
 		    inferApplyArgTypesX(fd, arguments);
 	    }
@@ -597,7 +634,7 @@ void inferApplyArgTypes(enum TOK op, Arguments *arguments, Expression *aggr)
 	    if (0 && aggr->op == TOKdelegate)
 	    {	DelegateExp *de = (DelegateExp *)aggr;
 
-		fd = de->func->isFuncDeclaration();
+		FuncDeclaration *fd = de->func->isFuncDeclaration();
 		if (fd)
 		    inferApplyArgTypesX(fd, arguments);
 	    }
@@ -734,12 +771,12 @@ static int inferApplyArgTypesY(TypeFunction *tf, Arguments *arguments)
 /**************************************
  */
 
-static void templateResolve(Match *m, TemplateDeclaration *td, Scope *sc, Loc loc, Objects *targsi, Expressions *arguments)
+static void templateResolve(Match *m, TemplateDeclaration *td, Scope *sc, Loc loc, Objects *targsi, Expression *ethis, Expressions *arguments)
 {
     FuncDeclaration *fd;
 
     assert(td);
-    fd = td->deduceFunctionTemplate(sc, loc, targsi, arguments);
+    fd = td->deduceFunctionTemplate(sc, loc, targsi, NULL, arguments);
     if (!fd)
 	return;
     m->anyf = fd;
