@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -231,13 +231,13 @@ Expression *AddrExp::optimize(int result)
 
 	if (ae->e2->op == TOKint64 && ae->e1->op == TOKvar)
 	{
-	    integer_t index = ae->e2->toInteger();
+	    dinteger_t index = ae->e2->toInteger();
 	    VarExp *ve = (VarExp *)ae->e1;
 	    if (ve->type->ty == Tsarray && ve->type->next->ty != Tbit
 		&& !ve->var->isImportedSymbol())
 	    {
 		TypeSArray *ts = (TypeSArray *)ve->type;
-		integer_t dim = ts->dim->toInteger();
+		dinteger_t dim = ts->dim->toInteger();
 		if (index < 0 || index >= dim)
 		    error("array index %"PRIdMAX" is out of bounds [0..%"PRIdMAX"]", index, dim);
 		e = new SymOffExp(loc, ve->var, index * ts->next->size());
@@ -285,6 +285,43 @@ Expression *PtrExp::optimize(int result)
     return this;
 }
 
+Expression *DotVarExp::optimize(int result)
+ {
+     //printf("DotVarExp::optimize(result = x%x) %s\n", result, toChars());
+     e1 = e1->optimize(result);
+ 
+ #if DMDV2
+     if (e1->op == TOKvar)
+     {	VarExp *ve = (VarExp *)e1;
+ 	VarDeclaration *v = ve->var->isVarDeclaration();
+ 	Expression *e = expandVar(result, v);
+ 	if (e && e->op == TOKstructliteral)
+ 	{   StructLiteralExp *sle = (StructLiteralExp *)e;
+ 	    VarDeclaration *vf = var->isVarDeclaration();
+ 	    if (vf)
+ 	    {
+		e = sle->getField(type, vf->offset);
+		if (e && e != EXP_CANT_INTERPRET)
+ 		    return e;
+ 	    }
+ 	}
+     }
+     else
+ #endif
+     if (e1->op == TOKstructliteral)
+     {   StructLiteralExp *sle = (StructLiteralExp *)e1;
+ 	VarDeclaration *vf = var->isVarDeclaration();
+ 	if (vf)
+ 	{
+ 	    Expression *e = sle->getField(type, vf->offset);
+ 	    if (e && e != EXP_CANT_INTERPRET)
+ 		return e;
+ 	}
+     }
+ 
+     return this;
+ }
+
 Expression *CallExp::optimize(int result)
 {   Expression *e = this;
 
@@ -310,6 +347,7 @@ Expression *CastExp::optimize(int result)
     //printf("CastExp::optimize(result = %d) %s\n", result, toChars());
     //printf("from %s to %s\n", type->toChars(), to->toChars());
     //printf("from %s\n", type->toChars());
+    //printf("e1->type %s\n", e1->type->toChars());
     //printf("type = %p\n", type);
     assert(type);
     enum TOK op1 = e1->op;
@@ -386,7 +424,7 @@ Expression *BinExp::optimize(int result)
     {
 	if (e2->isConst() == 1)
 	{
-	    integer_t i2 = e2->toInteger();
+	    dinteger_t i2 = e2->toInteger();
 	    d_uns64 sz = e1->type->size() * 8;
 	    if (i2 < 0 || i2 > sz)
 	    {   error("shift assign by %jd is outside the range 0..%zu", i2, sz);
@@ -481,10 +519,10 @@ Expression *shift_optimize(int result, BinExp *e, Expression *(*shift)(Type *, E
     e->e2 = e->e2->optimize(result);
     if (e->e2->isConst() == 1)
     {
-	integer_t i2 = e->e2->toInteger();
+	dinteger_t i2 = e->e2->toInteger();
 	target_size_t sz = e->e1->type->size() * 8;
 	if (i2 < 0 || i2 > sz)
-	{   error("shift left by %"PRIdMAX" exceeds %"PRIuTSIZE, i2, sz);
+	{   e->error("shift by %"PRIdMAX" is outside the range 0..%"PRIuTSIZE, i2, sz);
 	    e->e2 = new IntegerExp(0);
 	}
 	if (e->e1->isConst() == 1)
