@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2008 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -19,6 +19,7 @@
 
 #include "arraytypes.h"
 #include "dsymbol.h"
+#include "lexer.h"
 
 struct OutBuffer;
 struct Scope;
@@ -43,6 +44,7 @@ struct AsmStatement;
 struct GotoStatement;
 struct ScopeStatement;
 struct TryCatchStatement;
+struct TryFinallyStatement;
 struct HdrGenState;
 struct InterState;
 
@@ -60,6 +62,21 @@ struct elem;
 #endif
 struct code;
 
+/* How a statement exits; this is returned by blockExit()
++  */
+ enum BE
+ {
+     BEnone =	 0,
+     BEfallthru = 1,
+     BEthrow =    2,
+     BEreturn =   4,
+     BEgoto =     8,
+     BEhalt =	 0x10,
+     BEbreak =	 0x20,
+     BEcontinue = 0x40,
+     BEany = (BEfallthru | BEthrow | BEreturn | BEgoto | BEhalt),
+ };
+
 struct Statement : Object
 {
     Loc loc;
@@ -71,6 +88,7 @@ struct Statement : Object
     char *toChars();
 
     void error(const char *format, ...);
+    void warning(const char *format, ...);
     virtual void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     virtual TryCatchStatement *isTryCatchStatement() { return NULL; }
     virtual GotoStatement *isGotoStatement() { return NULL; }
@@ -84,7 +102,7 @@ struct Statement : Object
     virtual int hasBreak();
     virtual int hasContinue();
     virtual int usesEH();
-    virtual int fallOffEnd();
+    virtual int blockExit();
     virtual int comeFrom();
     virtual void scopeCode(Statement **sentry, Statement **sexit, Statement **sfinally);
     virtual Statements *flatten(Scope *sc);
@@ -113,7 +131,7 @@ struct ExpStatement : Statement
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
+    int blockExit();
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -129,6 +147,7 @@ struct CompileStatement : Statement
     CompileStatement(Loc loc, Expression *exp);
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    Statements *flatten(Scope *sc);
     Statement *semantic(Scope *sc);
 };
 
@@ -156,7 +175,7 @@ struct CompoundStatement : Statement
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Statements *flatten(Scope *sc);
     ReturnStatement *isReturnStatement();
@@ -171,6 +190,13 @@ struct CompoundStatement : Statement
     CompoundStatement *isCompoundStatement() { return this; }
 };
 
+struct CompoundDeclarationStatement : CompoundStatement
+{
+    CompoundDeclarationStatement(Loc loc, Statements *s);
+    Statement *syntaxCopy();
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
 /* The purpose of this is so that continue will go to the next
  * of the statements, and break will go to the end of the statements.
  */
@@ -184,7 +210,7 @@ struct UnrolledLoopStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -208,7 +234,7 @@ struct ScopeStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
 
@@ -228,7 +254,7 @@ struct WhileStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -249,7 +275,7 @@ struct DoStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -273,7 +299,7 @@ struct ForStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -304,7 +330,7 @@ struct ForeachStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -314,6 +340,36 @@ struct ForeachStatement : Statement
     void toIR(IRState *irs);
 };
 
+#if DMDV2
+ struct ForeachRangeStatement : Statement
+ {
+     enum TOK op;		// TOKforeach or TOKforeach_reverse
+     Argument *arg;		// loop index variable
+     Expression *lwr;
+     Expression *upr;
+     Statement *body;
+ 
+     VarDeclaration *key;
+ 
+     ForeachRangeStatement(Loc loc, enum TOK op, Argument *arg,
+ 	Expression *lwr, Expression *upr, Statement *body);
+     Statement *syntaxCopy();
+     Statement *semantic(Scope *sc);
+     int hasBreak();
+     int hasContinue();
+     int usesEH();
+     int blockExit();
+
+     int comeFrom();
+     Expression *interpret(InterState *istate);
+     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+ 
+     Statement *inlineScan(InlineScanState *iss);
+ 
+     void toIR(IRState *irs);
+ };
+ #endif
+
 struct IfStatement : Statement
 {
     Argument *arg;
@@ -329,7 +385,7 @@ struct IfStatement : Statement
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     IfStatement *isIfStatement() { return this; }
 
     int inlineCost(InlineCostState *ics);
@@ -350,6 +406,7 @@ struct ConditionalStatement : Statement
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
     int usesEH();
+    int blockExit();
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -364,7 +421,7 @@ struct PragmaStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -395,7 +452,7 @@ struct SwitchStatement : Statement
     Statement *semantic(Scope *sc);
     int hasBreak();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
@@ -416,7 +473,7 @@ struct CaseStatement : Statement
     Statement *semantic(Scope *sc);
     int compare(Object *obj);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -437,7 +494,7 @@ struct DefaultStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -455,7 +512,7 @@ struct GotoDefaultStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -470,7 +527,7 @@ struct GotoCaseStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -479,7 +536,7 @@ struct GotoCaseStatement : Statement
 struct SwitchErrorStatement : Statement
 {
     SwitchErrorStatement(Loc loc);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -493,7 +550,7 @@ struct ReturnStatement : Statement
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
-    int fallOffEnd();
+    int blockExit();
     Expression *interpret(InterState *istate);
 
     int inlineCost(InlineCostState *ics);
@@ -513,7 +570,7 @@ struct BreakStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -527,7 +584,7 @@ struct ContinueStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -544,7 +601,7 @@ struct SynchronizedStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -566,7 +623,7 @@ struct WithStatement : Statement
     Statement *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -583,7 +640,7 @@ struct TryCatchStatement : Statement
     Statement *semantic(Scope *sc);
     int hasBreak();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -603,6 +660,7 @@ struct Catch : Object
     Catch(Loc loc, Type *t, Identifier *id, Statement *handler);
     Catch *syntaxCopy();
     void semantic(Scope *sc);
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
@@ -618,7 +676,7 @@ struct TryFinallyStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -632,6 +690,7 @@ struct OnScopeStatement : Statement
 
     OnScopeStatement(Loc loc, TOK tok, Statement *statement);
     Statement *syntaxCopy();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     int usesEH();
@@ -648,7 +707,7 @@ struct ThrowStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    int fallOffEnd();
+    int blockExit();
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -663,7 +722,7 @@ struct VolatileStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
-    int fallOffEnd();
+    int blockExit();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -680,7 +739,7 @@ struct GotoStatement : Statement
     GotoStatement(Loc loc, Identifier *ident);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    int fallOffEnd();
+    int blockExit();
     Expression *interpret(InterState *istate);
 
     void toIR(IRState *irs);
@@ -701,7 +760,7 @@ struct LabelStatement : Statement
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
     int usesEH();
-    int fallOffEnd();
+    int blockExit();
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -734,6 +793,7 @@ struct AsmStatement : Statement
     AsmStatement(Loc loc, Token *tokens);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
+    int blockExit();
     int comeFrom();
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -754,6 +814,8 @@ struct ExtAsmStatement : Statement
     Expressions *argConstraints;  // of StringExp*
     unsigned nOutputArgs;
     Expressions *clobbers;        // of StringExp*
+    
+    int blockExit();
 
     ExtAsmStatement(Loc loc, Expression *insnTemplate, Expressions *args, Array *argNames,
 	Expressions *argConstraints, int nOutputArgs, Expressions *clobbers);
