@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -194,7 +194,6 @@ dt_t *StructInitializer::toDt()
 	    {	target_size_t sz = dt_size(d);
 		target_size_t vsz = v->type->size();
 		target_size_t voffset = v->offset;
-		assert(sz <= vsz);
 
 #ifdef IN_GCC
 		if (offset < voffset)
@@ -214,7 +213,9 @@ dt_t *StructInitializer::toDt()
 		     vt = vt->next->toBasetype())
 		{   TypeSArray *tsa = (TypeSArray *)vt;
 		    dim *= tsa->dim->toInteger();
-		}
+		}	
+		//printf("sz = %d, dim = %d, vsz = %d\n", sz, dim, vsz);
+ 		assert(sz == vsz || sz * dim <= vsz);
 
 		for (target_size_t i = 0; i < dim; i++)
 		{
@@ -398,6 +399,7 @@ dt_t *ArrayInitializer::toDt()
 
 dt_t *ArrayInitializer::toDtBit()
 {
+	#if DMDV1
     unsigned size;
     unsigned length;
     unsigned i;
@@ -416,7 +418,7 @@ dt_t *ArrayInitializer::toDtBit()
 	 * use instead the dimension of the type in order
 	 * to get the whole thing.
 	 */
-	integer_t value = ((TypeSArray*)tb)->dim->toInteger();
+	dinteger_t value = ((TypeSArray*)tb)->dim->toInteger();
 	tadim = value;
 	assert(tadim == value);	 // truncation overflow should already be checked
 	databits.resize(tadim);
@@ -443,7 +445,7 @@ dt_t *ArrayInitializer::toDtBit()
 
 	idx = (Expression *)index.data[i];
 	if (idx)
-	{   integer_t value;
+	{   dinteger_t value;
 	    value = idx->toInteger();
 	    length = value;
 	    if (length != value)
@@ -509,6 +511,9 @@ dt_t *ArrayInitializer::toDtBit()
 	    assert(0);
     }
     return d;
+    #else
+     return NULL;
+ 	#endif
 }
 
 
@@ -548,7 +553,7 @@ dt_t **IntegerExp::toDt(dt_t **pdt)
     return pdt;
 }
 
-static char zeropad[2];
+static char zeropad[6];
 
 dt_t **RealExp::toDt(dt_t **pdt)
 {
@@ -576,6 +581,7 @@ dt_t **RealExp::toDt(dt_t **pdt)
 	    evalue = value;
 	    pdt = dtnbytes(pdt,REALSIZE - REALPAD,(char *)&evalue);
 	    pdt = dtnbytes(pdt,REALPAD,zeropad);
+	    assert(REALPAD <= sizeof(zeropad));
 	    break;
 
 	default:
@@ -658,7 +664,7 @@ dt_t **StringExp::toDt(dt_t **pdt)
 
 	case Tsarray:
 	{   TypeSArray *tsa = (TypeSArray *)type;
-	    integer_t dim;
+	    dinteger_t dim;
 
 #ifndef IN_GCC
 	    pdt = dtnbytes(pdt, len * sz, (const char *)string);
@@ -902,7 +908,14 @@ dt_t **VarExp::toDt(dt_t **pdt)
     VarDeclaration *v = var->isVarDeclaration();
     if (v && v->isConst() && type->toBasetype()->ty != Tsarray && v->init)
     {
+    if (v->inuse)
+ 	{
+ 	    error("recursive reference %s", toChars());
+ 	    return pdt;
+ 	}
+ 	v->inuse++;
 	*pdt = v->init->toDt();
+	v->inuse--;
 	return pdt;
     }
     SymbolDeclaration *sd = var->isSymbolDeclaration();
@@ -991,7 +1004,7 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
 	if (dt)
 	{
 	    if (v->offset < offset)
-		error("2duplicate union initialization for %s", v->toChars());
+		error("duplicated union initialization for %s", v->toChars());
 	    else
 	    {
 		if (offset < v->offset)
