@@ -10,7 +10,8 @@
 /* NOTE: This file has been patched from the original DMD distribution to
    work with the GDC compiler.
 
-   Modified by David Friedman, September 2004
+   Modified by Michael Parrott, September 2009
+   Using David Friedman's code
 */
 
 #include <stdio.h>
@@ -30,8 +31,8 @@
 #include "id.h"
 #include "import.h"
 #include "template.h"
+#include "rmem.h" //commented out for gdc??
 
-#include <mem.h>
 #ifndef IN_GCC
 #include "cc.h"
 #include "global.h"
@@ -133,6 +134,13 @@ void Module::genmoduleinfo()
 	dtdword(&dt, 0);		// BUG: should be an assert()
     }
     dtdword(&dt, 0);			// monitor
+    
+    #if DMDV2
+     FuncDeclaration *sgetmembers = findGetMembers();
+     if (sgetmembers)
+ 	dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
+     else
+ #endif
 
     // name[]
     char *name = toPrettyChars();
@@ -170,7 +178,7 @@ void Module::genmoduleinfo()
 	    aimports_dim--;
     }
     dtdword(&dt, aimports_dim);
-    if (aimports.dim)
+    if (aimports_dim)
 	dtxoff(&dt, csym, sizeof_ModuleInfo, TYnptr);
     else
 	dtdword(&dt, 0);
@@ -232,7 +240,13 @@ void Module::genmoduleinfo()
 
 	if (m->needModuleInfo())
 	{   Symbol *s = m->toSymbol();
-	    s->Sflags |= SFLweak;
+	    /* Weak references don't pull objects in from the library,
++ 	     * they resolve to 0 if not pulled in by something else.
++ 	     * Don't pull in a module just because it was imported.
++ 	     */
+#if !OMFOBJ // Optlink crashes with weak symbols at EIP 41AFE7, 402000
+     s->Sflags |= SFLweak;
+#endif
 	    dtxoff(&dt, s, 0, TYnptr);
 	}
     }
@@ -244,7 +258,7 @@ void Module::genmoduleinfo()
     }
 
     csym->Sdt = dt;
-#if ELFOBJ
+#if ELFOBJ || MACHOBJ
     // Cannot be CONST because the startup code sets flag bits in it
     csym->Sseg = DATA;
 #endif
@@ -302,10 +316,9 @@ void ClassDeclaration::toObjFile(int multiobj)
 
     assert(!scope);	// semantic() should have been run to completion
 
-    if (parent && parent->isTemplateInstance())
-	scclass = SCcomdat;
-    else
-	scclass = SCglobal;
+    scclass = SCglobal;
+     if (inTemplateInstance()) //possible changes needed here
+  	scclass = SCcomdat;
 
     // Put out the members
     for (i = 0; i < members->dim; i++)
@@ -350,9 +363,7 @@ void ClassDeclaration::toObjFile(int multiobj)
 	    for (i = 0; i < dtors.dim; i++)
 	    {	DtorDeclaration *d = (DtorDeclaration *)dtors.data[i];
 		Symbol *s = d->toSymbol();
-		elem *e;
-
-		e = el_bin(OPcall, TYvoid, el_var(s), el_var(sthis));
+		elem *e = el_bin(OPcall, TYvoid, el_var(s), el_var(sthis));
 		edtor = el_combine(e, edtor);
 	    }
 
@@ -389,10 +400,19 @@ void ClassDeclaration::toObjFile(int multiobj)
     sinit->Sfl = FLdata;
 #if ELFOBJ // Burton
     sinit->Sseg = CDATA;
-#endif /* ELFOBJ */
+#endif
+#if MACHOBJ
+    sinit->Sseg = DATA;
+#endif
     toDt(&sinit->Sdt);
     outdata(sinit);
 
+    //////////////////////////////////////////////
+    
+    // Put out the TypeInfo
+    type->getTypeInfo(NULL);
+    type->vtinfo->toObjFile(multiobj);
+
     //////////////////////////////////////////////
 
     // Put out the ClassInfo
@@ -414,6 +434,8 @@ void ClassDeclaration::toObjFile(int multiobj)
 	    void *deallocator;
 	    OffsetTypeInfo[] offTi;
 	    void *defaultConstructor;
+	    const(MemberInfo[]) function(string) xgetMembers;	// module getMembers() function
+	    TypeInfo typeinfo;
        }
      */
     dt_t *dt = NULL;
@@ -421,7 +443,7 @@ void ClassDeclaration::toObjFile(int multiobj)
     if (classinfo)
     {
 	if (classinfo->structsize != CLASSINFO_SIZE)
-	    error("D compiler and phobos/object.d are mismatched");
+	    error("D compiler and phobos' object.d are mismatched");
     }
 
     if (classinfo)
@@ -476,6 +498,10 @@ void ClassDeclaration::toObjFile(int multiobj)
 
     // flags
     int flags = 4 | isCOMclass();
+    #if DMDV2
+     flags |= 16;
+ 	 #endif
+ 	flags |= 32;
     if (ctor)
 	flags |= 8;
     for (ClassDeclaration *cd = this; cd; cd = cd->baseClass)
@@ -511,6 +537,17 @@ void ClassDeclaration::toObjFile(int multiobj)
 	dtxoff(&dt, defaultCtor->toSymbol(), 0, TYnptr);
     else
 	dtdword(&dt, 0);
+	
+	#if DMDV2
+     FuncDeclaration *sgetmembers = findGetMembers();
+     if (sgetmembers)
+ 	dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
+     else
+ 	dtdword(&dt, 0);	// module getMembers() function
+ #endif
+ 
+ dtxoff(&dt, type->vtinfo->toSymbol(), 0, TYnptr);	// typeinfo
+     //dtdword(&dt, 0);
 
     //////////////////////////////////////////////
 
@@ -680,10 +717,10 @@ void ClassDeclaration::toObjFile(int multiobj)
 
 
     csym->Sdt = dt;
-#if ELFOBJ // Burton
+#if ELFOBJ || MACHOBJ // Burton
     // ClassInfo cannot be const data, because we use the monitor on it
     csym->Sseg = DATA;
-#endif /* ELFOBJ */
+#endif
     outdata(csym);
     if (isExport())
 	obj_export(csym,0);
@@ -705,8 +742,37 @@ void ClassDeclaration::toObjFile(int multiobj)
 
 	//printf("\tvtbl[%d] = %p\n", i, fd);
 	if (fd && (fd->fbody || !isAbstract()))
-	{
-	    dtxoff(&dt, fd->toSymbol(), 0, TYnptr);
+	{   Symbol *s = fd->toSymbol();
+ 
+ #if DMDV2
+ 	    if (isFuncHidden(fd))
+ 	    {	/* fd is hidden from the view of this class.
+ 		 * If fd overlaps with any function in the vtbl[], then
+ 		 * issue 'hidden' error.
+ 		 */
+ 		for (int j = 1; j < vtbl.dim; j++)
+ 		{   if (j == i)
+ 			continue;
+ 		    FuncDeclaration *fd2 = ((Dsymbol *)vtbl.data[j])->isFuncDeclaration();
+ 		    if (!fd2->ident->equals(fd->ident))
+ 			continue;
+ 		    if (fd->leastAsSpecialized(fd2) || fd2->leastAsSpecialized(fd))
+ 		    {
+ 			if (global.params.warnings)
+ 			{
+ 			    TypeFunction *tf = (TypeFunction *)fd->type;
+ 			    if (tf->ty == Tfunction)
+ 				warning("%s%s is hidden by %s\n", fd->toPrettyChars(), Argument::argsTypesToChars(tf->parameters, tf->varargs), toChars());
+ 			    else
+ 				warning("%s is hidden by %s\n", fd->toPrettyChars(), toChars());
+ 			}
+ 			s = rtlsym[RTLSYM_DHIDDENFUNC];
+ 			break;
+ 		    }
+ 		}
+ 	    }
+ #endif //V2
+ 	    dtxoff(&dt, s, 0, TYnptr);
 	}
 	else
 	    dtdword(&dt, 0);
@@ -714,9 +780,12 @@ void ClassDeclaration::toObjFile(int multiobj)
     vtblsym->Sdt = dt;
     vtblsym->Sclass = scclass;
     vtblsym->Sfl = FLdata;
-#if ELFOBJ // Burton
-    vtblsym->Sseg = CDATA;
-#endif /* ELFOBJ */
+#if ELFOBJ
+      vtblsym->Sseg = CDATA;
+#endif
+#if MACHOBJ
+    vtblsym->Sseg = DATA;
+#endif
     outdata(vtblsym);
     if (isExport())
 	obj_export(vtblsym,0);
@@ -809,7 +878,7 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     if (global.params.symdebug)
 	toDebug();
 
-    if (parent && parent->isTemplateInstance())
+    if (inTemplateInstance())
 	scclass = SCcomdat;
     else
 	scclass = SCglobal;
@@ -827,6 +896,12 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     // Generate C symbols
     toSymbol();
 
+    //////////////////////////////////////////////
+    
+    // Put out the TypeInfo
+    type->getTypeInfo(NULL);
+    type->vtinfo->toObjFile(multiobj);
+
     //////////////////////////////////////////////
 
     // Put out the ClassInfo
@@ -848,6 +923,10 @@ void InterfaceDeclaration::toObjFile(int multiobj)
 	    void *deallocator;
 	    OffsetTypeInfo[] offTi;
 	    void *defaultConstructor;
+	    #if DMDV2
+ 	    const(MemberInfo[]) function(string) xgetMembers;	// module getMembers() function
+ 		#endif
+ 		TypeInfo typeinfo;
        }
      */
     dt_t *dt = NULL;
@@ -895,7 +974,7 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     dtdword(&dt, 0);
 
     // flags
-    dti32(&dt, 4 | isCOMinterface(), true);
+    dtdword(&dt, 4 | isCOMinterface() | 32); //dti32(&dt, 4 | isCOMinterface(), true);
 
     // deallocator
     dtdword(&dt, 0);
@@ -906,6 +985,13 @@ void InterfaceDeclaration::toObjFile(int multiobj)
 
     // defaultConstructor
     dtdword(&dt, 0);
+    
+    #if DMDV2
+     // xgetMembers
+     dtdword(&dt, 0);
+ 	#endif
+ 	
+ 	dtxoff(&dt, type->vtinfo->toSymbol(), 0, TYnptr);	// typeinfo
 
     //////////////////////////////////////////////
 
@@ -929,9 +1015,12 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     }
 
     csym->Sdt = dt;
-#if ELFOBJ // Burton
-    csym->Sseg = CDATA;
-#endif /* ELFOBJ */
+#if ELFOBJ
+      csym->Sseg = CDATA;
+#endif
+#if MACHOBJ
+    csym->Sseg = DATA;
+#endif
     outdata(csym);
     if (isExport())
 	obj_export(csym,0);
@@ -940,9 +1029,14 @@ void InterfaceDeclaration::toObjFile(int multiobj)
 /* ================================================================== */
 
 void StructDeclaration::toObjFile(int multiobj)
-{   unsigned i;
+{
 
     //printf("StructDeclaration::toObjFile('%s')\n", toChars());
+    
+    if (multiobj)
+     {	obj_append(this);
+ 	return;
+     }
 
     // Anonymous structs/unions only exist as part of others,
     // do not output forward referenced structs's
@@ -960,15 +1054,19 @@ void StructDeclaration::toObjFile(int multiobj)
 #if 0
 	    sinit->Sclass = SCcomdat;
 #else
-	    if (parent && parent->isTemplateInstance())
+	    if (inTemplateInstance())
+ 	    {
 		sinit->Sclass = SCcomdat;
+		}
 	    else
+	    {
 		sinit->Sclass = SCglobal;
+		}
 #endif
 	    sinit->Sfl = FLdata;
 	    toDt(&sinit->Sdt);
 
-#if !ELFOBJ
+#if OMFOBJ
 	    /* For OMF, common blocks aren't pulled in from the library.
 	     */
 	    /* ELF comdef's generate multiple
@@ -980,21 +1078,25 @@ void StructDeclaration::toObjFile(int multiobj)
 	    if (sinit->Sclass == SCcomdat &&
 		sinit->Sdt &&
 		sinit->Sdt->dt == DT_azeros &&
-		sinit->Sdt->DTnext == NULL)
+		sinit->Sdt->DTnext == NULL &&
+ 		!global.params.multiobj)
 	    {
 		sinit->Sclass = SCglobal;
 		sinit->Sdt->dt = DT_common;
 	    }
 #endif
 
-#if ELFOBJ // Burton
+#if ELFOBJ
 	    sinit->Sseg = CDATA;
+#endif
+#if MACHOBJ
+	    sinit->Sseg = DATA;
 #endif
 	    outdata(sinit);
 	}
 
 	// Put out the members
-	for (i = 0; i < members->dim; i++)
+	for (unsigned i = 0; i < members->dim; i++)
 	{
 	    Dsymbol *member;
 
@@ -1019,6 +1121,14 @@ void VarDeclaration::toObjFile(int multiobj)
     {	toAlias()->toObjFile(0);
 	return;
     }
+    
+    #if DMDV2
+     // Do not store variables we cannot take the address of
+     if (!canTakeAddressOf())
+     {
+ 	return;
+     }
+ #endif
 
     if (isDataseg() && !(storage_class & STCextern))
     {
@@ -1026,7 +1136,7 @@ void VarDeclaration::toObjFile(int multiobj)
 	sz = type->size();
 
 	parent = this->toParent();
-#if 1	/* private statics should still get a global symbol, in case
+#if DMDV1	/* private statics should still get a global symbol, in case
 	 * another module inlines a function that references it.
 	 */
 	if (/*protection == PROTprivate ||*/
@@ -1050,6 +1160,7 @@ void VarDeclaration::toObjFile(int multiobj)
 		 */
 		if (parent->isTemplateInstance() && !parent->isTemplateMixin())
 		{
+			#if DMDV1
 		    /* These symbol constants have already been copied,
 		     * so no reason to output them.
 		     * Note that currently there is no way to take
@@ -1058,6 +1169,8 @@ void VarDeclaration::toObjFile(int multiobj)
 		    if (isConst() && type->toBasetype()->ty != Tsarray &&
 			init && init->isExpInitializer())
 			return;
+			
+			#endif
 
 		    s->Sclass = SCcomdat;
 		    break;
@@ -1116,12 +1229,12 @@ void VarDeclaration::toObjFile(int multiobj)
 	    s->Sdt->dt = DT_common;
 	}
 
-#if ELFOBJ // Burton
+#if ELFOBJ || MACHOBJ // Burton
 	if (s->Sdt && s->Sdt->dt == DT_azeros && s->Sdt->DTnext == NULL)
 	    s->Sseg = UDATA;
 	else
 	    s->Sseg = DATA;
-#endif /* ELFOBJ */
+#endif
 	if (sz)
 	{   outdata(s);
 	    if (isExport())
@@ -1156,14 +1269,8 @@ void TypedefDeclaration::toObjFile(int multiobj)
     else
     {
 	enum_SC scclass = SCglobal;
-	for (Dsymbol *parent = this->parent; parent; parent = parent->parent)
-	{
-	    if (parent->isTemplateInstance())
-	    {
+	if (inTemplateInstance())
 		scclass = SCcomdat;
-		break;
-	    }
-	}
 
 	// Generate static initializer
 	toInitializer();
@@ -1171,7 +1278,10 @@ void TypedefDeclaration::toObjFile(int multiobj)
 	sinit->Sfl = FLdata;
 #if ELFOBJ // Burton
 	sinit->Sseg = CDATA;
-#endif /* ELFOBJ */
+#endif
+#if MACHOBJ
+	sinit->Sseg = DATA;
+#endif
 	if (! sinit->Sdt)
 	    sinit->Sdt = tc->sym->init->toDt();
 	outdata(sinit);
@@ -1183,6 +1293,11 @@ void TypedefDeclaration::toObjFile(int multiobj)
 void EnumDeclaration::toObjFile(int multiobj)
 {
     //printf("EnumDeclaration::toObjFile('%s')\n", toChars());
+    
+    #if DMDV2
+     if (isAnonymous())
+ 	return;
+ #endif
 
     if (global.params.symdebug)
 	toDebug();
@@ -1190,19 +1305,13 @@ void EnumDeclaration::toObjFile(int multiobj)
     type->getTypeInfo(NULL);	// generate TypeInfo
 
     TypeEnum *tc = (TypeEnum *)type;
-    if (type->isZeroInit() || !tc->sym->defaultval)
+    if (!tc->sym->defaultval || type->isZeroInit())
 	;
     else
     {
 	enum_SC scclass = SCglobal;
-	for (Dsymbol *parent = this->parent; parent; parent = parent->parent)
-	{
-	    if (parent->isTemplateInstance())
-	    {
+	if (inTemplateInstance())
 		scclass = SCcomdat;
-		break;
-	    }
-	}
 
 	// Generate static initializer
 	toInitializer();
@@ -1210,7 +1319,17 @@ void EnumDeclaration::toObjFile(int multiobj)
 	sinit->Sfl = FLdata;
 #if ELFOBJ // Burton
 	sinit->Sseg = CDATA;
-#endif /* ELFOBJ */
+#endif
+#if MACHOBJ
+ 	sinit->Sseg = DATA;
+#endif
+#if DMDV1
+  	dtnbytes(&sinit->Sdt, tc->size(0), (char *)&tc->sym->defaultval);
+  	//sinit->Sdt = tc->sym->init->toDt();
+#endif
+#if DMDV2
+ 	tc->sym->defaultval->toDt(&sinit->Sdt);
+#endif
 #ifndef IN_GCC
 	dtnbytes(&sinit->Sdt, tc->size(0), (char *)&tc->sym->defaultval);
 #else
