@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2009 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -33,7 +33,7 @@
 #include "import.h"
 #include "aggregate.h"
 
-#include <mem.h>
+#include "rmem.h"
 #ifndef IN_GCC
 #include "cc.h"
 #include "global.h"
@@ -118,10 +118,21 @@ Expression *Type::getTypeInfo(Scope *sc)
     Type *t;
 
     //printf("Type::getTypeInfo() %p, %s\n", this, toChars());
-    t = merge();	// do this since not all Type's are merge'd
+    t = merge2();	// do this since not all Type's are merge'd
     if (!t->vtinfo)
-    {	t->vtinfo = t->getTypeInfoDeclaration();
+    {
+#if DMDV2
+	if (t->isShared())	// does both 'shared' and 'shared const'
+	    t->vtinfo = new TypeInfoSharedDeclaration(t);
+	else if (t->isConst())
+	    t->vtinfo = new TypeInfoConstDeclaration(t);
+	else if (t->isInvariant())
+	    t->vtinfo = new TypeInfoInvariantDeclaration(t);
+	else
+#endif
+	    t->vtinfo = t->getTypeInfoDeclaration();
 	assert(t->vtinfo);
+	vtinfo = t->vtinfo;
 
 	/* If this has a custom implementation in std/typeinfo, then
 	 * do not generate a COMDAT for it.
@@ -222,6 +233,45 @@ void TypeInfoDeclaration::toDt(dt_t **pdt)
     dtdword(pdt, 0);			    // monitor
 }
 
+#if DMDV2
+ void TypeInfoConstDeclaration::toDt(dt_t **pdt)
+ {
+     //printf("TypeInfoConstDeclaration::toDt() %s\n", toChars());
+     dtxoff(pdt, Type::typeinfoconst->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Const
+     dtdword(pdt, 0);			    // monitor
+     Type *tm = tinfo->mutableOf();
+     tm = tm->merge();
+     tm->getTypeInfo(NULL);
+     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+ }
+ 
+ void TypeInfoInvariantDeclaration::toDt(dt_t **pdt)
+ {
+     //printf("TypeInfoInvariantDeclaration::toDt() %s\n", toChars());
+     dtxoff(pdt, Type::typeinfoinvariant->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Invariant
+     dtdword(pdt, 0);			    // monitor
+     Type *tm = tinfo->mutableOf();
+     tm = tm->merge();
+     tm->getTypeInfo(NULL);
+     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+ }
+ 
+ void TypeInfoSharedDeclaration::toDt(dt_t **pdt)
+ {
+     //printf("TypeInfoSharedDeclaration::toDt() %s\n", toChars());
+     dtxoff(pdt, Type::typeinfoshared->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Shared
+     dtdword(pdt, 0);			    // monitor
+     Type *tm;
+     if (tinfo->isConst())		// it was 'shared const'
+ 	tm = tinfo->constOf();
+     else				// it was just 'shared'
+ 	tm = tinfo->mutableOf();
+     tm = tm->merge();
+    tm->getTypeInfo(NULL);
+     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+ }
+#endif
+
 void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoTypedefDeclaration::toDt() %s\n", toChars());
@@ -290,7 +340,7 @@ void TypeInfoEnumDeclaration::toDt(dt_t **pdt)
     dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 
     // void[] init;
-    if (tinfo->isZeroInit() || !sd->defaultval)
+    if (!sd->defaultval || tinfo->isZeroInit())
     {	// 0 initializer, or the same as the base type
 	dtdword(pdt, 0);	// init.length
 	dtdword(pdt, 0);	// init.ptr
@@ -522,6 +572,29 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
 
     // uint m_flags;
     dti32(pdt, tc->hasPointers(), false);
+    
+     #if DMDV2
+     // xgetMembers
+     FuncDeclaration *sgetmembers = sd->findGetMembers();
+     if (sgetmembers)
+ 	dtxoff(pdt, sgetmembers->toSymbol(), 0, TYnptr);
+     else
+ 	dtdword(pdt, 0);			// xgetMembers
+ 
+     // xdtor
+     FuncDeclaration *sdtor = sd->dtor;
+     if (sdtor)
+ 	dtxoff(pdt, sdtor->toSymbol(), 0, TYnptr);
+     else
+ 	dtdword(pdt, 0);			// xdtor
+ 
+     // xpostblit
+     FuncDeclaration *spostblit = sd->postblit;
+     if (spostblit)
+ 	dtxoff(pdt, spostblit->toSymbol(), 0, TYnptr);
+     else
+ 	dtdword(pdt, 0);			// xpostblit
+ 	#endif
 
     // name[]
     dtnbytes(pdt, namelen + 1, name);
@@ -625,12 +698,12 @@ void TypeInfoDeclaration::toObjFile(int multiobj)
 	s->Sdt->dt = DT_common;
     }
 
-#if ELFOBJ // Burton
+#if ELFOBJ || MACHOBJ // Burton
     if (s->Sdt && s->Sdt->dt == DT_azeros && s->Sdt->DTnext == NULL)
 	s->Sseg = UDATA;
     else
 	s->Sseg = DATA;
-#endif /* ELFOBJ */
+#endif
     outdata(s);
     if (isExport())
 	obj_export(s,0);
@@ -651,12 +724,30 @@ int Type::builtinTypeInfo()
 
 int TypeBasic::builtinTypeInfo()
 {
+    #if DMDV2
+    return mod ? 0 : 1;
+	 #else
     return 1;
+	 #endif
 }
 
 int TypeDArray::builtinTypeInfo()
 {
+    #if DMDV2
+    return !mod && (next->isTypeBasic() != NULL && !next->mod ||
+	 // strings are so common, make them builtin
+	 next->ty == Tchar && next->mod == MODinvariant);
+	 #else
     return next->isTypeBasic() != NULL;
+	 #endif
+}
+
+int TypeClass::builtinTypeInfo()
+{
+    /* This is statically put out with the ClassInfo, so
+     * claim it is built in so it isn't regenerated by each module.
+     */
+    return 1;
 }
 
 /* ========================================================================= */
