@@ -12,6 +12,7 @@
    work with the GDC compiler.
 
    Modified by David Friedman, December 2006
+   Modified by Vincenzo Ampolo, September 2009
 */
 
 #include <stdio.h>
@@ -37,7 +38,7 @@
 #include "parse.h"
 #include "template.h"
 
-extern void obj_includelib(char *name);
+extern void obj_includelib(const char *name);
 void obj_startaddress(Symbol *s);
 
 
@@ -56,18 +57,15 @@ Array *AttribDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 
 int AttribDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-    unsigned i;
     int m = 0;
     Array *d = include(sc, sd);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    m |= s->addMember(sc, sd, m | memnum);
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+			{   Dsymbol *s = (Dsymbol *)d->data[i];
+    	  	    m |= s->addMember(sc, sd, m | memnum);
+    	  	}
     }
     return m;
 }
@@ -90,50 +88,42 @@ void AttribDeclaration::semantic(Scope *sc)
 
 void AttribDeclaration::semantic2(Scope *sc)
 {
-    unsigned i;
     Array *d = include(sc, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    s->semantic2(sc);
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+			{   Dsymbol *s = (Dsymbol *)d->data[i];
+    	  	    s->semantic2(sc);
+    	  	}
     }
 }
 
 void AttribDeclaration::semantic3(Scope *sc)
 {
-    unsigned i;
     Array *d = include(sc, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    s->semantic3(sc);
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+		{   Dsymbol *s = (Dsymbol *)d->data[i];
+    	  	    s->semantic3(sc);
+    	}
     }
 }
 
 void AttribDeclaration::inlineScan()
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
+    	for (unsigned i = 0; i < d->dim; i++)
+		{   Dsymbol *s = (Dsymbol *)d->data[i];
 
-	    s = (Dsymbol *)d->data[i];
 	    //printf("AttribDeclaration::inlineScan %s\n", s->toChars());
-	    s->inlineScan();
-	}
+			s->inlineScan();
+		}
     }
 }
 
@@ -141,17 +131,14 @@ void AttribDeclaration::addComment(unsigned char *comment)
 {
     if (comment)
     {
-	unsigned i;
-	Array *d = include(NULL, NULL);
+    	Array *d = include(NULL, NULL);
 
 	if (d)
 	{
-	    for (i = 0; i < d->dim; i++)
-	    {   Dsymbol *s;
-
-		s = (Dsymbol *)d->data[i];
+		for (unsigned i = 0; i < d->dim; i++)
+		{   Dsymbol *s = (Dsymbol *)d->data[i];
 		//printf("AttribDeclaration::addComment %s\n", s->toChars());
-		s->addComment(comment);
+			s->addComment(comment);
 	    }
 	}
     }
@@ -167,55 +154,46 @@ void AttribDeclaration::emitComment(Scope *sc)
 //    if (sc->docbuf)
 //	return;
 
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
+    	for (unsigned i = 0; i < d->dim; i++)
+    	{   Dsymbol *s = (Dsymbol *)d->data[i];
 	    //printf("AttribDeclaration::emitComment %s\n", s->toChars());
-	    s->emitComment(sc);
-	}
+			s->emitComment(sc);
+    	}
     }
 }
 
 void AttribDeclaration::toObjFile(int multiobj)
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    s->toObjFile(multiobj);
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+    	{   Dsymbol *s = (Dsymbol *)d->data[i];
+			s->toObjFile(multiobj);
+    	}
     }
 }
 
 int AttribDeclaration::cvMember(unsigned char *p)
 {
-    unsigned i;
     int nwritten = 0;
     int n;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    n = s->cvMember(p);
-	    if (p)
-		p += n;
-	    nwritten += n;
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+    	{   Dsymbol *s = (Dsymbol *)d->data[i];
+			n = s->cvMember(p);
+			if (p)
+			p += n;
+			nwritten += n;
+    	}
     }
     return nwritten;
 }
@@ -250,17 +228,14 @@ int AttribDeclaration::oneMember(Dsymbol **ps)
 
 void AttribDeclaration::checkCtorConstInit()
 {
-    unsigned i;
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    s->checkCtorConstInit();
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+    	{   Dsymbol *s = (Dsymbol *)d->data[i];
+			s->checkCtorConstInit();
+    	}
     }
 }
 
@@ -268,17 +243,15 @@ void AttribDeclaration::checkCtorConstInit()
  */
 
 void AttribDeclaration::addLocalClass(ClassDeclarations *aclasses)
-{   unsigned i;
+{
     Array *d = include(NULL, NULL);
 
     if (d)
     {
-	for (i = 0; i < d->dim; i++)
-	{   Dsymbol *s;
-
-	    s = (Dsymbol *)d->data[i];
-	    s->addLocalClass(aclasses);
-	}
+    	for (unsigned i = 0; i < d->dim; i++)
+    	{   Dsymbol *s = (Dsymbol *)d->data[i];
+			s->addLocalClass(aclasses);
+    	}
     }
 }
 
@@ -765,7 +738,7 @@ Dsymbol *PragmaDeclaration::syntaxCopy(Dsymbol *s)
 
 void PragmaDeclaration::semantic(Scope *sc)
 {   // Should be merged with PragmaStatement
-    Scope sc_save;
+	Scope sc_save;
 
     //printf("\tPragmaDeclaration::semantic '%s'\n",toChars());
     if (ident == Id::msg)
@@ -1241,6 +1214,7 @@ Dsymbol *StaticIfDeclaration::syntaxCopy(Dsymbol *s)
 
 int StaticIfDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
+	//printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
     /* This is deferred until semantic(), so that
      * expressions in the condition can refer to declarations
      * in the same scope, such as:
@@ -1267,7 +1241,7 @@ void StaticIfDeclaration::semantic(Scope *sc)
 {
     Array *d = include(sc, sd);
 
-    //printf("\tStaticIfDeclaration::semantic '%s'\n",toChars());
+    //printf("\tStaticIfDeclaration::semantic '%s', d = %p\n",toChars(), d);
     if (d)
     {
 	if (!addisdone)
@@ -1297,6 +1271,7 @@ CompileDeclaration::CompileDeclaration(Loc loc, Expression *exp)
 {
     this->exp = exp;
     this->sd = NULL;
+    this->compiled = 0;
 }
 
 Dsymbol *CompileDeclaration::syntaxCopy(Dsymbol *s)
@@ -1308,20 +1283,28 @@ Dsymbol *CompileDeclaration::syntaxCopy(Dsymbol *s)
 
 int CompileDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-    this->sd = sd;
+	//printf("CompileDeclaration::addMember(sc = %p)\n", sc);
+	this->sd = sd;
+	if (memnum == 0)
+	{	/* No members yet, so parse the mixin now */
+	 	compileIt(sc);
+ 		memnum |= AttribDeclaration::addMember(sc, sd, memnum);
+		compiled = 1;
+     }
     return memnum;
 }
 
-void CompileDeclaration::semantic(Scope *sc)
+void CompileDeclaration::compileIt(Scope *sc)
 {
-    //printf("CompileDeclaration::semantic()\n");
+	//printf("CompileDeclaration::compileIt()\n");
     exp = exp->semantic(sc);
     exp = resolveProperties(sc, exp);
     exp = exp->optimize(WANTvalue | WANTinterpret);
     if (exp->op != TOKstring)
     {	error("argument to mixin must be a string, not (%s)", exp->toChars());
-	return;
     }
+    else
+    {
     StringExp *se = (StringExp *)exp;
     se = se->toUTF8(sc);
     Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
@@ -1329,11 +1312,20 @@ void CompileDeclaration::semantic(Scope *sc)
     p.nextToken();
     decl = p.parseDeclDefs(0);
     if (p.token.value != TOKeof)
-    {
-	error("incomplete mixin declaration (%s)", se->toChars());
+    	error("incomplete mixin declaration (%s)", se->toChars());
     }
+}
 
-    AttribDeclaration::addMember(sc, sd, 0);
+void CompileDeclaration::semantic(Scope *sc)
+ {
+	//printf("CompileDeclaration::semantic()\n");
+
+	if (!compiled)
+	{
+		compileIt(sc);
+		AttribDeclaration::addMember(sc, sd, 0);
+		compiled = 1;
+	}
     AttribDeclaration::semantic(sc);
 }
 
