@@ -1348,31 +1348,32 @@ Expression *BinExp::scaleFactor(Scope *sc)
     return this;
 }
 
-/************************************
- * Bring leaves to common type.
- */
+/**************************************
+  * Combine types.
+  * Output:
+  *	*pt	merged type, if *pt is not NULL
+  *	*pe1	rewritten e1
+  *	*pe2	rewritten e2
+  * Returns:
+  *	!=0	success
+  *	0	failed
+  */
 
-Expression *BinExp::typeCombine(Scope *sc)
-{
-    Type *t1;
-    Type *t2;
-    Type *t;
-    TY ty;
+ int typeMerge(Scope *sc, Type **pt, Expression **pe1, Expression **pe2)
+  {
+	 //printf("typeMerge() %s\n", toChars());
+	 //dump(0);
 
-    //printf("BinExp::typeCombine() %s\n", toChars());
-    //dump(0);
+	 Expression *e1 = (*pe1)->integralPromotions(sc);
+	 Expression *e2 = (*pe2)->integralPromotions(sc);
 
-    e1 = e1->integralPromotions(sc);
-    e2 = e2->integralPromotions(sc);
+	 Type *t1 = e1->type;
+	 Type *t2 = e2->type;
+	 assert(t1);
+	 Type *t = t1;
 
-    // BUG: do toBasetype()
-    t1 = e1->type;
-    t2 = e2->type;
-    assert(t1);
-    t = t1;
-
-    //if (t1) printf("\tt1 = %s\n", t1->toChars());
-    //if (t2) printf("\tt2 = %s\n", t2->toChars());
+	 //if (t1) printf("\tt1 = %s\n", t1->toChars());
+	 //if (t2) printf("\tt2 = %s\n", t2->toChars());
 #ifdef DEBUG
     if (!t2) printf("\te2 = '%s'\n", e2->toChars());
 #endif
@@ -1381,7 +1382,7 @@ Expression *BinExp::typeCombine(Scope *sc)
     Type *t1b = t1->toBasetype();
     Type *t2b = t2->toBasetype();
 
-    ty = (TY)Type::impcnvResult[t1b->ty][t2b->ty];
+    TY ty = (TY)Type::impcnvResult[t1b->ty][t2b->ty];
     if (ty != Terror)
     {	TY ty1;
 	TY ty2;
@@ -1393,73 +1394,37 @@ Expression *BinExp::typeCombine(Scope *sc)
 	{
 	    if (t1 == t2)
 	    {
-		if (!type)
-		    type = t1;
-		return this;
+	    	t = t1;
+	    	goto Lret;
 	    }
 
 	    if (t1b == t2b)
 	    {
-		if (!type)
-		    type = t1b;
-		return this;
+	    	t = t1b;
+	    	goto Lret;
 	    }
 	}
 
-	if (!type)
-	    type = Type::basic[ty];
+	t = Type::basic[ty];
 
 	t1 = Type::basic[ty1];
 	t2 = Type::basic[ty2];
 	e1 = e1->castTo(sc, t1);
 	e2 = e2->castTo(sc, t2);
-#if 0
-	if (type != Type::basic[ty])
-	{   t = type;
-	    type = Type::basic[ty];
-	    return castTo(sc, t);
-	}
-#endif
+
 	//printf("after typeCombine():\n");
 	//dump(0);
 	//printf("ty = %d, ty1 = %d, ty2 = %d\n", ty, ty1, ty2);
-	return this;
+	goto Lret;
     }
 
     t1 = t1b;
     t2 = t2b;
 
-    if (op == TOKcat)
-    {
-	if ((t1->ty == Tsarray || t1->ty == Tarray) &&
-	    (t2->ty == Tsarray || t2->ty == Tarray) &&
-	    (t1->nextOf()->mod || t2->nextOf()->mod) &&
-	    (t1->nextOf()->mod != t2->nextOf()->mod)
-	   )
-	{
-	    t1 = t1->nextOf()->mutableOf()->constOf()->arrayOf();
-	    t2 = t2->nextOf()->mutableOf()->constOf()->arrayOf();
-//t1 = t1->constOf();
-//t2 = t2->constOf();
-	    if (e1->op == TOKstring && !((StringExp *)e1)->committed)
-		e1->type = t1;
-	    else
-		e1 = e1->castTo(sc, t1);
-	    if (e2->op == TOKstring && !((StringExp *)e2)->committed)
-		e2->type = t2;
-	    else
-		e2 = e2->castTo(sc, t2);
-	    t = t1;
-	    goto Lagain;
-	}
-    }
 
 Lagain:
     if (t1 == t2)
     {
-	if ((t1->ty == Tstruct || t1->ty == Tclass) &&
-	    (op == TOKmin || op == TOKadd))
-	    goto Lincompatible;
     }
     else if (t1->ty == Tpointer && t2->ty == Tpointer)
     {
@@ -1575,7 +1540,7 @@ Lagain:
 		/* Pick 'tightest' type
 		 */
 		ClassDeclaration *cd1 = tc1->sym->baseClass;
-		ClassDeclaration *cd2 = tc1->sym->baseClass;
+		ClassDeclaration *cd2 = tc2->sym->baseClass;
 
 		if (cd1 && cd2)
 		{   t1 = cd1->type;
@@ -1628,20 +1593,21 @@ Lagain:
     else
     {
      Lincompatible:
-	incompatibleTypes();
+     return 0;
     }
 Lret:
-    if (!type)
-	type = t;
+	if (!*pt)
+		*pt = t;
+	*pe1 = e1;
+	*pe2 = e2;
 #if 0
-    printf("-BinExp::typeCombine() %s\n", toChars());
+	printf("-typeMerge() %s\n", toChars());
     if (e1->type) printf("\tt1 = %s\n", e1->type->toChars());
     if (e2->type) printf("\tt2 = %s\n", e2->type->toChars());
     printf("\ttype = %s\n", type->toChars());
 #endif
     //dump(0);
-    return this;
-
+    return 1;
 
 Lt1:
     e2 = e2->castTo(sc, t1);
@@ -1654,6 +1620,53 @@ Lt2:
     goto Lret;
 }
 
+ /************************************
+ * Bring leaves to common type.
+ */
+
+Expression *BinExp::typeCombine(Scope *sc)
+{
+	Type *t1 = e1->type->toBasetype();
+	Type *t2 = e2->type->toBasetype();
+
+	if (op == TOKcat)
+	{
+		if ((t1->ty == Tsarray || t1->ty == Tarray) &&
+				(t2->ty == Tsarray || t2->ty == Tarray) &&
+				(t1->nextOf()->mod || t2->nextOf()->mod) &&
+				(t1->nextOf()->mod != t2->nextOf()->mod)
+		)
+		{
+			t1 = t1->nextOf()->mutableOf()->constOf()->arrayOf();
+			t2 = t2->nextOf()->mutableOf()->constOf()->arrayOf();
+			//t1 = t1->constOf();
+			//t2 = t2->constOf();
+			if (e1->op == TOKstring && !((StringExp *)e1)->committed)
+				e1->type = t1;
+			else
+				e1 = e1->castTo(sc, t1);
+			if (e2->op == TOKstring && !((StringExp *)e2)->committed)
+				e2->type = t2;
+			else
+				e2 = e2->castTo(sc, t2);
+		}
+	}
+	else if (op == TOKmin || op == TOKadd)
+	{
+		if (t1 == t2 && (t1->ty == Tstruct || t1->ty == Tclass))
+			goto Lerror;
+	}
+
+	if (!typeMerge(sc, &type, &e1, &e2))
+		goto Lerror;
+	return this;
+
+Lerror:
+	incompatibleTypes();
+	type = Type::terror;
+	return this;
+}
+
 /***********************************
  * Do integral promotions (convertchk).
  * Don't convert <array of> to <pointer to>
