@@ -476,7 +476,7 @@ dt_t *ArrayInitializer::toDtBit()
 	    if (dim > tadim)
 	    {
 #ifdef DEBUG
-		printf("2: ");
+		printf("3: ");
 #endif
 		error(loc, "too many initializers, %d, for array[%d]", dim, tadim);
 	    }
@@ -882,7 +882,9 @@ dt_t **SymOffExp::toDt(dt_t **pdt)
 
     //printf("SymOffExp::toDt('%s')\n", var->toChars());
     assert(var);
-    if (!(var->isDataseg() || var->isCodeseg()) || var->needThis())
+    if (!(var->isDataseg() || var->isCodeseg()) ||
+	var->needThis() ||
+	var->storage_class & STCtls)
     {
 #ifdef DEBUG
 	printf("SymOffExp::toDt()\n");
@@ -901,7 +903,8 @@ dt_t **VarExp::toDt(dt_t **pdt)
 	;
 
     VarDeclaration *v = var->isVarDeclaration();
-    if (v && v->isConst() && type->toBasetype()->ty != Tsarray && v->init)
+    if (v && (v->isConst() || v->isInvariant()) &&
+	type->toBasetype()->ty != Tsarray && v->init)
     {
 	*pdt = v->init->toDt();
 	return pdt;
