@@ -11,6 +11,7 @@
    work with the GDC compiler.
 
    Modified by David Friedman, September 2004
+   Modified by Vincenzo Ampolo, September 2009
 */
 
 #include <stdio.h>
@@ -188,24 +189,26 @@ void Module::genmoduleinfo()
 	dti32(&dt, 8|4, true);		// flags (4 means MIstandalone)
 
     if (sctor)
-	dtxoff(&dt, sctor, 0, TYnptr);
+    	dtxoff(&dt, sctor, 0, TYnptr);
     else
-	dtdword(&dt, 0);
+    	dtdword(&dt, 0);
 
     if (sdtor)
-	dtxoff(&dt, sdtor, 0, TYnptr);
+    	dtxoff(&dt, sdtor, 0, TYnptr);
     else
-	dtdword(&dt, 0);
+    	dtdword(&dt, 0);
 
     if (stest)
-	dtxoff(&dt, stest, 0, TYnptr);
+    	dtxoff(&dt, stest, 0, TYnptr);
     else
-	dtdword(&dt, 0);
+    	dtdword(&dt, 0);
 
+#if V2
     FuncDeclaration *sgetmembers = findGetMembers();
     if (sgetmembers)
 	dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
     else
+#endif
 	dtdword(&dt, 0);			// xgetMembers
 
     if (sictor)
@@ -305,10 +308,9 @@ void ClassDeclaration::toObjFile(int multiobj)
 
     assert(!scope);	// semantic() should have been run to completion
 
-    if (parent && parent->isTemplateInstance())
-	scclass = SCcomdat;
-    else
 	scclass = SCglobal;
+	if (inTemplateInstance())
+	  	    scclass = SCcomdat;
 
     // Put out the members
     for (i = 0; i < members->dim; i++)
@@ -479,7 +481,10 @@ void ClassDeclaration::toObjFile(int multiobj)
 	dtdword(&dt, 0);
 
     // flags
-    int flags = 16 | 4 | isCOMclass();
+    int flags = 4 | isCOMclass();
+#if V2
+    flags |= 16;
+#endif
     if (ctor)
 	flags |= 8;
     for (ClassDeclaration *cd = this; cd; cd = cd->baseClass)
@@ -516,11 +521,13 @@ void ClassDeclaration::toObjFile(int multiobj)
     else
 	dtdword(&dt, 0);
 
+#if V2
     FuncDeclaration *sgetmembers = findGetMembers();
     if (sgetmembers)
 	dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
     else
 	dtdword(&dt, 0);	// module getMembers() function
+#endif
 
     //////////////////////////////////////////////
 
@@ -715,17 +722,37 @@ void ClassDeclaration::toObjFile(int multiobj)
 
 	//printf("\tvtbl[%d] = %p\n", i, fd);
 	if (fd && (fd->fbody || !isAbstract()))
-	{
-	    if (isFuncHidden(fd))
-	    {
-		if (global.params.warnings)
-		{   fprintf(stdmsg, "warning - ");
-		    error("%s %s is hidden in %s\n", fd->toParent()->toChars(), fd->toChars(), toChars());
-		}
-		dtxoff(&dt, rtlsym[RTLSYM_DHIDDENFUNC], 0, TYnptr);
-	    }
-	    else
-		dtxoff(&dt, fd->toSymbol(), 0, TYnptr);
+ 	{   Symbol *s = fd->toSymbol();
+
+#if V2
+  	    if (isFuncHidden(fd))
+ 	    {	/* fd is hidden from the view of this class.
+			 * If fd overlaps with any function in the vtbl[], then
+			 * issue 'hidden' error.
+			 */
+			for (int j = 1; j < vtbl.dim; j++)
+			{   if (j == i)
+				continue;
+				FuncDeclaration *fd2 = ((Dsymbol *)vtbl.data[j])->isFuncDeclaration();
+				if (!fd2->ident->equals(fd->ident))
+					continue;
+		 		if (fd->leastAsSpecialized(fd2) || fd2->leastAsSpecialized(fd))
+		  	    {
+		  		if (global.params.warnings)
+		  		{   fprintf(stdmsg, "warning - ");
+		 			    TypeFunction *tf = (TypeFunction *)fd->type;
+		 			    if (tf->ty == Tfunction)
+		 				error("%s%s is hidden by %s\n", fd->toPrettyChars(), Argument::argsTypesToChars(tf->parameters, tf->varargs), toChars());
+		 			    else
+		 				error("%s is hidden by %s\n", fd->toPrettyChars(), toChars());
+		 		}
+		 			s = rtlsym[RTLSYM_DHIDDENFUNC];
+		 			break;
+		  	    }
+				}
+		 	    }
+#endif
+ 	    dtxoff(&dt, s, 0, TYnptr);
 	}
 	else
 	    dtdword(&dt, 0);
@@ -828,10 +855,9 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     if (global.params.symdebug)
 	toDebug();
 
-    if (parent && parent->isTemplateInstance())
-	scclass = SCcomdat;
-    else
-	scclass = SCglobal;
+    scclass = SCglobal;
+    if (inTemplateInstance())
+        scclass = SCcomdat;
 
     // Put out the members
     for (i = 0; i < members->dim; i++)
@@ -867,7 +893,9 @@ void InterfaceDeclaration::toObjFile(int multiobj)
 	    void *deallocator;
 	    OffsetTypeInfo[] offTi;
 	    void *defaultConstructor;
+#if V2
 	    const(MemberInfo[]) function(string) xgetMembers;	// module getMembers() function
+#endif
        }
      */
     dt_t *dt = NULL;
@@ -927,8 +955,10 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     // defaultConstructor
     dtdword(&dt, 0);
 
+#if V2
     // xgetMembers
     dtdword(&dt, 0);
+#endif
 
     //////////////////////////////////////////////
 
@@ -1043,11 +1073,13 @@ void VarDeclaration::toObjFile(int multiobj)
 	return;
     }
 
+#if V2
     // Do not store variables we cannot take the address of
     if (!canTakeAddressOf())
     {
 	return;
     }
+#endif
 
     if (isDataseg() && !(storage_class & STCextern))
     {
@@ -1055,7 +1087,7 @@ void VarDeclaration::toObjFile(int multiobj)
 	sz = type->size();
 
 	parent = this->toParent();
-#if 0	/* private statics should still get a global symbol, in case
+#if V1	/* private statics should still get a global symbol, in case
 	 * another module inlines a function that references it.
 	 */
 	if (/*protection == PROTprivate ||*/
@@ -1079,7 +1111,7 @@ void VarDeclaration::toObjFile(int multiobj)
                  */
                 if (parent->isTemplateInstance() && !parent->isTemplateMixin())
                 {
-#if 0
+#if V1
                     /* These symbol constants have already been copied,
                      * so no reason to output them.
                      * Note that currently there is no way to take
@@ -1185,15 +1217,9 @@ void TypedefDeclaration::toObjFile(int multiobj)
 	;
     else
     {
-	enum_SC scclass = SCglobal;
-	for (Dsymbol *parent = this->parent; parent; parent = parent->parent)
-	{
-	    if (parent->isTemplateInstance())
-	    {
-		scclass = SCcomdat;
-		break;
-	    }
-	}
+    enum_SC scclass = SCglobal;
+   	if (inTemplateInstance())
+   		scclass = SCcomdat;
 
 	// Generate static initializer
 	toInitializer();
@@ -1214,8 +1240,10 @@ void EnumDeclaration::toObjFile(int multiobj)
 {
     //printf("EnumDeclaration::toObjFile('%s')\n", toChars());
 
+#if V2
     if (isAnonymous())
 	return;
+#endif
 
     if (global.params.symdebug)
 	toDebug();
@@ -1227,15 +1255,9 @@ void EnumDeclaration::toObjFile(int multiobj)
 	;
     else
     {
-	enum_SC scclass = SCglobal;
-	for (Dsymbol *parent = this->parent; parent; parent = parent->parent)
-	{
-	    if (parent->isTemplateInstance())
-	    {
-		scclass = SCcomdat;
-		break;
-	    }
-	}
+    enum_SC scclass = SCglobal;
+    if (inTemplateInstance())
+    	scclass = SCcomdat;
 
 	// Generate static initializer
 	toInitializer();
@@ -1244,10 +1266,13 @@ void EnumDeclaration::toObjFile(int multiobj)
 #if ELFOBJ // Burton
 	sinit->Sseg = CDATA;
 #endif /* ELFOBJ */
-	tc->sym->defaultval->toDt(&sinit->Sdt);
-	//sinit->Sdt = tc->sym->init->toDt();
-	outdata(sinit);
+#if V1
+ 	dtnbytes(&sinit->Sdt, tc->size(0), (char *)&tc->sym->defaultval);
+ 	//sinit->Sdt = tc->sym->init->toDt();
+#endif
+#if V2
+  	tc->sym->defaultval->toDt(&sinit->Sdt);
+#endif
+  	outdata(sinit);
     }
 }
-
-
