@@ -451,7 +451,15 @@ x3_gcc_execute(X3_Array extra_args)
 {
     int status;
     const char * result;
-    char * cmd = concat("exec \"$@\"",
+#if _WIN32
+    /* x3 programs get the c:\foo\bar form of path names.  Passing that
+       as a program name to the MSYS shell's 'exec' causes the path
+       to be mangled. */
+    const char * exec_bit = "\"$@\"";
+#else
+    const char * exec_bit = "exec \"$@\"";
+#endif
+    char * cmd = concat(exec_bit,
 	global->log_path ? " >>" : "",
 	global->log_path ? global->log_path : "", // TODO: assumptions about charaters in the path
 	global->log_path ? " 2>&1" : "",
