@@ -8,20 +8,13 @@ else
 fi
 top=`pwd`
 
-# D 2.0 will be the default eventually
-#if test -d gcc/d/dmd2; then
-#    d_lang_version=2
-#else
-#    d_lang_version=1
-#fi
-
-if test -d gcc/d/dmd; then
-    d_lang_version=1
-elif test -d gcc/d/dmd2; then
+# D 2.0 is the defalt
+if test -d gcc/d/dmd2; then
     d_lang_version=2
+else
+    d_lang_version=1
 fi
 
-
 # Read command line arguments
 for arg in "$@"; do
     case "$arg" in
@@ -49,6 +42,8 @@ elif grep version_string gcc/version.c | grep -q '"4.0'; then
     gcc_ver=4.0
 elif grep -q '^4\.1\.' gcc/BASE-VER; then
     gcc_ver=4.1
+elif grep -q '^4\.3\.' gcc/BASE-VER; then
+    gcc_ver=4.3
 fi
 
 gcc_patch_key=${gcc_ver}.x
